@@ -1,5 +1,14 @@
 #include <Elementary.h>
 #ifndef ELM_LIB_QUICKLAUNCH
+
+/* Hint:
+ * In this example some calls to elm_genlist_item_append() are used which give the 
+ * value of an int as 'item data' and 'func data' after casting into (void*). For 
+ * sure this works only on architectures where sizeof(int)==sizeof(void*).
+ * For real world usage you should hold a data structure or value in your 
+ * application and then give only a pointer to this data as data pointer.
+ */
+
 typedef struct _Testitem
 {
    Elm_Genlist_Item *item;
@@ -65,6 +74,24 @@ _bt1500_cb(void *data, Evas_Object *obj, void *event_info)
     elm_genlist_item_middle_bring_in(data);
 }
 
+static void
+_gl_selected(void *data, Evas_Object *obj, void *event_info)
+{
+   printf("selected: %p\n", event_info);
+}
+
+static void
+_gl_clicked(void *data, Evas_Object *obj, void *event_info)
+{
+   printf("clicked: %p\n", event_info);
+}
+
+static void
+_gl_longpress(void *data, Evas_Object *obj, void *event_info)
+{
+   printf("longpress %p\n", event_info);
+}
+
 void
 test_genlist(void *data, Evas_Object *obj, void *event_info)
 {
@@ -88,6 +115,9 @@ test_genlist(void *data, Evas_Object *obj, void *event_info)
    evas_object_show(bx);
 
    gl = elm_genlist_add(win);
+   evas_object_smart_callback_add(gl, "selected", _gl_selected, NULL);
+   evas_object_smart_callback_add(gl, "clicked", _gl_clicked, NULL);
+   evas_object_smart_callback_add(gl, "longpressed", _gl_longpress, NULL);
    // FIXME: This causes genlist to resize the horiz axis very slowly :(
    // Reenable this and resize the window horizontally, then try to resize it back
    //elm_genlist_horizontal_mode_set(gl, ELM_LIST_LIMIT);
@@ -122,17 +152,16 @@ test_genlist(void *data, Evas_Object *obj, void *event_info)
 
    for (i = 0; i < 2000; i++)
      {
-         gli = elm_genlist_item_append(gl, &itc1,
-                 (void *)i/* item data */,
-                 NULL/* parent */,
-                 ELM_GENLIST_ITEM_NONE,
-                 gl_sel/* func */,
-                 (void *)(i * 10)/* func data */);
-
-         if(i==50)
-             evas_object_smart_callback_add(bt_50, "clicked", _bt50_cb, gli);
-         else if(i==1500)
-             evas_object_smart_callback_add(bt_1500, "clicked", _bt1500_cb, gli);
+        gli = elm_genlist_item_append(gl, &itc1,
+                                      (void *)i/* item data */,
+                                      NULL/* parent */,
+                                      ELM_GENLIST_ITEM_NONE,
+                                      gl_sel/* func */,
+                                      (void *)(i * 10)/* func data */);
+        if (i == 50)
+          evas_object_smart_callback_add(bt_50, "clicked", _bt50_cb, gli);
+        else if (i == 1500)
+          evas_object_smart_callback_add(bt_1500, "clicked", _bt1500_cb, gli);
      }
    evas_object_resize(win, 480, 800);
    evas_object_show(win);
@@ -292,13 +321,25 @@ my_gl_last(void *data, Evas_Object *obj, void *event_info)
    elm_genlist_item_selected_set(gli, 1);
 }
 
+static Eina_Bool
+my_gl_flush_delay(void *data)
+{
+   elm_all_flush();
+   return ECORE_CALLBACK_CANCEL;
+}
+
+static void
+my_gl_flush(void *data, Evas_Object *obj, void *event_info)
+{
+   ecore_timer_add(1.2, my_gl_flush_delay, NULL);
+}
+
 void
 test_genlist2(void *data, Evas_Object *obj, void *event_info)
 {
    Evas_Object *win, *bg, *gl, *bx, *bx2, *bx3, *bt;
    Elm_Genlist_Item *gli[10];
    char buf[PATH_MAX];
-   int i;
 
    win = elm_win_add(NULL, "genlist-2", ELM_WIN_BASIC);
    elm_win_title_set(win, "Genlist 2");
@@ -447,6 +488,14 @@ test_genlist2(void *data, Evas_Object *obj, void *event_info)
    elm_box_pack_end(bx3, bt);
    evas_object_show(bt);
 
+   bt = elm_button_add(win);
+   elm_button_label_set(bt, "Flush");
+   evas_object_smart_callback_add(bt, "clicked", my_gl_flush, gl);
+   evas_object_size_hint_align_set(bt, EVAS_HINT_FILL, EVAS_HINT_FILL);
+   evas_object_size_hint_weight_set(bt, EVAS_HINT_EXPAND, 0.0);
+   elm_box_pack_end(bx3, bt);
+   evas_object_show(bt);
+
    elm_box_pack_end(bx, bx3);
    evas_object_show(bx3);
 
@@ -460,14 +509,14 @@ test_genlist2(void *data, Evas_Object *obj, void *event_info)
 static Elm_Genlist_Item_Class itc2;
 char *gl2_label_get(const void *data, Evas_Object *obj, const char *part)
 {
-   Testitem *tit = data;
+   const Testitem *tit = data;
    char buf[256];
    snprintf(buf, sizeof(buf), "Item mode %i", tit->mode);
    return strdup(buf);
 }
 Evas_Object *gl2_icon_get(const void *data, Evas_Object *obj, const char *part)
 {
-   Testitem *tit = data;
+   const Testitem *tit = data;
    char buf[PATH_MAX];
    Evas_Object *ic = elm_icon_add(obj);
    if (!strcmp(part, "elm.swallow.icon"))
@@ -518,7 +567,6 @@ test_genlist3(void *data, Evas_Object *obj, void *event_info)
 {
    Evas_Object *win, *bg, *gl, *bx, *bx2, *bt;
    static Testitem tit[3];
-   int i;
 
    win = elm_win_add(NULL, "genlist-3", ELM_WIN_BASIC);
    elm_win_title_set(win, "Genlist 3");
@@ -559,7 +607,7 @@ test_genlist3(void *data, Evas_Object *obj, void *event_info)
 					 NULL/* func data */);
 
    elm_box_pack_end(bx, gl);
-   evas_object_show(bx2);
+   evas_object_show(bx);
 
    bx2 = elm_box_add(win);
    elm_box_horizontal_set(bx2, 1);
@@ -611,17 +659,26 @@ my_gl_item_check_changed(void *data, Evas_Object *obj, void *event_info)
 static Elm_Genlist_Item_Class itc3;
 char *gl3_label_get(const void *data, Evas_Object *obj, const char *part)
 {
-   Testitem *tit = data;
+   const Testitem *tit = data;
    char buf[256];
    snprintf(buf, sizeof(buf), "Item mode %i", tit->mode);
    return strdup(buf);
 }
 Evas_Object *gl3_icon_get(const void *data, Evas_Object *obj, const char *part)
 {
-   Testitem *tit = data;
+   const Testitem *tit = data;
    char buf[PATH_MAX];
    if (!strcmp(part, "elm.swallow.icon"))
      {
+        Evas_Object *lb;
+        
+        lb = elm_label_add(obj);
+        elm_label_line_wrap_set(lb, 1);
+        elm_label_wrap_width_set(lb, 201);
+        elm_label_label_set(lb, "ashj ascjscjh n asjkl hcjlh ls hzshnn zjh sh zxjcjsnd h dfw sdv edev efe fwefvv vsd cvs ws wf  fvwf wd fwe f  we wef we wfe rfwewef wfv wswf wefg sdfws w wsdcfwcf wsc vdv  sdsd sdcd cv wsc sdcv wsc d sdcdcsd sdcdsc wdvd sdcsd wscxcv wssvd sd");
+        evas_object_show(lb);
+        return lb;
+        
 	Evas_Object *bx = elm_box_add(obj);
 	Evas_Object *ic;
 	ic = elm_icon_add(obj);
@@ -664,7 +721,6 @@ test_genlist4(void *data, Evas_Object *obj, void *event_info)
 {
    Evas_Object *win, *bg, *gl, *bx, *bx2, *bt;
    static Testitem tit[3];
-   int i;
 
    win = elm_win_add(NULL, "genlist-4", ELM_WIN_BASIC);
    elm_win_title_set(win, "Genlist 4");
@@ -706,7 +762,7 @@ test_genlist4(void *data, Evas_Object *obj, void *event_info)
 					 NULL/* func data */);
 
    elm_box_pack_end(bx, gl);
-   evas_object_show(bx2);
+   evas_object_show(bx);
 
    bx2 = elm_box_add(win);
    elm_box_horizontal_set(bx2, 1);
@@ -758,7 +814,7 @@ my_gl_item_check_changed2(void *data, Evas_Object *obj, void *event_info)
 static Elm_Genlist_Item_Class itc5;
 char *gl5_label_get(const void *data, Evas_Object *obj, const char *part)
 {
-   Testitem *tit = data;
+   const Testitem *tit = data;
    char buf[256];
    if (!strcmp(part, "elm.text"))
      {
@@ -772,7 +828,7 @@ char *gl5_label_get(const void *data, Evas_Object *obj, const char *part)
 }
 Evas_Object *gl5_icon_get(const void *data, Evas_Object *obj, const char *part)
 {
-   Testitem *tit = data;
+   const Testitem *tit = data;
    char buf[PATH_MAX];
    if (!strcmp(part, "elm.swallow.icon"))
      {
@@ -850,12 +906,17 @@ item_drag_stop(void *data, Evas_Object *obj, void *event_info)
    printf("drag stop\n");
 }
 
+static void
+item_longpress(void *data, Evas_Object *obj, void *event_info)
+{
+   printf("longpress\n");
+}
+
 void
 test_genlist5(void *data, Evas_Object *obj, void *event_info)
 {
    Evas_Object *win, *bg, *gl, *bx, *bx2, *bt;
    static Testitem tit[3];
-   int i;
 
    win = elm_win_add(NULL, "genlist-5", ELM_WIN_BASIC);
    elm_win_title_set(win, "Genlist 5");
@@ -896,7 +957,7 @@ test_genlist5(void *data, Evas_Object *obj, void *event_info)
 					 NULL/* func data */);
 
    elm_box_pack_end(bx, gl);
-   evas_object_show(bx2);
+   evas_object_show(bx);
 
    evas_object_smart_callback_add(gl, "drag,start,up", item_drag_up, NULL);
    evas_object_smart_callback_add(gl, "drag,start,down", item_drag_down, NULL);
@@ -904,6 +965,7 @@ test_genlist5(void *data, Evas_Object *obj, void *event_info)
    evas_object_smart_callback_add(gl, "drag,start,right", item_drag_right, NULL);
    evas_object_smart_callback_add(gl, "drag", item_drag, NULL);
    evas_object_smart_callback_add(gl, "drag,stop", item_drag_stop, NULL);
+   evas_object_smart_callback_add(gl, "longpressed", item_longpress, NULL);
 
    bx2 = elm_box_add(win);
    elm_box_horizontal_set(bx2, 1);
@@ -1068,7 +1130,7 @@ test_genlist6(void *data, Evas_Object *obj, void *event_info)
    evas_object_smart_callback_add(gl, "contracted", gl4_con, gl);
 
    elm_box_pack_end(bx, gl);
-   evas_object_show(bx2);
+   evas_object_show(bx);
 
    bx2 = elm_box_add(win);
    elm_box_horizontal_set(bx2, 1);
