@@ -1,15 +1,27 @@
-/*
+/**
+ * @defgroup Fileselector Fileselector
+ *
+ * A fileselector is a widget that allows a user to navigate through a tree
+ * of files.  It contains buttons for Home(~) and Up(..) as well as cancel/ok
+ * buttons to confirm/cancel a selection.  This widget is currently very much
+ * in progress.
+ *
  * TODO
- *  userdefined icon/label cb
- *  show/hide/add buttons ???
- *  need a background ???
- *  show/Hide hidden files
- *  double click to choose a file
- *  multiselection
- *  write docs
+ * userdefined icon/label cb
+ * show/hide/add buttons ???
+ * need a background ???
+ * show/Hide hidden files
+ * double click to choose a file
+ * multiselection
+ * make variable/function names that are sensible
+ *
+ * Signals that you can add callbacks for are:
+ *
+ * "selected" - the user clicks on a file
+ * "directory,open" - the list is populated with a new content. event_info is a directory.
+ * "done" - the user clicks on the ok or cancel button
  */
 
-#include <Ecore_Str.h>
 #include <Elementary.h>
 #include "elm_priv.h"
 
@@ -22,8 +34,9 @@ struct _Widget_Data
    const char *selection;
    Eina_Bool only_folder;
    Eina_Bool expand;
+   Ecore_Idler *sel_idler;
 
-   struct 
+   struct
    {
     Evas_Object *bx;
     Evas_Object *ok;
@@ -31,8 +44,26 @@ struct _Widget_Data
    } buttons;
 };
 
+struct sel_data
+{
+   Evas_Object *fs;
+   const char *path;
+};
+
 Elm_Genlist_Item_Class itc;
 
+static const char *widtype = NULL;
+
+static const char SIG_DIRECTORY_OPEN[]= "directory,open";
+static const char SIG_DONE[] = "done";
+static const char SIG_SELECTED[] = "selected";
+static const Evas_Smart_Cb_Description _signals[] = {
+  {SIG_DIRECTORY_OPEN, "s"},
+  {SIG_DONE, "s"},
+  {SIG_SELECTED, "s"},
+  {NULL, NULL}
+};
+
 static void _populate(Evas_Object *obj, const char *path, Elm_Genlist_Item *parent);
 static void _do_anchors(Evas_Object *obj, const char *path);
 
@@ -40,9 +71,19 @@ static void _do_anchors(Evas_Object *obj, const char *path);
 static void
 _del_hook(Evas_Object *obj)
 {
-   Widget_Data *wd = elm_widget_data_get(obj);
+   Widget_Data *wd;
+   void *sd;
+
+   wd = elm_widget_data_get(obj);
+   if (!wd) return;
+
    if (wd->path) eina_stringshare_del(wd->path);
    if (wd->selection) eina_stringshare_del(wd->selection);
+   if (wd->sel_idler)
+     {
+	sd = ecore_idler_del(wd->sel_idler);
+	free(sd);
+     }
    free(wd);
 }
 
@@ -51,16 +92,15 @@ _sizing_eval(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    Evas_Coord minw = -1, minh = -1;
-
+   if (!wd) return;
    evas_object_size_hint_min_get(wd->vbox, &minw, &minh);
    evas_object_size_hint_min_set(obj, minw, minh);
-
 //   printf("***** SIZING EVAL [min %d %d] *************\n", minw, minh);
 }
 
 /***  GENLIST "MODEL"  ***/
 static char*
-_itc_label_get(const void *data, Evas_Object *obj, const char *source)
+_itc_label_get(const void *data, Evas_Object *obj __UNUSED__, const char *source __UNUSED__)
 {
    //~ printf("LABEL_GET: %s\n", (char*) data);
    return strdup(ecore_file_file_get(data)); // NOTE this will be free() by the caller
@@ -72,7 +112,6 @@ _itc_icon_get(const void *data, Evas_Object *obj, const char *source)
    Evas_Object *ic;
 
    //~ printf("ICON GET for %s (source: %s)\n", (char*)data, source);
-
    if (!strcmp(source, "elm.swallow.icon"))
      {
 	ic = elm_icon_add(obj);
@@ -84,141 +123,167 @@ _itc_icon_get(const void *data, Evas_Object *obj, const char *source)
 	evas_object_show(ic);
 	return ic;
      }
-
    return NULL;
 }
 
 static Eina_Bool
-_itc_state_get(const void *data, Evas_Object *obj, const char *source)
+_itc_state_get(const void *data __UNUSED__, Evas_Object *obj __UNUSED__, const char *source __UNUSED__)
 {
    return EINA_FALSE;
 }
 
 static void
-_itc_del(const void *data, Evas_Object *obj)
+_itc_del(const void *data, Evas_Object *obj __UNUSED__)
 {
    //~ printf("DEL DATA [%s]\n", (char*)data);
    eina_stringshare_del(data);
 }
 
 static void
-_expand_done(void *data, Evas_Object *obj, void *event_info)
+_expand_done(void *data, Evas_Object *obj __UNUSED__, void *event_info)
 {
    Elm_Genlist_Item *it = event_info;
    const char *path = elm_genlist_item_data_get(it);
-
 //   printf("EXPAND %s\n", path);
    _populate(data, path, it);
 }
 
 static void
-_contract_done(void *data, Evas_Object *obj, void *event_info)
+_contract_done(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
 {
    Elm_Genlist_Item *it = event_info;
-   const char *path = elm_genlist_item_data_get(it);
-
+//   const char *path = elm_genlist_item_data_get(it);
 //   printf("CONTRACT %s\n", path);
    elm_genlist_item_subitems_clear(it);
 }
 
 static void
-_expand_req(void *data, Evas_Object *obj, void *event_info)
+_expand_req(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
 {
    Elm_Genlist_Item *it = event_info;
    elm_genlist_item_expanded_set(it, 1);
 }
 
 static void
-_contract_req(void *data, Evas_Object *obj, void *event_info)
+_contract_req(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
 {
    Elm_Genlist_Item *it = event_info;
    elm_genlist_item_expanded_set(it, 0);
 }
 
 /***  PRIVATES  ***/
-static void
-_sel(void *data, Evas_Object *obj, void *event_info)
+static Eina_Bool
+_sel_do(void *data)
 {
-   Evas_Object *fs = data;
-   Widget_Data *wd = elm_widget_data_get(fs);
-   const char *path = elm_genlist_item_data_get(event_info);
+   struct sel_data *sd;
+   const char *path;
+   Widget_Data *wd;
    const char *p;
 
+   sd = data;
+   wd = elm_widget_data_get(sd->fs);
+   path = sd->path;
+
    if (ecore_file_is_dir(path))
      {
-//	printf("SELECTED DIR: %s\n", path);
-	if (wd->expand)
-	  {
-	     _do_anchors(fs, path);
-	     if (wd->entry2) elm_entry_entry_set(wd->entry2, "");
-	  }
-	else
-	  {
-	     // keep a ref to path 'couse it will be destroyed by _populate
-	     p = eina_stringshare_add(path);
-	     _populate(fs, p, NULL);
-	     eina_stringshare_del(p);
-	  }
-	return;
+	//	printf("SELECTED DIR: %s\n", path);
+        if (wd->expand)
+          {
+             _do_anchors(sd->fs, path);
+             if (wd->entry2) elm_entry_entry_set(wd->entry2, "");
+          }
+        else
+          {
+             // keep a ref to path 'couse it will be destroyed by _populate
+             p = eina_stringshare_add(path);
+             _populate(sd->fs, p, NULL);
+             eina_stringshare_del(p);
+          }
+	goto end;
      }
    else
      {
-//	printf("SELECTED FILE: %s\n", path);
+	//	printf("SELECTED FILE: %s\n", path);
 	if (wd->entry2)
 	  elm_entry_entry_set(wd->entry2, ecore_file_file_get(path));
      }
 
-   evas_object_smart_callback_call(fs, "selected", (void*)path);
+   evas_object_smart_callback_call(sd->fs, SIG_SELECTED, (void*)path);
+
+ end:
+   wd->sel_idler = NULL;
+   free(sd);
+   return ECORE_CALLBACK_CANCEL;
 }
 
 static void
-_up(void *data, Evas_Object *obj, void *event_info)
+_sel(void *data, Evas_Object *obj __UNUSED__, void *event_info)
+{
+   struct sel_data *sd;
+   Widget_Data *wd;
+   void *old_sd;
+
+   wd = elm_widget_data_get(data);
+   if (!wd) return;
+
+   sd = malloc(sizeof(*sd));
+   sd->fs = data;
+   sd->path = elm_genlist_item_data_get(event_info);
+
+   if (wd->sel_idler)
+     {
+	old_sd = ecore_idler_del(wd->sel_idler);
+	free(old_sd);
+     }
+   wd->sel_idler = ecore_idler_add(_sel_do, sd);
+}
+
+static void
+_up(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Evas_Object *fs = data;
    Widget_Data *wd = elm_widget_data_get(fs);
+   if (!wd) return;
    char *parent = ecore_file_dir_get(wd->path);
-
    _populate(fs, parent, NULL);
    free(parent);
 }
 
 static void
-_home(void *data, Evas_Object *obj, void *event_info)
+_home(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Evas_Object *fs = data;
    _populate(fs, getenv("HOME"), NULL);
 }
 
 static void
-_ok(void *data, Evas_Object *obj, void *event_info)
+_ok(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Evas_Object *fs = data;
-   evas_object_smart_callback_call(fs, "done",
+   evas_object_smart_callback_call(fs, SIG_DONE,
 				   (void*)elm_fileselector_selected_get(fs));
 }
 
 static void
-_canc(void *data, Evas_Object *obj, void *event_info)
+_canc(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Evas_Object *fs = data;
-   evas_object_smart_callback_call(fs, "done", NULL);
+   evas_object_smart_callback_call(fs, SIG_DONE, NULL);
 }
 
 static void
-_anchor_clicked(void *data, Evas_Object *obj, void *event_info)
+_anchor_clicked(void *data, Evas_Object *obj __UNUSED__, void *event_info)
 {
    Evas_Object *fs = data;
    Widget_Data *wd = elm_widget_data_get(fs);
    Elm_Entry_Anchor_Info *info = event_info;
    const char *p;
-
+   if (!wd) return;
    //~ printf("ANCHOR CLICKED %s\n", info->name);
-
    // keep a ref to path 'couse it will be destroyed by _populate
    p = eina_stringshare_add(info->name);
    _populate(fs, p, NULL);
-
-   evas_object_smart_callback_call(data, "selected", (void*)p);
+   evas_object_smart_callback_call(data, SIG_SELECTED, (void*)p);
    eina_stringshare_del(p);
 }
 
@@ -228,26 +293,25 @@ _do_anchors(Evas_Object *obj, const char *path)
    Widget_Data *wd = elm_widget_data_get(obj);
    char **tok, buf[PATH_MAX*3];
    int i, j;
-
+   if (!wd) return;
    //~ printf("ANCHORIZE...\n");
-
    buf[0] = '\0';
-   tok = ecore_str_split(path, "/", 0);
+   tok = eina_str_split(path, "/", 0);
    for (i = 0; tok[i]; i++)
      {
 	if (strlen(tok[i]) < 1) continue;
 	//~ printf("TOK: %s\n", tok[i]);
-	ecore_strlcat(buf, "/<a href=", sizeof(buf));
+	eina_strlcat(buf, "/<a href=", sizeof(buf));
 	for (j = 0; j <= i; j++)
 	  {
 	     if (strlen(tok[j]) < 1) continue;
 	     //~ printf("REV: %s\n",tok[j]);
-	     ecore_strlcat(buf, "/", sizeof(buf));
-	     ecore_strlcat(buf, tok[j], sizeof(buf));
+	     eina_strlcat(buf, "/", sizeof(buf));
+	     eina_strlcat(buf, tok[j], sizeof(buf));
 	  }
-	ecore_strlcat(buf, ">", sizeof(buf));
-	ecore_strlcat(buf, tok[i], sizeof(buf));
-	ecore_strlcat(buf, "</a>", sizeof(buf));
+	eina_strlcat(buf, ">", sizeof(buf));
+	eina_strlcat(buf, tok[i], sizeof(buf));
+	eina_strlcat(buf, "</a>", sizeof(buf));
      }
    free(tok[0]);
    free(tok);
@@ -260,7 +324,6 @@ static void
 _populate(Evas_Object *obj, const char *path, Elm_Genlist_Item *parent)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
-
    DIR *dir;
    struct dirent *dp;
    char buf[PATH_MAX];
@@ -268,23 +331,17 @@ _populate(Evas_Object *obj, const char *path, Elm_Genlist_Item *parent)
    Eina_List *files = NULL, *dirs = NULL, *l;
 
    if ((!wd) || (!ecore_file_is_dir(path))) return;
-
    dir = opendir(path);
    if (!dir) return;
-
-   evas_object_smart_callback_call(obj, "directory,open", (void*)path);
-
+   evas_object_smart_callback_call(obj, SIG_DIRECTORY_OPEN, (void*)path);
    if (!parent)
      {
 	elm_genlist_clear(wd->list);
-	if (wd->path) eina_stringshare_del(wd->path);
-	wd->path = eina_stringshare_add(path);
+	eina_stringshare_replace(&wd->path, path);
 	_do_anchors(obj, path);
      }
 
-   if (wd->entry2)
-     elm_entry_entry_set(wd->entry2, "");
-
+   if (wd->entry2) elm_entry_entry_set(wd->entry2, "");
    while ((dp = readdir(dir)) != NULL)
      {
 	if (dp->d_name[0] == '.') continue; // TODO make this configurable
@@ -298,9 +355,8 @@ _populate(Evas_Object *obj, const char *path, Elm_Genlist_Item *parent)
      }
    closedir(dir);
 
-   files = eina_list_sort(files, ECORE_SORT_MIN, ECORE_COMPARE_CB(strcoll));
-   dirs = eina_list_sort(dirs, ECORE_SORT_MIN, ECORE_COMPARE_CB(strcoll));
-
+   files = eina_list_sort(files, eina_list_count(files), EINA_COMPARE_CB(strcoll));
+   dirs = eina_list_sort(dirs, eina_list_count(dirs), EINA_COMPARE_CB(strcoll));
    EINA_LIST_FOREACH(dirs, l, real)
      {
 	//~ printf("DIR: %s\n", real);
@@ -308,7 +364,7 @@ _populate(Evas_Object *obj, const char *path, Elm_Genlist_Item *parent)
 				eina_stringshare_add(real), /* item data */
 				parent,
 				wd->expand ? ELM_GENLIST_ITEM_SUBITEMS :
-					     ELM_GENLIST_ITEM_NONE,
+                                ELM_GENLIST_ITEM_NONE,
 				NULL, NULL);
 	free(real);
      }
@@ -327,6 +383,14 @@ _populate(Evas_Object *obj, const char *path, Elm_Genlist_Item *parent)
 }
 
 /***  API  ***/
+/**
+ * Add a new Fileselector object
+ *
+ * @param parent The parent object
+ * @return The new object or NULL if it cannot be created
+ *
+ * @ingroup Fileselector
+ */
 EAPI Evas_Object *
 elm_fileselector_add(Evas_Object *parent)
 {
@@ -337,6 +401,7 @@ elm_fileselector_add(Evas_Object *parent)
    wd = ELM_NEW(Widget_Data);
    wd->expand = EINA_FALSE;
    obj = elm_widget_add(evas_object_evas_get(parent));
+   ELM_SET_WIDTYPE(widtype, "fileselector");
    elm_widget_type_set(obj, "fileselector");
    elm_widget_sub_object_add(parent, obj);
    elm_widget_data_set(obj, wd);
@@ -426,21 +491,34 @@ elm_fileselector_add(Evas_Object *parent)
    evas_object_show(wd->scr2);
 
    elm_fileselector_buttons_ok_cancel_set(obj, 1);
-   
+
    // Is this the right way to show sub-objs ?? or use the show/hide cbs ??
    //~ evas_object_event_callback_add(obj, EVAS_CALLBACK_SHOW, _show, obj);
    //~ evas_object_event_callback_add(obj, EVAS_CALLBACK_CHANGED_SIZE_HINTS,
-				       //~ _changed_size_hints, obj);
-
+   //~ _changed_size_hints, obj);
    _sizing_eval(obj);
+
+   // TODO: convert Elementary to subclassing of Evas_Smart_Class
+   // TODO: and save some bytes, making descriptions per-class and not instance!
+   evas_object_smart_callbacks_descriptions_set(obj, _signals);
    return obj;
 }
 
+/**
+ * This enables/disables the file name entry box where the user can
+ * type in a name for the file to be saved as.
+ *
+ * @param obj The fileselector object
+ * @param is_save If true, the fileselector is a save dialog
+ *
+ * @ingroup Fileselector
+ */
 EAPI void
 elm_fileselector_is_save_set(Evas_Object *obj, Eina_Bool is_save)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-
+   if (!wd) return;
    if (is_save)
      {
 	if (wd->entry2) return;
@@ -460,107 +538,224 @@ elm_fileselector_is_save_set(Evas_Object *obj, Eina_Bool is_save)
      }
 }
 
+/**
+ * This returns whether the fileselector is a "save" type fileselector
+ *
+ * @param obj The fileselector object
+ * @return If true, the fileselector is a save type.
+ *
+ * @ingroup Fileselector
+ */
 EAPI Eina_Bool
-elm_fileselector_is_save_get(Evas_Object *obj)
+elm_fileselector_is_save_get(const Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) EINA_FALSE;
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return EINA_FALSE;
    return wd->entry2 ? EINA_TRUE : EINA_FALSE;
 }
 
+
+/**
+ * This enables/disables folder-only view in the fileselector.
+ *
+ * @param obj The fileselector object
+ * @param only If true, the fileselector will only display directories.
+ * If false, files are displayed also.
+ *
+ * @ingroup Fileselector
+ */
 EAPI void
 elm_fileselector_folder_only_set(Evas_Object *obj, Eina_Bool only)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-
-   if(wd->only_folder == only) return ;
-
+   if (!wd) return;
+   if (wd->only_folder == only) return;
    wd->only_folder = only;
 }
 
+
+/**
+ * This gets the state of file display in the fileselector.
+ *
+ * @param obj The fileselector object
+ * @return If true, files are not being shown in the fileselector.
+ * If false, files are being shown.
+ *
+ * @ingroup Fileselector
+ */
 EAPI Eina_Bool
-elm_fileselector_only_folder_get(Evas_Object *obj)
+elm_fileselector_folder_only_get(const Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) EINA_FALSE;
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return EINA_FALSE;
    return wd->only_folder;
 }
 
+
+/**
+ * This enables/disables the file name entry box where the user can
+ * type in the name of a file to be selected.
+ *
+ * @param obj The fileselector object
+ * @param only If true, a box containing ok and cancel buttons is created.
+ * If false, the box and the buttons are destroyed.
+ *
+ * @ingroup Fileselector
+ */
 EAPI void
 elm_fileselector_buttons_ok_cancel_set(Evas_Object *obj, Eina_Bool only)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
    Evas_Object *box, *bt;
-
+   if (!wd) return;
    if (only)
      {
 	if (wd->buttons.bx) return;
-   // buttons box
-   box = elm_box_add(obj);
-   wd->buttons.bx = box;
-   elm_box_horizontal_set(box, 1);
-   elm_widget_sub_object_add(obj, box);
-   elm_box_pack_end(wd->vbox, box);
-   evas_object_show(box);
+        // buttons box
+        box = elm_box_add(obj);
+        wd->buttons.bx = box;
+        elm_box_horizontal_set(box, 1);
+        elm_widget_sub_object_add(obj, box);
+        elm_box_pack_end(wd->vbox, box);
+        evas_object_show(box);
 
-   // cancel btn
-   bt = elm_button_add(obj);
-   wd->buttons.cancel = bt;
-   elm_button_label_set(bt, "Cancel");
-   elm_widget_sub_object_add(obj, bt);
-   elm_box_pack_end(box, bt);
-   evas_object_smart_callback_add(bt, "clicked", _canc, obj);
-   evas_object_show(bt);
+        // cancel btn
+        bt = elm_button_add(obj);
+        wd->buttons.cancel = bt;
+        elm_button_label_set(bt, "Cancel");
+        elm_widget_sub_object_add(obj, bt);
+        elm_box_pack_end(box, bt);
+        evas_object_smart_callback_add(bt, "clicked", _canc, obj);
+        evas_object_show(bt);
 
-   // ok btn
-   bt = elm_button_add(obj);
-   wd->buttons.ok = bt;
-   elm_button_label_set(bt, "OK");
-   elm_widget_sub_object_add(obj, bt);
-   elm_box_pack_end(box, bt);
-   evas_object_smart_callback_add(bt, "clicked", _ok, obj);
-   evas_object_show(bt);
+        // ok btn
+        bt = elm_button_add(obj);
+        wd->buttons.ok = bt;
+        elm_button_label_set(bt, "OK");
+        elm_widget_sub_object_add(obj, bt);
+        elm_box_pack_end(box, bt);
+        evas_object_smart_callback_add(bt, "clicked", _ok, obj);
+        evas_object_show(bt);
      }
    else
      {
-   evas_object_del(wd->buttons.bx);
-   evas_object_del(wd->buttons.ok);
-   evas_object_del(wd->buttons.cancel);
+        evas_object_del(wd->buttons.bx);
+        evas_object_del(wd->buttons.ok);
+        evas_object_del(wd->buttons.cancel);
 	wd->buttons.bx = NULL;
      }
 }
 
+
+/**
+ * This gets the state of the box containing ok and cancel buttons.
+ *
+ * @param obj The fileselector object
+ * @return If true, the box exists.
+ * If false, the box does not exist.
+ *
+ * @ingroup Fileselector
+ */
 EAPI Eina_Bool
-elm_fileselector_ok_cancel_get(Evas_Object *obj)
+elm_fileselector_buttons_ok_cancel_get(const Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) EINA_FALSE;
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return EINA_FALSE;
    return wd->buttons.bx ? EINA_TRUE : EINA_FALSE;
 }
 
+
+/**
+ * This enables tree view in the fileselector.  Arrows are created on the
+ * sides of directories, allowing them to expand in place.
+ *
+ * @param obj The fileselector object
+ * @param expand If true, tree view is enabled.
+ * If false, tree view is disabled.
+ *
+ * @ingroup Fileselector
+ */
 EAPI void
 elm_fileselector_expandable_set(Evas_Object *obj, Eina_Bool expand)
 {
-   Widget_Data *wd = elm_widget_data_get(obj);
+   ELM_CHECK_WIDTYPE(obj, widtype);
+   Widget_Data *wd;
+
+   wd = elm_widget_data_get(obj);
+   if (!wd) return;
+
    wd->expand = expand;
 }
 
+/**
+ * This gets the state of tree view in the fileselector.
+ *
+ * @param obj The fileselector object
+ * @return If true, tree view is enabled and folders will be expandable.
+ * If false, tree view is disabled.
+ *
+ * @ingroup Fileselector
+ */
+EAPI Eina_Bool
+elm_fileselector_expandable_get(const Evas_Object *obj)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype) EINA_FALSE;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return EINA_FALSE;
+   return wd->expand;
+}
+
+/**
+ * This sets the path that the fileselector will display.
+ *
+ * @param obj The fileselector object
+ * @param path The path of the fileselector
+ *
+ * @ingroup Fileselector
+ */
 EAPI void
 elm_fileselector_path_set(Evas_Object *obj, const char *path)
 {
    _populate(obj, path, NULL);
 }
 
+/**
+ * This gets the path that the fileselector displays.
+ *
+ * @param obj The fileselector object
+ * @return The path that the fileselector is displaying
+ *
+ * @ingroup Fileselector
+ */
 EAPI const char *
-elm_fileselector_path_get(Evas_Object *obj)
+elm_fileselector_path_get(const Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return NULL;
    return wd->path;
 }
 
-EAPI const char*
-elm_fileselector_selected_get(Evas_Object *obj)
+/**
+ * This gets the currently selected path in the file selector.
+ *
+ * @param obj The file selector object
+ * @return The absolute path of the selected object in the fileselector
+ *
+ * @ingroup Fileselector
+ */
+EAPI const char *
+elm_fileselector_selected_get(const Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
    Widget_Data *wd = elm_widget_data_get(obj);
    Elm_Genlist_Item *it;
-
+   if (!wd) return NULL;
    if (wd->entry2)
      {
 	const char *name;
@@ -569,8 +764,7 @@ elm_fileselector_selected_get(Evas_Object *obj)
 	name = elm_entry_entry_get(wd->entry2);
 	//TODO remove <br>
 	snprintf(buf, sizeof(buf), "%s/%s", wd->path, name);
-	if (wd->selection) eina_stringshare_del(wd->selection);
-	wd->selection = eina_stringshare_add(buf);
+	eina_stringshare_replace(&wd->selection, buf);
 	return wd->selection;
      }
 
@@ -579,3 +773,37 @@ elm_fileselector_selected_get(Evas_Object *obj)
 
    return wd->path;
 }
+
+/**
+ * This sets the currently selected path in the file selector.
+ *
+ * @param obj The file selector object
+ * @param path The path to a file or directory
+ * @return @c EINA_TRUE on success, @c EINA_FALSE on failure. The
+ * latter case occurs if the directory or file pointed to do not
+ * exist.
+ *
+ * @ingroup Fileselector
+ */
+EAPI Eina_Bool
+elm_fileselector_selected_set(Evas_Object *obj, const char *path)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype) EINA_FALSE;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return EINA_FALSE;
+
+   if (ecore_file_is_dir(path)) _populate(obj, path, NULL);
+   else
+     {
+	if (!ecore_file_exists(path)) return EINA_FALSE;
+
+	_populate(obj, ecore_file_dir_get(path), NULL);
+	if (wd->entry2)
+	  {
+	     elm_entry_entry_set(wd->entry2, ecore_file_file_get(path));
+	     eina_stringshare_replace(&wd->selection, path);
+	  }
+     }
+
+   return EINA_TRUE;
+}
