@@ -26,6 +26,7 @@ struct _Widget_Data
    Evas_Object *btn, *hover;
    Evas_Object *hover_parent;
    Eina_List *items;
+   Eina_Bool horizontal : 1;
 };
 
 struct _Elm_Hoversel_Item
@@ -35,11 +36,12 @@ struct _Elm_Hoversel_Item
    const char *icon_file;
    const char *icon_group;
    Elm_Icon_Type icon_type;
-   void (*func) (void *data, Evas_Object *obj, void *event_info);
-   void (*del_cb) (void *data, Evas_Object *obj, void *event_info);
+   Evas_Smart_Cb func;
+   Evas_Smart_Cb del_cb;
    void *data;
 };
 
+static const char *widtype = NULL;
 static void _del_pre_hook(Evas_Object *obj);
 static void _del_hook(Evas_Object *obj);
 static void _disable_hook(Evas_Object *obj);
@@ -52,7 +54,7 @@ _del_pre_hook(Evas_Object *obj)
 {
    Elm_Hoversel_Item *it;
    Widget_Data *wd = elm_widget_data_get(obj);
-
+   if (!wd) return;
    elm_hoversel_hover_end(obj);
    elm_hoversel_hover_parent_set(obj, NULL);
    EINA_LIST_FREE(wd->items, it)
@@ -69,6 +71,7 @@ static void
 _del_hook(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    free(wd);
 }
 
@@ -77,9 +80,12 @@ _theme_hook(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    char buf[4096];
-
    if (!wd) return;
-   snprintf(buf, sizeof(buf), "hoversel_vertical/%s", elm_widget_style_get(obj));
+   elm_hoversel_hover_end(obj);
+   if (wd->horizontal)
+     snprintf(buf, sizeof(buf), "hoversel_horizontal/%s", elm_widget_style_get(obj));
+   else
+     snprintf(buf, sizeof(buf), "hoversel_vertical/%s", elm_widget_style_get(obj));
    elm_object_style_set(wd->btn, buf);
 }
 
@@ -87,7 +93,7 @@ static void
 _disable_hook(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
-
+   if (!wd) return;
    if (elm_widget_disabled_get(obj))
      elm_widget_disabled_set(wd->btn, 1);
    else
@@ -99,7 +105,6 @@ _sizing_eval(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    Evas_Coord minw = -1, minh = -1, maxw = -1, maxh = -1;
-
    if (!wd) return;
    evas_object_size_hint_min_get(wd->btn, &minw, &minh);
    evas_object_size_hint_max_get(wd->btn, &maxw, &maxh);
@@ -108,19 +113,19 @@ _sizing_eval(Evas_Object *obj)
 }
 
 static void
-_changed_size_hints(void *data, Evas *e, Evas_Object *obj, void *event_info)
+_changed_size_hints(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    _sizing_eval(data);
 }
 
 static void
-_hover_clicked(void *data, Evas_Object *obj, void *event_info)
+_hover_clicked(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    elm_hoversel_hover_end(data);
 }
 
 static void
-_item_clicked(void *data, Evas_Object *obj, void *event_info)
+_item_clicked(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Elm_Hoversel_Item *it = data;
    Evas_Object *obj2 = it->obj;
@@ -141,9 +146,11 @@ _activate(Evas_Object *obj)
 
    if (!wd) return;
    if (elm_widget_disabled_get(obj)) return;
-
    wd->hover = elm_hover_add(obj);
-   snprintf(buf, sizeof(buf), "hoversel_vertical/%s", elm_widget_style_get(obj));
+   if (wd->horizontal)
+     snprintf(buf, sizeof(buf), "hoversel_horizontal/%s", elm_widget_style_get(obj));
+   else
+     snprintf(buf, sizeof(buf), "hoversel_vertical/%s", elm_widget_style_get(obj));
    elm_object_style_set(wd->hover, buf);
    evas_object_smart_callback_add(wd->hover, "clicked", _hover_clicked, obj);
    elm_hover_parent_set(wd->hover, wd->hover_parent);
@@ -151,9 +158,15 @@ _activate(Evas_Object *obj)
 
    bx = elm_box_add(wd->hover);
    elm_box_homogenous_set(bx, 1);
-
-   snprintf(buf, sizeof(buf), "hoversel_vertical_entry/%s",
-            elm_widget_style_get(obj));
+   
+   elm_box_horizontal_set(bx, wd->horizontal);
+   
+   if (wd->horizontal)
+     snprintf(buf, sizeof(buf), "hoversel_horizontal_entry/%s",
+              elm_widget_style_get(obj));
+   else
+     snprintf(buf, sizeof(buf), "hoversel_vertical_entry/%s",
+              elm_widget_style_get(obj));
    EINA_LIST_FOREACH(wd->items, l, it)
      {
 	bt = elm_button_add(wd->hover);
@@ -177,27 +190,34 @@ _activate(Evas_Object *obj)
 	evas_object_show(bt);
      }
 
-   elm_hover_content_set(wd->hover,
-                         elm_hover_best_content_location_get(wd->hover,
-                                                             ELM_HOVER_AXIS_VERTICAL),
-                         bx);
+   if (wd->horizontal)
+     elm_hover_content_set(wd->hover,
+                           elm_hover_best_content_location_get(wd->hover,
+                                                               ELM_HOVER_AXIS_HORIZONTAL),
+                           bx);
+   else
+     elm_hover_content_set(wd->hover,
+                           elm_hover_best_content_location_get(wd->hover,
+                                                               ELM_HOVER_AXIS_VERTICAL),
+                           bx);
    evas_object_show(bx);
 
    evas_object_show(wd->hover);
    evas_object_smart_callback_call(obj, "clicked", NULL);
+
+//   if (wd->horizontal) evas_object_hide(wd->btn);
 }
 
 static void
-_button_clicked(void *data, Evas_Object *obj, void *event_info)
+_button_clicked(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-  _activate(data);
+   _activate(data);
 }
 
 static void
-_parent_del(void *data, Evas *e, Evas_Object *obj, void *event_info)
+_parent_del(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Widget_Data *wd = elm_widget_data_get(data);
-
    if (!wd) return;
    wd->hover_parent = NULL;
 }
@@ -216,19 +236,26 @@ elm_hoversel_add(Evas_Object *parent)
    Evas_Object *obj;
    Evas *e;
    Widget_Data *wd;
+   char buf[4096];
 
    wd = ELM_NEW(Widget_Data);
    e = evas_object_evas_get(parent);
    obj = elm_widget_add(e);
+   ELM_SET_WIDTYPE(widtype, "hoversel");
    elm_widget_type_set(obj, "hoversel");
    elm_widget_sub_object_add(parent, obj);
    elm_widget_data_set(obj, wd);
    elm_widget_del_pre_hook_set(obj, _del_pre_hook);
    elm_widget_del_hook_set(obj, _del_hook);
+   elm_widget_theme_hook_set(obj, _theme_hook);
    elm_widget_disable_hook_set(obj, _disable_hook);
 
    wd->btn = elm_button_add(parent);
-   elm_object_style_set(wd->btn, "hoversel_vertical");
+   if (wd->horizontal)
+     snprintf(buf, sizeof(buf), "hoversel_horizontal/%s", elm_widget_style_get(obj));
+   else
+     snprintf(buf, sizeof(buf), "hoversel_vertical/%s", elm_widget_style_get(obj));
+   elm_object_style_set(wd->btn, buf);
    elm_widget_resize_object_set(obj, wd->btn);
    evas_object_event_callback_add(wd->btn, EVAS_CALLBACK_CHANGED_SIZE_HINTS,
 				  _changed_size_hints, obj);
@@ -240,7 +267,8 @@ elm_hoversel_add(Evas_Object *parent)
 /**
  * Set the Hover parent
  *
- * Sets the hover parent object. See Hover objects for more information.
+ * Sets the hover parent object. Should probably be the window that the hoversel
+ * is in.  See Hover objects for more information.
  *
  * @param obj The hoversel object
  * @param parent The parent to use
@@ -250,8 +278,8 @@ elm_hoversel_add(Evas_Object *parent)
 EAPI void
 elm_hoversel_hover_parent_set(Evas_Object *obj, Evas_Object *parent)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-
    if (!wd) return;
    if (wd->hover_parent)
      evas_object_event_callback_del_full(wd->hover_parent, EVAS_CALLBACK_DEL,
@@ -276,8 +304,8 @@ elm_hoversel_hover_parent_set(Evas_Object *obj, Evas_Object *parent)
 EAPI void
 elm_hoversel_label_set(Evas_Object *obj, const char *label)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-
    if (!wd) return;
    elm_button_label_set(wd->btn, label);
 }
@@ -290,15 +318,51 @@ elm_hoversel_label_set(Evas_Object *obj, const char *label)
  *
  * @ingroup Hoversel
  */
-EAPI const char*
-elm_hoversel_label_get(Evas_Object *obj)
+EAPI const char *
+elm_hoversel_label_get(const Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
    Widget_Data *wd = elm_widget_data_get(obj);
-
    if ((!wd) || (!wd->btn)) return NULL;
    return elm_button_label_get(wd->btn);
 }
 
+/**
+ * This sets the hoversel to expand horizontally.  The initial button
+ * will display horizontally regardless of this setting.
+ *
+ * @param obj The hoversel object
+ * @param horizontal If true, the hover will expand horizontally to the right.
+ *
+ * @ingroup Hoversel
+ */
+EAPI void
+elm_hoversel_horizontal_set(Evas_Object *obj, Eina_Bool horizontal)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype);
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
+   wd->horizontal = !!horizontal;
+}
+
+
+/**
+ * This returns whether the hoversel is set to expand horizontally.
+ *
+ * @param obj The hoversel object
+ * @return If true, the hover will expand horizontally to the right.
+ *
+ * @ingroup Hoversel
+ */
+EAPI Eina_Bool
+elm_hoversel_horizontal_get(const Evas_Object *obj)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype) EINA_FALSE;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return EINA_FALSE;
+   return wd->horizontal;
+}
+
 /**
  * Set the icon of the hoversel button
  *
@@ -313,8 +377,8 @@ elm_hoversel_label_get(Evas_Object *obj)
 EAPI void
 elm_hoversel_icon_set(Evas_Object *obj, Evas_Object *icon)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-
    if (!wd) return;
    elm_button_icon_set(wd->btn, icon);
 }
@@ -331,18 +395,17 @@ elm_hoversel_icon_set(Evas_Object *obj, Evas_Object *icon)
  * @ingroup Hoversel
  */
 EAPI Evas_Object *
-elm_hoversel_icon_get(Evas_Object *obj)
+elm_hoversel_icon_get(const Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
    Widget_Data *wd = elm_widget_data_get(obj);
-
    if ((!wd) || (!wd->btn)) return NULL;
    return elm_button_icon_get(wd->btn);
 }
 
 /**
- * Trigger the hoversel popup from code
- *
- * This makes the hoversel popup activate with the items added being listed.
+ * This triggers the hoversel popup from code, the same as though the
+ * user clicked the button.
  *
  * @param obj The hoversel object
  *
@@ -351,17 +414,15 @@ elm_hoversel_icon_get(Evas_Object *obj)
 EAPI void
 elm_hoversel_hover_begin(Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-
    if (!wd) return;
    if (wd->hover) return;
    _activate(obj);
 }
 
 /**
- * This ends the hoversel popup.
- *
- * This will close the hoversel popup, making it disappear, if it was active.
+ * This ends the hoversel popup as though the user clicked outside the hover.
  *
  * @param obj The hoversel object
  *
@@ -370,8 +431,8 @@ elm_hoversel_hover_begin(Evas_Object *obj)
 EAPI void
 elm_hoversel_hover_end(Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-
    if (!wd) return;
    if (!wd->hover) return;
    evas_object_del(wd->hover);
@@ -380,10 +441,27 @@ elm_hoversel_hover_end(Evas_Object *obj)
 }
 
 /**
- * Remove all the items from the given hoversel object.
+ * Returns whether the hoversel is expanded.
  *
+ * @param obj The hoversel object
+ * @return  This will return EINA_TRUE if the hoversel
+ * is expanded or EINA_FALSE if it is not expanded.
+ *
+ * @ingroup Hoversel
+ */
+EAPI Eina_Bool
+elm_hoversel_expanded_get(Evas_Object *obj)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype) EINA_FALSE;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return EINA_FALSE;
+   return (wd->hover) ? EINA_TRUE : EINA_FALSE;
+}
+  
+/**
  * This will remove all the children items from the hoversel. (should not be
- * called while the hoversel is active).
+ * called while the hoversel is active; use elm_hoversel_expanded_get()
+ * to check first).
  *
  * @param obj The hoversel object
  *
@@ -394,15 +472,14 @@ elm_hoversel_clear(Evas_Object *obj)
 {
    Elm_Hoversel_Item *it;
    Eina_List *l, *ll;
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-
    if (!wd) return;
-   EINA_LIST_FOREACH_SAFE(wd->items, l, ll, it)
-      elm_hoversel_item_del(it);
+   EINA_LIST_FOREACH_SAFE(wd->items, l, ll, it) elm_hoversel_item_del(it);
 }
 
 /**
- * Get the list of items.
+ * Get the list of items within the given hoversel.
  *
  * @param obj The hoversel object
  * @return Returns a list of Elm_Hoversel_Item*
@@ -410,12 +487,10 @@ elm_hoversel_clear(Evas_Object *obj)
  * @ingroup Hoversel
  */
 EAPI const Eina_List *
-elm_hoversel_items_get(Evas_Object *obj)
+elm_hoversel_items_get(const Evas_Object *obj)
 {
-   Elm_Hoversel_Item *it;
-   Eina_List *l, *ll;
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
    Widget_Data *wd = elm_widget_data_get(obj);
-
    if (!wd) return NULL;
    return wd->items;
 }
@@ -428,19 +503,20 @@ elm_hoversel_items_get(Evas_Object *obj)
  * right after the this function, and set icon_file to NULL here.
  *
  * @param obj The hoversel object
- * @param label The text abel to use for the item (NULL if not desired)
- * @param icon_file A image file path on disk to use for the icon  or standard
- * icon name(NULL if not desired)
+ * @param label The text label to use for the item (NULL if not desired)
+ * @param icon_file An image file path on disk to use for the icon or standard
+ * icon name (NULL if not desired)
  * @param icon_type The icon type if relevant
  * @param func Convenience function to call when this item is selected
- * @param data Data to pass to the conveience function
+ * @param data Data to pass to item-related functions
  * @return A handle to the item added.
  *
  * @ingroup Hoversel
  */
 EAPI Elm_Hoversel_Item *
-elm_hoversel_item_add(Evas_Object *obj, const char *label, const char *icon_file, Elm_Icon_Type icon_type, void (*func) (void *data, Evas_Object *obj, void *event_info), const void *data)
+elm_hoversel_item_add(Evas_Object *obj, const char *label, const char *icon_file, Elm_Icon_Type icon_type, Evas_Smart_Cb func, const void *data)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return NULL;
    Elm_Hoversel_Item *it = calloc(1, sizeof(Elm_Hoversel_Item));
@@ -459,7 +535,8 @@ elm_hoversel_item_add(Evas_Object *obj, const char *label, const char *icon_file
  * Delete an item from the hoversel
  *
  * This deletes the item from the hoversel (should not be called while the
- * hoversel is active).
+ * hoversel is active; use elm_hoversel_expanded_get()
+ * to check first).
  *
  * @param it The item to delete
  *
@@ -468,8 +545,8 @@ elm_hoversel_item_add(Evas_Object *obj, const char *label, const char *icon_file
 EAPI void
 elm_hoversel_item_del(Elm_Hoversel_Item *it)
 {
+   if (!it) return;
    Widget_Data *wd = elm_widget_data_get(it->obj);
-
    if (it->del_cb) it->del_cb((void *)it->data, it->obj, it);
    if (!wd) return;
    elm_hoversel_hover_end(it->obj);
@@ -481,7 +558,12 @@ elm_hoversel_item_del(Elm_Hoversel_Item *it)
 }
 
 /**
- * Set the function called when an hoversel item is freed.
+ * Set the function called when an item within the hoversel
+ * is freed. That function will receive these parameters:
+ *
+ * void *item_data
+ * Evas_Object *the_item_object
+ * Elm_Hoversel_Item *the_object_struct
  *
  * @param it The item to set the callback on
  * @param func The function called
@@ -489,16 +571,15 @@ elm_hoversel_item_del(Elm_Hoversel_Item *it)
  * @ingroup Hoversel
  */
 EAPI void
-elm_hoversel_item_del_cb_set(Elm_Hoversel_Item *it, void (*func)(void *data, Evas_Object *obj, void *event_info))
+elm_hoversel_item_del_cb_set(Elm_Hoversel_Item *it, Evas_Smart_Cb func)
 {
+   if (!it) return;
    it->del_cb = func;
 }
 
 /**
- * Get the data pointer passed to the item add function
- *
  * This returns the data pointer supplied with elm_hoversel_item_add() that
- * will be passed to the select function callback.
+ * will be passed to associated function callbacks.
  *
  * @param it The item to get the data from
  * @return The data pointer set with elm_hoversel_item_add()
@@ -513,12 +594,10 @@ elm_hoversel_item_data_get(Elm_Hoversel_Item *it)
 }
 
 /**
- * Get the text label of an hoversel item
- *
- * This returns the text of the label of the given hoversel item
+ * This returns the label text of the given hoversel item.
  *
  * @param it The item to get the label
- * @return The text label of the hoversel item
+ * @return The label text of the hoversel item
  *
  * @ingroup Hoversel
  */
@@ -530,10 +609,8 @@ elm_hoversel_item_label_get(Elm_Hoversel_Item *it)
 }
 
 /**
- * Set the icon of the hoversel item
- *
- * This set the icon for the given hoversel item. The icon can be loaded from
- * the standard set, from an image file or from an edje file.
+ * This sets the icon for the given hoversel item. The icon can be loaded from
+ * the standard set, from an image file, or from an edje file.
  *
  * @param it The item to set the icon
  * @param icon_file An image file path on disk to use for the icon or standard
@@ -548,13 +625,8 @@ EAPI void
 elm_hoversel_item_icon_set(Elm_Hoversel_Item *it, const char *icon_file, const char *icon_group, Elm_Icon_Type icon_type)
 {
    if (!it) return;
-
-   if (it->icon_file) eina_stringshare_del(it->icon_file);
-   it->icon_file = eina_stringshare_add(icon_file);
-
-   if (it->icon_group) eina_stringshare_del(it->icon_group);
-   it->icon_group = eina_stringshare_add(icon_group);
-
+   eina_stringshare_replace(&it->icon_file, icon_file);
+   eina_stringshare_replace(&it->icon_group, icon_group);
    it->icon_type = icon_type;
 }
 
@@ -563,10 +635,10 @@ elm_hoversel_item_icon_set(Elm_Hoversel_Item *it, const char *icon_file, const c
  *
  * @param it The item to get the icon from
  * @param icon_file The image file path on disk used for the icon or standard
- * icon name. return
+ * icon name
  * @param icon_group The edje group used if @p icon_file is an edje file. NULL
- * if the icon is not an edje file. return
- * @param icon_type The icon type. return
+ * if the icon is not an edje file
+ * @param icon_type The icon type
  *
  * @ingroup Hoversel
  */
@@ -574,7 +646,6 @@ EAPI void
 elm_hoversel_item_icon_get(Elm_Hoversel_Item *it, const char **icon_file, const char **icon_group, Elm_Icon_Type *icon_type)
 {
    if (!it) return;
-
    if (icon_file) *icon_file = it->icon_file;
    if (icon_group) *icon_group = it->icon_group;
    if (icon_type) *icon_type = it->icon_type;
