@@ -11,9 +11,9 @@
  * NONE
  *
  * A notepad object contains a scroller and an entry. It is a convenience
- * widget that loads a text file indicated, puts it int he scrollable entry
+ * widget that loads a text file indicated, puts it in the scrollable entry
  * and allows the user to edit it. Changes are written back to the original
- * file after a short delay. The file to load and save to is specificed by
+ * file after a short delay. The file to load and save to is specified by
  * elm_notepad_file_set().
  */
 typedef struct _Widget_Data Widget_Data;
@@ -28,6 +28,7 @@ struct _Widget_Data
    Eina_Bool auto_write : 1;
 };
 
+static const char *widtype = NULL;
 static void _del_hook(Evas_Object *obj);
 static void _sizing_eval(Evas_Object *obj);
 static void _on_focus_hook(void *data, Evas_Object *obj);
@@ -38,6 +39,7 @@ static void
 _del_hook(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    if (wd->file) eina_stringshare_del(wd->file);
    if (wd->delay_write)
      {
@@ -51,17 +53,17 @@ static void
 _sizing_eval(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
-
+   if (!wd) return;
    evas_object_size_hint_min_set(obj, -1, -1);
    evas_object_size_hint_max_set(obj, -1, -1);
 }
 
 static void
-_on_focus_hook(void *data, Evas_Object *obj)
+_on_focus_hook(void *data __UNUSED__, Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
-   if (elm_widget_focus_get(obj))
-     elm_widget_focus_steal(wd->entry);
+   if (!wd) return;
+   if (elm_widget_focus_get(obj)) elm_widget_focus_steal(wd->entry);
 }
 
 static char *
@@ -119,7 +121,7 @@ _load(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    char *text;
-
+   if (!wd) return;
    if (!wd->file)
      {
 	elm_entry_entry_set(wd->entry, "");
@@ -127,15 +129,15 @@ _load(Evas_Object *obj)
      }
    switch (wd->format)
      {
-      case ELM_TEXT_FORMAT_PLAIN_UTF8:
+     case ELM_TEXT_FORMAT_PLAIN_UTF8:
 	text = _load_plain(wd->file);
 	break;
-      case ELM_TEXT_FORMAT_MARKUP_UTF8:
+     case ELM_TEXT_FORMAT_MARKUP_UTF8:
 	text = _load_file(wd->file);
 	break;
-      default:
+     default:
 	elm_entry_entry_set(wd->entry, "Unknown Text Format");
-	break;
+	text = NULL;
      }
    if (text)
      {
@@ -150,7 +152,7 @@ static void
 _save_markup_utf8(const char *file, const char *text)
 {
    FILE *f;
-
+   
    if ((!text) || (text[0] == 0))
      {
 	ecore_file_unlink(file);
@@ -170,7 +172,7 @@ static void
 _save_plain_utf8(const char *file, const char *text)
 {
    char *text2;
-
+   
    text2 = elm_entry_markup_to_utf8(text);
    if (text2)
      {
@@ -183,6 +185,7 @@ static void
 _save(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    if (!wd->file) return;
    switch (wd->format)
      {
@@ -197,19 +200,21 @@ _save(Evas_Object *obj)
      }
 }
 
-static int
+static Eina_Bool
 _delay_write(void *data)
 {
    Widget_Data *wd = elm_widget_data_get(data);
+   if (!wd) return ECORE_CALLBACK_CANCEL;
    _save(data);
    wd->delay_write = NULL;
-   return 0;
+   return ECORE_CALLBACK_CANCEL;
 }
 
 static void
-_entry_changed(void *data, Evas_Object *obj, void *event_info)
+_entry_changed(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Widget_Data *wd = elm_widget_data_get(data);
+   if (!wd) return;
    if (wd->delay_write)
      {
 	ecore_timer_del(wd->delay_write);
@@ -220,7 +225,7 @@ _entry_changed(void *data, Evas_Object *obj, void *event_info)
 }
 
 static void
-_hold_on(void *data, Evas_Object *obj, void *event_info)
+_hold_on(void *data __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
@@ -228,7 +233,7 @@ _hold_on(void *data, Evas_Object *obj, void *event_info)
 }
 
 static void
-_hold_off(void *data, Evas_Object *obj, void *event_info)
+_hold_off(void *data __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
@@ -236,7 +241,7 @@ _hold_off(void *data, Evas_Object *obj, void *event_info)
 }
 
 static void
-_freeze_on(void *data, Evas_Object *obj, void *event_info)
+_freeze_on(void *data __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
@@ -244,7 +249,7 @@ _freeze_on(void *data, Evas_Object *obj, void *event_info)
 }
 
 static void
-_freeze_off(void *data, Evas_Object *obj, void *event_info)
+_freeze_off(void *data __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
@@ -269,6 +274,7 @@ elm_notepad_add(Evas_Object *parent)
    wd = ELM_NEW(Widget_Data);
    e = evas_object_evas_get(parent);
    obj = elm_widget_add(e);
+   ELM_SET_WIDTYPE(widtype, "notepad");
    elm_widget_type_set(obj, "notepad");
    elm_widget_sub_object_add(parent, obj);
    elm_widget_on_focus_hook_set(obj, _on_focus_hook, NULL);
@@ -299,30 +305,48 @@ elm_notepad_add(Evas_Object *parent)
 }
 
 /**
- * Set the file to load  text from and save text back to
+ * This sets the file (and implicitly loads it) for the text to display and
+ * then edit. All changes are written back to the file after a short delay.
  *
  * @param obj The notepad object
  * @param file The path to the file to load and save
  * @param format The file format
  *
- * This sets the file (and implicitly loads it) for the text to display and
- * then edit. All changes are written back to the file after a short delay.
  *
  * @ingroup Notepad
  */
 EAPI void
 elm_notepad_file_set(Evas_Object *obj, const char *file, Elm_Text_Format format)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    if (wd->delay_write)
      {
 	ecore_timer_del(wd->delay_write);
 	wd->delay_write = NULL;
      }
    _save(obj);
-   if (wd->file) eina_stringshare_del(wd->file);
-   wd->file = NULL;
-   if (file) wd->file = eina_stringshare_add(file);
+   eina_stringshare_replace(&wd->file, file);
    wd->format = format;
    _load(obj);
 }
+
+/**
+ * This will enable or disable the scroller bounce mode for the notepad. See
+ * elm_scroller_bounce_set() for details
+ *
+ * @param obj The notepad object
+ * @param h_bounce Allow bounce horizontally
+ * @param v_bounce Allow bounce vertically
+ *
+ * @ingroup Notepad
+ */
+EAPI void
+elm_notepad_bounce_set(Evas_Object *obj, Eina_Bool h_bounce, Eina_Bool v_bounce)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype);
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
+   elm_scroller_bounce_set(wd->scr, h_bounce, v_bounce);
+}
