@@ -18,6 +18,7 @@ struct _Widget_Data
    const char  *file, *group;
 };
 
+static const char *widtype = NULL;
 static void _del_hook(Evas_Object *obj);
 static void _theme_hook(Evas_Object *obj);
 static void _custom_resize(void *data, Evas *a, Evas_Object *obj, void *event_info);
@@ -33,13 +34,12 @@ static void
 _theme_hook(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
-   _elm_theme_set(wd->img, "bg", "base", elm_widget_style_get(obj));
+   _elm_theme_object_set(obj, wd->img, "bg", "base", elm_widget_style_get(obj));
 }
 
 static void
-_custom_resize(void *data, Evas *a, Evas_Object *obj, void *event_info)
+_custom_resize(void *data __UNUSED__, Evas *e __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
 {
-   Widget_Data *wd = data;
    int iw = 0, ih = 0;
    Evas_Coord x, y, w, h, ow = 0, oh = 0;
 
@@ -77,6 +77,7 @@ elm_bg_add(Evas_Object *parent)
    wd = ELM_NEW(Widget_Data);
    e = evas_object_evas_get(parent);
    obj = elm_widget_add(e);
+   ELM_SET_WIDTYPE(widtype, "bg");
    elm_widget_type_set(obj, "bg");
    elm_widget_sub_object_add(parent, obj);
    elm_widget_data_set(obj, wd);
@@ -85,7 +86,7 @@ elm_bg_add(Evas_Object *parent)
    elm_widget_can_focus_set(obj, 0);
 
    wd->img = edje_object_add(e);
-   _elm_theme_set(wd->img, "bg", "base", "default");
+   _elm_theme_object_set(obj, wd->img, "bg", "base", "default");
    elm_widget_resize_object_set(obj, wd->img);
    return obj;
 }
@@ -106,6 +107,7 @@ elm_bg_add(Evas_Object *parent)
 EAPI void
 elm_bg_file_set(Evas_Object *obj, const char *file, const char *group)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
    const char *p;
 
@@ -115,12 +117,8 @@ elm_bg_file_set(Evas_Object *obj, const char *file, const char *group)
 	wd->custom_img = NULL;
      }
    if (!file) return;
-   if (wd->file) eina_stringshare_del(wd->file);
-   if (file) wd->file = eina_stringshare_add(file);
-   else wd->file = NULL;
-   if (wd->group) eina_stringshare_del(wd->group);
-   if (group) wd->group = eina_stringshare_add(group);
-   else wd->group = NULL;
+   eina_stringshare_replace(&wd->file, file);
+   eina_stringshare_replace(&wd->group, group);
    if (((p = strrchr(file, '.'))) && (!strcasecmp(p, ".edj")))
      {
 	wd->custom_img = edje_object_add(evas_object_evas_get(wd->img));
