@@ -15,6 +15,8 @@
  * will set the packing weight. The weights of all items being packed are added
  * up and if items are to be sized up to fit, those with the higher weights get
  * proportionally more space.
+ *
+ * NOTE: Objects should not be added to box objects using _add() calls.
  */
 typedef struct _Widget_Data Widget_Data;
 
@@ -25,6 +27,7 @@ struct _Widget_Data
    Eina_Bool homogeneous:1;
 };
 
+static const char *widtype = NULL;
 static void _del_hook(Evas_Object *obj);
 static void _sizing_eval(Evas_Object *obj);
 static void _changed_size_hints(void *data, Evas *e, Evas_Object *obj, void *event_info);
@@ -34,7 +37,7 @@ static void
 _del_pre_hook(Evas_Object *obj)
 {
     Widget_Data *wd = elm_widget_data_get(obj);
-
+   if (!wd) return;
     evas_object_event_callback_del_full
         (wd->box, EVAS_CALLBACK_CHANGED_SIZE_HINTS, _changed_size_hints, obj);
     evas_object_box_remove_all(wd->box, 0);
@@ -44,6 +47,7 @@ static void
 _del_hook(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    free(wd);
 }
 
@@ -53,7 +57,7 @@ _sizing_eval(Evas_Object *obj)
    Widget_Data *wd = elm_widget_data_get(obj);
    Evas_Coord minw = -1, minh = -1, maxw = -1, maxh = -1;
    Evas_Coord w, h;
-
+   if (!wd) return;
    evas_object_size_hint_min_get(wd->box, &minw, &minh);
    evas_object_size_hint_max_get(wd->box, &maxw, &maxh);
    evas_object_size_hint_min_set(obj, minw, minh);
@@ -67,13 +71,13 @@ _sizing_eval(Evas_Object *obj)
 }
 
 static void
-_changed_size_hints(void *data, Evas *e, Evas_Object *obj, void *event_info)
+_changed_size_hints(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    _sizing_eval(data);
 }
 
 static void
-_sub_del(void *data, Evas_Object *obj, void *event_info)
+_sub_del(void *data __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
 {
    _sizing_eval(obj);
 }
@@ -82,7 +86,7 @@ static void
 _layout(Evas_Object *o, Evas_Object_Box_Data *priv, void *data)
 {
    Widget_Data *wd = data;
-
+   if (!wd) return;
    _els_box_layout(o, priv, wd->horizontal, wd->homogeneous);
 }
 
@@ -105,6 +109,7 @@ elm_box_add(Evas_Object *parent)
    wd = ELM_NEW(Widget_Data);
    e = evas_object_evas_get(parent);
    obj = elm_widget_add(e);
+   ELM_SET_WIDTYPE(widtype, "box");
    elm_widget_type_set(obj, "box");
    elm_widget_sub_object_add(parent, obj);
    elm_widget_data_set(obj, wd);
@@ -140,7 +145,9 @@ elm_box_add(Evas_Object *parent)
 EAPI void
 elm_box_horizontal_set(Evas_Object *obj, Eina_Bool horizontal)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    wd->horizontal = !!horizontal;
    evas_object_smart_calculate(wd->box);
    /*if (wd->horizontal)
@@ -177,7 +184,9 @@ elm_box_horizontal_set(Evas_Object *obj, Eina_Bool horizontal)
 EAPI void
 elm_box_homogenous_set(Evas_Object *obj, Eina_Bool homogenous)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    wd->homogeneous = !!homogenous;
    evas_object_smart_calculate(wd->box);
    /*if (wd->horizontal)
@@ -214,7 +223,9 @@ elm_box_homogenous_set(Evas_Object *obj, Eina_Bool homogenous)
 EAPI void
 elm_box_pack_start(Evas_Object *obj, Evas_Object *subobj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    elm_widget_sub_object_add(obj, subobj);
    evas_object_box_prepend(wd->box, subobj);
 }
@@ -233,7 +244,9 @@ elm_box_pack_start(Evas_Object *obj, Evas_Object *subobj)
 EAPI void
 elm_box_pack_end(Evas_Object *obj, Evas_Object *subobj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    elm_widget_sub_object_add(obj, subobj);
    evas_object_box_append(wd->box, subobj);
 }
@@ -255,7 +268,9 @@ elm_box_pack_end(Evas_Object *obj, Evas_Object *subobj)
 EAPI void
 elm_box_pack_before(Evas_Object *obj, Evas_Object *subobj, Evas_Object *before)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    elm_widget_sub_object_add(obj, subobj);
    evas_object_box_insert_before(wd->box, subobj, before);
 }
@@ -277,7 +292,9 @@ elm_box_pack_before(Evas_Object *obj, Evas_Object *subobj, Evas_Object *before)
 EAPI void
 elm_box_pack_after(Evas_Object *obj, Evas_Object *subobj, Evas_Object *after)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    elm_widget_sub_object_add(obj, subobj);
    evas_object_box_insert_after(wd->box, subobj, after);
 }
@@ -294,7 +311,9 @@ elm_box_pack_after(Evas_Object *obj, Evas_Object *subobj, Evas_Object *after)
 EAPI void
 elm_box_clear(Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    evas_object_box_remove_all(wd->box, 1);
 }
 
@@ -311,7 +330,9 @@ elm_box_clear(Evas_Object *obj)
 EAPI void
 elm_box_unpack(Evas_Object *obj, Evas_Object *subobj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    evas_object_box_remove(wd->box, subobj);
 }
 
@@ -328,6 +349,8 @@ elm_box_unpack(Evas_Object *obj, Evas_Object *subobj)
 EAPI void
 elm_box_unpack_all(Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    evas_object_box_remove_all(wd->box, 0);
 }
