@@ -16,11 +16,12 @@ struct _Elm_Carousel_Item
 {
    Evas_Object *obj, *base, *icon;
    const char *label;
-   void (*func) (void *data, Evas_Object *obj, void *event_info);
+   Evas_Smart_Cb func;
    const void *data;
    Eina_Bool selected : 1;
 };
 
+static const char *widtype = NULL;
 static void _del_hook(Evas_Object *obj);
 static void _theme_hook(Evas_Object *obj);
 static void _sizing_eval(Evas_Object *obj);
@@ -30,7 +31,7 @@ _item_show(Elm_Carousel_Item *it)
 {
    Widget_Data *wd = elm_widget_data_get(it->obj);
    Evas_Coord x, y, w, h, bx, by;
-
+   if (!wd) return;
    evas_object_geometry_get(wd->bx, &bx, &by, NULL, NULL);
    evas_object_geometry_get(it->base, &x, &y, &w, &h);
    elm_smart_scroller_child_region_show(wd->scr, x - bx, y - by, w, h);
@@ -43,7 +44,7 @@ _item_select(Elm_Carousel_Item *it)
    Widget_Data *wd = elm_widget_data_get(it->obj);
    Evas_Object *obj2;
    const Eina_List *l;
-
+   if (!wd) return;
    if (it->selected) return;
    EINA_LIST_FOREACH(wd->items, l, it2)
      {
@@ -66,6 +67,7 @@ static void
 _del_hook(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    free(wd);
 }
 
@@ -75,14 +77,14 @@ _theme_hook(Evas_Object *obj)
    Widget_Data *wd = elm_widget_data_get(obj);
    const Eina_List *l;
    const Elm_Carousel_Item *it;
-
+   if (!wd) return;
    EINA_LIST_FOREACH(wd->items, l, it)
      {
         Evas_Coord mw, mh;
 
 	if (it->selected)
 	  edje_object_signal_emit(it->base, "elm,state,selected", "elm");
-	_elm_theme_set(it->base, "carousel", "item", elm_widget_style_get(obj));
+	_elm_theme_object_set(obj, it->base, "carousel", "item", elm_widget_style_get(obj));
 	edje_object_scale_set(it->base, elm_widget_scale_get(obj) * _elm_config->scale);
 	if (it->icon)
 	  {
@@ -105,7 +107,7 @@ _sizing_eval(Evas_Object *obj)
    Widget_Data *wd = elm_widget_data_get(obj);
    Evas_Coord minw = -1, minh = -1, maxw = -1, maxh = -1;
    Evas_Coord vw = 0, vh = 0;
-
+   if (!wd) return;
    edje_object_size_min_calc(elm_smart_scroller_edje_object_get(wd->scr), &minw, &minh);
    evas_object_resize(wd->scr, 500, 500);
    evas_object_size_hint_min_get(wd->bx, &minw, &minh);
@@ -118,13 +120,13 @@ _sizing_eval(Evas_Object *obj)
 }
 
 static void
-_resize(void *data, Evas *e, Evas_Object *obj, void *event_info)
+_resize(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Widget_Data *wd = elm_widget_data_get(data);
    Evas_Coord mw, mh, vw, vh, w, h;
    const Eina_List *l;
    Elm_Carousel_Item *it;
-
+   if (!wd) return;
    elm_smart_scroller_child_viewport_size_get(wd->scr, &vw, &vh);
    evas_object_size_hint_min_get(wd->bx, &mw, &mh);
    evas_object_geometry_get(wd->bx, NULL, NULL, &w, &h);
@@ -144,7 +146,7 @@ _resize(void *data, Evas *e, Evas_Object *obj, void *event_info)
 }
 
 static void
-_select(void *data, Evas_Object *obj, const char *emission, const char *source)
+_select(void *data, Evas_Object *obj __UNUSED__, const char *emission __UNUSED__, const char *source __UNUSED__)
 {
    _item_select(data);
 }
@@ -159,6 +161,7 @@ elm_carousel_add(Evas_Object *parent)
    wd = ELM_NEW(Widget_Data);
    e = evas_object_evas_get(parent);
    obj = elm_widget_add(e);
+   ELM_SET_WIDTYPE(widtype, "carousel");
    elm_widget_type_set(obj, "carousel");
    elm_widget_sub_object_add(parent, obj);
    elm_widget_data_set(obj, wd);
@@ -167,7 +170,8 @@ elm_carousel_add(Evas_Object *parent)
    elm_widget_can_focus_set(obj, 0);
 
    wd->scr = elm_smart_scroller_add(e);
-   elm_smart_scroller_theme_set(wd->scr, "carousel", "base", "default");
+   elm_smart_scroller_widget_set(wd->scr, obj);
+   elm_smart_scroller_object_theme_set(obj, wd->scr, "carousel", "base", "default");
    elm_widget_resize_object_set(obj, wd->scr);
    elm_smart_scroller_policy_set(wd->scr,
 				 ELM_SMART_SCROLLER_POLICY_AUTO,
@@ -190,9 +194,11 @@ elm_carousel_add(Evas_Object *parent)
 }
 
 EAPI Elm_Carousel_Item *
-elm_carousel_item_add(Evas_Object *obj, Evas_Object *icon, const char *label, void (*func) (void *data, Evas_Object *obj, void *event_info), const void *data)
+elm_carousel_item_add(Evas_Object *obj, Evas_Object *icon, const char *label, Evas_Smart_Cb func, const void *data)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return NULL;
    Evas_Coord mw, mh;
    Elm_Carousel_Item *it = calloc(1, sizeof(Elm_Carousel_Item));
 
@@ -204,7 +210,7 @@ elm_carousel_item_add(Evas_Object *obj, Evas_Object *icon, const char *label, vo
    it->func = func;
    it->data = data;
    it->base = edje_object_add(evas_object_evas_get(obj));
-   _elm_theme_set(it->base, "carousel", "item", elm_widget_style_get(obj));
+   _elm_theme_object_set(obj, it->base, "carousel", "item", elm_widget_style_get(obj));
    edje_object_signal_callback_add(it->base, "elm,action,click", "elm",
 				   _select, it);
    elm_widget_sub_object_add(obj, it->base);
@@ -234,7 +240,7 @@ elm_carousel_item_del(Elm_Carousel_Item *it)
 {
    Widget_Data *wd = elm_widget_data_get(it->obj);
    Evas_Object *obj2 = it->obj;
-
+   if (!wd) return;
    wd->items = eina_list_remove(wd->items, it);
    eina_stringshare_del(it->label);
    if (it->icon) evas_object_del(it->icon);
