@@ -29,6 +29,7 @@ struct _Widget_Data
    const char *label;
 };
 
+static const char *widtype = NULL;
 static void _del_hook(Evas_Object *obj);
 static void _theme_hook(Evas_Object *obj);
 static void _disable_hook(Evas_Object *obj);
@@ -39,10 +40,17 @@ static void _signal_check_off(void *data, Evas_Object *obj, const char *emission
 static void _signal_check_on(void *data, Evas_Object *obj, const char *emission, const char *source);
 static void _signal_check_toggle(void *data, Evas_Object *obj, const char *emission, const char *source);
 
+static const char SIG_CHANGED[] = "changed";
+static const Evas_Smart_Cb_Description _signals[] = {
+  {SIG_CHANGED, ""},
+  {NULL, NULL}
+};
+
 static void
 _del_hook(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    if (wd->label) eina_stringshare_del(wd->label);
    free(wd);
 }
@@ -51,7 +59,8 @@ static void
 _theme_hook(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
-   _elm_theme_set(wd->chk, "check", "base", elm_widget_style_get(obj));
+   if (!wd) return;
+   _elm_theme_object_set(obj, wd->chk, "check", "base", elm_widget_style_get(obj));
    if (wd->icon)
      edje_object_signal_emit(wd->chk, "elm,state,icon,visible", "elm");
    else
@@ -74,6 +83,7 @@ static void
 _disable_hook(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    if (elm_widget_disabled_get(obj))
      edje_object_signal_emit(wd->chk, "elm,state,disabled", "elm");
    else
@@ -85,7 +95,7 @@ _sizing_eval(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    Evas_Coord minw = -1, minh = -1, maxw = -1, maxh = -1;
-
+   if (!wd) return;
    elm_coords_finger_size_adjust(1, &minw, 1, &minh);
    edje_object_size_min_restricted_calc(wd->chk, &minw, &minh, minw, minh);
    elm_coords_finger_size_adjust(1, &minw, 1, &minh);
@@ -94,9 +104,10 @@ _sizing_eval(Evas_Object *obj)
 }
 
 static void
-_changed_size_hints(void *data, Evas *e, Evas_Object *obj, void *event_info)
+_changed_size_hints(void *data, Evas *e __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
 {
    Widget_Data *wd = elm_widget_data_get(data);
+   if (!wd) return;
    if (obj != wd->icon) return;
    Evas_Coord mw, mh;
    evas_object_size_hint_min_get(obj, &mw, &mh);
@@ -104,10 +115,11 @@ _changed_size_hints(void *data, Evas *e, Evas_Object *obj, void *event_info)
 }
 
 static void
-_sub_del(void *data, Evas_Object *obj, void *event_info)
+_sub_del(void *data __UNUSED__, Evas_Object *obj, void *event_info)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    Evas_Object *sub = event_info;
+   if (!wd) return;
    if (sub == wd->icon)
      {
 	edje_object_signal_emit(wd->chk, "elm,state,icon,hidden", "elm");
@@ -115,40 +127,44 @@ _sub_del(void *data, Evas_Object *obj, void *event_info)
                                        _changed_size_hints, obj);
 	wd->icon = NULL;
 	_sizing_eval(obj);
+	edje_object_message_signal_process(wd->chk);
      }
 }
 
 static void
-_signal_check_off(void *data, Evas_Object *obj, const char *emission, const char *source)
+_signal_check_off(void *data, Evas_Object *obj __UNUSED__, const char *emission __UNUSED__, const char *source __UNUSED__)
 {
    Widget_Data *wd = elm_widget_data_get(data);
+   if (!wd) return;
    wd->state = EINA_FALSE;
    if (wd->statep) *wd->statep = wd->state;
    edje_object_signal_emit(wd->chk, "elm,state,check,off", "elm");
-   evas_object_smart_callback_call(data, "changed", NULL);
+   evas_object_smart_callback_call(data, SIG_CHANGED, NULL);
 }
 
 static void
-_signal_check_on(void *data, Evas_Object *obj, const char *emission, const char *source)
+_signal_check_on(void *data, Evas_Object *obj __UNUSED__, const char *emission __UNUSED__, const char *source __UNUSED__)
 {
    Widget_Data *wd = elm_widget_data_get(data);
+   if (!wd) return;
    wd->state = EINA_TRUE;
    if (wd->statep) *wd->statep = wd->state;
    edje_object_signal_emit(wd->chk, "elm,state,check,on", "elm");
-   evas_object_smart_callback_call(data, "changed", NULL);
+   evas_object_smart_callback_call(data, SIG_CHANGED, NULL);
 }
 
 static void
-_signal_check_toggle(void *data, Evas_Object *obj, const char *emission, const char *source)
+_signal_check_toggle(void *data, Evas_Object *obj __UNUSED__, const char *emission __UNUSED__, const char *source __UNUSED__)
 {
    Widget_Data *wd = elm_widget_data_get(data);
+   if (!wd) return;
    wd->state = !wd->state;
    if (wd->statep) *wd->statep = wd->state;
    if (wd->state)
      edje_object_signal_emit(wd->chk, "elm,state,check,on", "elm");
    else
      edje_object_signal_emit(wd->chk, "elm,state,check,off", "elm");
-   evas_object_smart_callback_call(data, "changed", NULL);
+   evas_object_smart_callback_call(data, SIG_CHANGED, NULL);
 }
 
 /**
@@ -169,6 +185,7 @@ elm_check_add(Evas_Object *parent)
    wd = ELM_NEW(Widget_Data);
    e = evas_object_evas_get(parent);
    obj = elm_widget_add(e);
+   ELM_SET_WIDTYPE(widtype, "check");
    elm_widget_type_set(obj, "check");
    elm_widget_sub_object_add(parent, obj);
    elm_widget_data_set(obj, wd);
@@ -177,7 +194,7 @@ elm_check_add(Evas_Object *parent)
    elm_widget_disable_hook_set(obj, _disable_hook);
 
    wd->chk = edje_object_add(e);
-   _elm_theme_set(wd->chk, "check", "base", "default");
+   _elm_theme_object_set(obj, wd->chk, "check", "base", "default");
    edje_object_signal_callback_add(wd->chk, "elm,action,check,on", "",
                                    _signal_check_on, obj);
    edje_object_signal_callback_add(wd->chk, "elm,action,check,off", "",
@@ -189,6 +206,10 @@ elm_check_add(Evas_Object *parent)
    evas_object_smart_callback_add(obj, "sub-object-del", _sub_del, obj);
 
    _sizing_eval(obj);
+
+   // TODO: convert Elementary to subclassing of Evas_Smart_Class
+   // TODO: and save some bytes, making descriptions per-class and not instance!
+   evas_object_smart_callbacks_descriptions_set(obj, _signals);
    return obj;
 }
 
@@ -203,20 +224,14 @@ elm_check_add(Evas_Object *parent)
 EAPI void
 elm_check_label_set(Evas_Object *obj, const char *label)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-   Evas_Coord mw, mh;
-
-   if (wd->label) eina_stringshare_del(wd->label);
+   if (!wd) return;
+   eina_stringshare_replace(&wd->label, label);
    if (label)
-     {
-	wd->label = eina_stringshare_add(label);
-	edje_object_signal_emit(wd->chk, "elm,state,text,visible", "elm");
-     }
+     edje_object_signal_emit(wd->chk, "elm,state,text,visible", "elm");
    else
-     {
-	wd->label = NULL;
-	edje_object_signal_emit(wd->chk, "elm,state,text,hidden", "elm");
-     }
+     edje_object_signal_emit(wd->chk, "elm,state,text,hidden", "elm");
    edje_object_message_signal_process(wd->chk);
    edje_object_part_text_set(wd->chk, "elm.text", label);
    _sizing_eval(obj);
@@ -230,9 +245,10 @@ elm_check_label_set(Evas_Object *obj, const char *label)
  *
  * @ingroup Check
  */
-EAPI const char*
-elm_check_label_get(Evas_Object *obj)
+EAPI const char *
+elm_check_label_get(const Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return NULL;
    return wd->label;
@@ -241,10 +257,7 @@ elm_check_label_get(Evas_Object *obj)
 /**
  * Set the icon object of the check object
  *
- * Once the icon object is set, it will become a child of the check object and
- * be deleted when the check object is deleted. If another icon object is set
- * then the previous one becomes orophaned and will no longer be deleted along
- * with the check.
+ * Once the icon object is set, a previously set one will be deleted.
  *
  * @param obj The check object
  * @param icon The icon object
@@ -254,19 +267,22 @@ elm_check_label_get(Evas_Object *obj)
 EAPI void
 elm_check_icon_set(Evas_Object *obj, Evas_Object *icon)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-   if ((wd->icon != icon) && (wd->icon))
-     elm_widget_sub_object_del(obj, wd->icon);
+   if (!wd) return;
+   if (wd->icon == icon) return;
+   if (wd->icon) evas_object_del(wd->icon);
    wd->icon = icon;
    if (icon)
      {
 	elm_widget_sub_object_add(obj, icon);
-	edje_object_part_swallow(wd->chk, "elm.swallow.content", icon);
-	edje_object_signal_emit(wd->chk, "elm,state,icon,visible", "elm");
 	evas_object_event_callback_add(icon, EVAS_CALLBACK_CHANGED_SIZE_HINTS,
 				       _changed_size_hints, obj);
-	_sizing_eval(obj);
+	edje_object_part_swallow(wd->chk, "elm.swallow.content", icon);
+	edje_object_signal_emit(wd->chk, "elm,state,icon,visible", "elm");
+	edje_object_message_signal_process(wd->chk);
      }
+   _sizing_eval(obj);
 }
 
 /**
@@ -278,8 +294,9 @@ elm_check_icon_set(Evas_Object *obj, Evas_Object *icon)
  * @ingroup Check
  */
 EAPI Evas_Object *
-elm_check_icon_get(Evas_Object *obj)
+elm_check_icon_get(const Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return NULL;
    return wd->icon;
@@ -299,7 +316,9 @@ elm_check_icon_get(Evas_Object *obj)
 EAPI void
 elm_check_state_set(Evas_Object *obj, Eina_Bool state)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    if (state != wd->state)
      {
 	wd->state = state;
@@ -322,7 +341,9 @@ elm_check_state_set(Evas_Object *obj, Eina_Bool state)
 EAPI Eina_Bool
 elm_check_state_get(const Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) EINA_FALSE;
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return EINA_FALSE;
    return wd->state;
 }
 
@@ -344,7 +365,9 @@ elm_check_state_get(const Evas_Object *obj)
 EAPI void
 elm_check_state_pointer_set(Evas_Object *obj, Eina_Bool *statep)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    if (statep)
      {
 	wd->statep = statep;
