@@ -5,7 +5,7 @@
  * @defgroup Clock Clock
  *
  * It's a widget to show clock with animation. The update of time is
- * showed in an animation like a flip of a sheet.
+ * shown in an animation like the flip of a sheet.
  *
  */
 
@@ -14,34 +14,40 @@ typedef struct _Widget_Data Widget_Data;
 struct _Widget_Data
 {
    Evas_Object *clk;
+   double interval;
    Eina_Bool seconds : 1;
    Eina_Bool am_pm : 1;
    Eina_Bool edit : 1;
+   Elm_Clock_Digedit digedit;
    int hrs, min, sec;
    Evas_Object *digit[6];
    Evas_Object *ampm;
-   Ecore_Timer *ticker;
-   struct 
+   Evas_Object *sel_obj;
+   Ecore_Timer *ticker, *spin;
+   struct
      {
         int hrs, min, sec;
         char ampm;
         Eina_Bool seconds : 1;
         Eina_Bool am_pm : 1;
         Eina_Bool edit : 1;
+	Elm_Clock_Digedit digedit;
      } cur;
 };
 
+static const char *widtype = NULL;
 static void _del_hook(Evas_Object *obj);
 static void _theme_hook(Evas_Object *obj);
-static int _ticker(void *data);
-static void _signal_clock_val_up(void *data, Evas_Object *obj, const char *emission, const char *source);
-static void _signal_clock_val_down(void *data, Evas_Object *obj, const char *emission, const char *source);
+static Eina_Bool _ticker(void *data);
+static Eina_Bool _signal_clock_val_up(void *data);
+static Eina_Bool _signal_clock_val_down(void *data);
 static void _time_update(Evas_Object *obj);
 
 static void
 _del_hook(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    int i;
    for (i = 0; i < 6; i++)
      {
@@ -49,6 +55,7 @@ _del_hook(Evas_Object *obj)
      }
    if (wd->ampm) evas_object_del(wd->ampm);
    if (wd->ticker) ecore_timer_del(wd->ticker);
+   if (wd->spin) ecore_timer_del(wd->spin);
    free(wd);
 }
 
@@ -56,11 +63,70 @@ static void
 _theme_hook(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
+   if (elm_widget_focus_get(obj))
+     edje_object_signal_emit(wd->clk, "elm,action,focus", "elm");
+   else
+     edje_object_signal_emit(wd->clk, "elm,action,unfocus", "elm");
    wd->cur.am_pm = !wd->cur.am_pm; /* hack - force update */
    _time_update(obj);
 }
 
-static int
+static void
+_on_focus_hook(void *data __UNUSED__, Evas_Object *obj)
+{
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
+   if (elm_widget_focus_get(obj))
+     edje_object_signal_emit(wd->clk, "elm,action,focus", "elm");
+   else
+     edje_object_signal_emit(wd->clk, "elm,action,unfocus", "elm");
+}
+
+static void
+_signal_emit_hook(Evas_Object *obj, const char *emission, const char *source)
+{
+   Widget_Data *wd = elm_widget_data_get(obj);
+   int i;
+   if (!wd) return;
+   edje_object_signal_emit(wd->clk, emission, source);
+   for (i = 0; i < 6; i++)
+     {
+	if (wd->digit[i])
+	  edje_object_signal_emit(wd->digit[i], emission, source);
+     }
+}
+
+static void
+_signal_listen_hook(Evas_Object *obj, const char *emission, const char *source, void (*func_cb) (void *data, Evas_Object *o, const char *emission, const char *source), void *data)
+{
+   Widget_Data *wd = elm_widget_data_get(obj);
+   int i;
+   if (!wd) return;
+   edje_object_signal_callback_add(wd->clk, emission, source, func_cb, data);
+   for (i = 0; i < 6; i++)
+     {
+	if (wd->digit[i])
+	  edje_object_signal_callback_add(wd->digit[i], emission, source,
+		func_cb, data);
+     }
+}
+
+static void *
+_signal_unlisten_hook(Evas_Object *obj, const char *emission, const char *source, void (*func_cb) (void *data, Evas_Object *o, const char *emission, const char *source))
+{
+   Widget_Data *wd = elm_widget_data_get(obj);
+   int i;
+   if (!wd) return NULL;
+   for (i = 0; i < 6; i++)
+     {
+	edje_object_signal_callback_del(wd->digit[i], emission, source,
+	      func_cb);
+     }
+   return edje_object_signal_callback_del(wd->clk, emission, source, func_cb);
+}
+
+static Eina_Bool
 _ticker(void *data)
 {
    Widget_Data *wd = elm_widget_data_get(data);
@@ -68,7 +134,7 @@ _ticker(void *data)
    struct timeval timev;
    struct tm *tm;
    time_t tt;
-
+   if (!wd) return ECORE_CALLBACK_CANCEL;
    gettimeofday(&timev, NULL);
    t = ((double)(1000000 - timev.tv_usec)) / 1000000.0;
    wd->ticker = ecore_timer_add(t, _ticker, data);
@@ -85,97 +151,145 @@ _ticker(void *data)
 	     _time_update(data);
 	  }
      }
-   return 0;
+   return ECORE_CALLBACK_CANCEL;
 }
 
-static void
-_signal_clock_val_up(void *data, Evas_Object *obj, const char *emission, const char *source)
+static Eina_Bool
+_signal_clock_val_up(void *data)
 {
    Widget_Data *wd = elm_widget_data_get(data);
-
-   if (!wd->edit) return;
-   if (obj == wd->digit[0])
+   if (!wd) goto clock_val_up_cancel;
+   if (!wd->edit) goto clock_val_up_cancel;
+   if (!wd->sel_obj) goto clock_val_up_cancel;
+   if (wd->sel_obj == wd->digit[0])
      {
 	wd->hrs = wd->hrs + 10;
 	if (wd->hrs >= 24) wd->hrs -= 24;
      }
-   if (obj == wd->digit[1])
+   if (wd->sel_obj == wd->digit[1])
      {
 	wd->hrs = wd->hrs + 1;
 	if (wd->hrs >= 24) wd->hrs -= 24;
      }
-   if (obj == wd->digit[2])
+   if (wd->sel_obj == wd->digit[2])
      {
 	wd->min = wd->min + 10;
 	if (wd->min >= 60) wd->min -= 60;
      }
-   if (obj == wd->digit[3])
+   if (wd->sel_obj == wd->digit[3])
      {
 	wd->min = wd->min + 1;
 	if (wd->min >= 60) wd->min -= 60;
      }
-   if (obj == wd->digit[4])
+   if (wd->sel_obj == wd->digit[4])
      {
 	wd->sec = wd->sec + 10;
 	if (wd->sec >= 60) wd->sec -= 60;
      }
-   if (obj == wd->digit[5])
+   if (wd->sel_obj == wd->digit[5])
      {
 	wd->sec = wd->sec + 1;
 	if (wd->sec >= 60) wd->sec -= 60;
      }
-   if (obj == wd->ampm)
+   if (wd->sel_obj == wd->ampm)
      {
 	wd->hrs = wd->hrs + 12;
 	if (wd->hrs > 23) wd->hrs -= 24;
      }
+   wd->interval = wd->interval / 1.05;
+   ecore_timer_interval_set(wd->spin, wd->interval);
    _time_update(data);
    evas_object_smart_callback_call(data, "changed", NULL);
+   return ECORE_CALLBACK_RENEW;
+clock_val_up_cancel:
+   wd->spin = NULL;
+   return ECORE_CALLBACK_CANCEL;
 }
 
-static void
-_signal_clock_val_down(void *data, Evas_Object *obj, const char *emission, const char *source)
+static Eina_Bool
+_signal_clock_val_down(void *data)
 {
    Widget_Data *wd = elm_widget_data_get(data);
-
-   if (!wd->edit) return;
-   if (obj == wd->digit[0])
+   if (!wd) goto clock_val_down_cancel;
+   if (!wd->edit) goto clock_val_down_cancel;
+   if (!wd->sel_obj) goto clock_val_down_cancel;
+   if (wd->sel_obj == wd->digit[0])
      {
 	wd->hrs = wd->hrs - 10;
 	if (wd->hrs < 0) wd->hrs += 24;
      }
-   if (obj == wd->digit[1])
+   if (wd->sel_obj == wd->digit[1])
      {
 	wd->hrs = wd->hrs - 1;
 	if (wd->hrs < 0) wd->hrs += 24;
      }
-   if (obj == wd->digit[2])
+   if (wd->sel_obj == wd->digit[2])
      {
 	wd->min = wd->min - 10;
 	if (wd->min < 0) wd->min += 60;
      }
-   if (obj == wd->digit[3])
+   if (wd->sel_obj == wd->digit[3])
      {
 	wd->min = wd->min - 1;
 	if (wd->min < 0) wd->min += 60;
      }
-   if (obj == wd->digit[4])
+   if (wd->sel_obj == wd->digit[4])
      {
 	wd->sec = wd->sec - 10;
 	if (wd->sec < 0) wd->sec += 60;
      }
-   if (obj == wd->digit[5])
+   if (wd->sel_obj == wd->digit[5])
      {
 	wd->sec = wd->sec - 1;
 	if (wd->sec < 0) wd->sec += 60;
      }
-   if (obj == wd->ampm)
+   if (wd->sel_obj == wd->ampm)
      {
 	wd->hrs = wd->hrs - 12;
 	if (wd->hrs < 0) wd->hrs += 24;
      }
+   wd->interval = wd->interval / 1.05;
+   ecore_timer_interval_set(wd->spin, wd->interval);
    _time_update(data);
    evas_object_smart_callback_call(data, "changed", NULL);
+   return ECORE_CALLBACK_RENEW;
+clock_val_down_cancel:
+   wd->spin = NULL;
+   return ECORE_CALLBACK_CANCEL;
+}
+
+static void
+_signal_clock_val_up_start(void *data, Evas_Object *obj, const char *emission __UNUSED__, const char *source __UNUSED__)
+{
+   Widget_Data *wd = elm_widget_data_get(data);
+   if (!wd) return;
+   wd->interval = 0.85;
+   wd->sel_obj = obj;
+   if (wd->spin) ecore_timer_del(wd->spin);
+   wd->spin = ecore_timer_add(wd->interval, _signal_clock_val_up, data);
+   _signal_clock_val_up(data);
+}
+
+static void
+_signal_clock_val_down_start(void *data, Evas_Object *obj, const char *emission __UNUSED__, const char *source __UNUSED__)
+{
+   Widget_Data *wd = elm_widget_data_get(data);
+   if (!wd) return;
+   wd->interval = 0.85;
+   wd->sel_obj = obj;
+   if (wd->spin) ecore_timer_del(wd->spin);
+   wd->spin = ecore_timer_add(wd->interval, _signal_clock_val_down, data);
+   _signal_clock_val_down(data);
+}
+
+static void
+_signal_clock_val_change_stop(void *data, Evas_Object *obj __UNUSED__, const char *emission __UNUSED__, const char *source __UNUSED__)
+{
+   Widget_Data *wd = elm_widget_data_get(data);
+   if (!wd) return;
+   if (wd->spin) ecore_timer_del(wd->spin);
+   wd->spin = NULL;
+   wd->sel_obj = NULL;
 }
 
 static void
@@ -185,9 +299,9 @@ _time_update(Evas_Object *obj)
    Edje_Message_Int msg;
    int ampm = 0;
    const char *style = elm_widget_style_get(obj);
-
+   if (!wd) return;
    if ((wd->cur.seconds != wd->seconds) || (wd->cur.am_pm != wd->am_pm) ||
-       (wd->cur.edit != wd->edit))
+       (wd->cur.edit != wd->edit) || (wd->cur.digedit != wd->digedit))
      {
 	int i;
 	Evas_Coord mw, mh;
@@ -207,13 +321,13 @@ _time_update(Evas_Object *obj)
 	  }
 
 	if ((wd->seconds) && (wd->am_pm))
-	  _elm_theme_set(wd->clk, "clock", "base-all", style);
+	  _elm_theme_object_set(obj, wd->clk, "clock", "base-all", style);
 	else if (wd->seconds)
-	  _elm_theme_set(wd->clk, "clock", "base-seconds", style);
+	  _elm_theme_object_set(obj, wd->clk, "clock", "base-seconds", style);
 	else if (wd->am_pm)
-	  _elm_theme_set(wd->clk, "clock", "base-am_pm", style);
+	  _elm_theme_object_set(obj, wd->clk, "clock", "base-am_pm", style);
 	else
-	  _elm_theme_set(wd->clk, "clock", "base", style);
+	  _elm_theme_object_set(obj, wd->clk, "clock", "base", style);
 	edje_object_scale_set(wd->clk, elm_widget_scale_get(obj) * 
                               _elm_config->scale);
 
@@ -223,15 +337,19 @@ _time_update(Evas_Object *obj)
 
 	     if ((!wd->seconds) && (i >= 4)) break;
 	     wd->digit[i] = edje_object_add(evas_object_evas_get(wd->clk));
-	     _elm_theme_set(wd->digit[i], "clock", "flipdigit", style);
+	     _elm_theme_object_set(obj, wd->digit[i], "clock", "flipdigit", style);
 	     edje_object_scale_set(wd->digit[i], elm_widget_scale_get(obj) * 
                                    _elm_config->scale);
-	     if (wd->edit)
+	     if (wd->edit && (wd->digedit & (1 << i)))
 	       edje_object_signal_emit(wd->digit[i], "elm,state,edit,on", "elm");
-	     edje_object_signal_callback_add(wd->digit[i], "elm,action,up", "",
-					     _signal_clock_val_up, obj);
-	     edje_object_signal_callback_add(wd->digit[i], "elm,action,down", "",
-					     _signal_clock_val_down, obj);
+	     edje_object_signal_callback_add(wd->digit[i], "elm,action,up,start",
+					     "", _signal_clock_val_up_start, obj);
+	     edje_object_signal_callback_add(wd->digit[i], "elm,action,up,stop",
+					     "", _signal_clock_val_change_stop, obj);
+	     edje_object_signal_callback_add(wd->digit[i], "elm,action,down,start",
+					     "", _signal_clock_val_down_start, obj);
+	     edje_object_signal_callback_add(wd->digit[i], "elm,action,down,stop",
+					     "", _signal_clock_val_change_stop, obj);
 	     mw = mh = -1;
 	     elm_coords_finger_size_adjust(1, &mw, 2, &mh);
 	     edje_object_size_min_restricted_calc(wd->digit[i], &mw, &mh, mw, mh);
@@ -244,15 +362,19 @@ _time_update(Evas_Object *obj)
 	if (wd->am_pm)
 	  {
 	     wd->ampm = edje_object_add(evas_object_evas_get(wd->clk));
-	     _elm_theme_set(wd->ampm, "clock", "flipampm", style);
+	     _elm_theme_object_set(obj, wd->ampm, "clock", "flipampm", style);
 	     edje_object_scale_set(wd->ampm, elm_widget_scale_get(obj) * 
                                    _elm_config->scale);
 	     if (wd->edit)
 	       edje_object_signal_emit(wd->ampm, "elm,state,edit,on", "elm");
-	     edje_object_signal_callback_add(wd->ampm, "elm,action,up", "",
-					     _signal_clock_val_up, obj);
-	     edje_object_signal_callback_add(wd->ampm, "elm,action,down", "",
-					     _signal_clock_val_down, obj);
+	     edje_object_signal_callback_add(wd->ampm, "elm,action,up,start",
+					     "", _signal_clock_val_up_start, obj);
+	     edje_object_signal_callback_add(wd->ampm, "elm,action,up,stop",
+					     "", _signal_clock_val_change_stop, obj);
+	     edje_object_signal_callback_add(wd->ampm, "elm,action,down,start",
+					     "", _signal_clock_val_down_start, obj);
+	     edje_object_signal_callback_add(wd->ampm, "elm,action,down,stop",
+					     "", _signal_clock_val_change_stop, obj);
 	     mw = mh = -1;
 	     elm_coords_finger_size_adjust(1, &mw, 2, &mh);
 	     edje_object_size_min_restricted_calc(wd->ampm, &mw, &mh, mw, mh);
@@ -272,6 +394,7 @@ _time_update(Evas_Object *obj)
 	wd->cur.seconds = wd->seconds;
 	wd->cur.am_pm = wd->am_pm;
 	wd->cur.edit = wd->edit;
+	wd->cur.digedit = wd->digedit;
      }
    if (wd->hrs != wd->cur.hrs)
      {
@@ -355,8 +478,6 @@ _time_update(Evas_Object *obj)
 	if (wd->hrs >= 12) ampm = 1;
 	if (ampm != wd->cur.ampm)
 	  {
-	     int d1, d2, dc1, dc2;
-
 	     if (wd->cur.ampm != ampm)
 	       {
 		  msg.val = ampm;
@@ -389,11 +510,16 @@ elm_clock_add(Evas_Object *parent)
    wd = ELM_NEW(Widget_Data);
    e = evas_object_evas_get(parent);
    obj = elm_widget_add(e);
+   ELM_SET_WIDTYPE(widtype, "clock");
    elm_widget_type_set(obj, "clock");
    elm_widget_sub_object_add(parent, obj);
    elm_widget_data_set(obj, wd);
    elm_widget_del_hook_set(obj, _del_hook);
    elm_widget_theme_hook_set(obj, _theme_hook);
+   elm_widget_on_focus_hook_set(obj, _on_focus_hook, NULL);
+   elm_widget_signal_emit_hook_set(obj, _signal_emit_hook);
+   elm_widget_signal_listen_hook_set(obj, _signal_listen_hook);
+   elm_widget_signal_unlisten_hook_set(obj, _signal_unlisten_hook);
 
    wd->clk = edje_object_add(e);
    elm_widget_resize_object_set(obj, wd->clk);
@@ -402,6 +528,7 @@ elm_clock_add(Evas_Object *parent)
    wd->cur.seconds = EINA_TRUE;
    wd->cur.am_pm = EINA_TRUE;
    wd->cur.edit = EINA_TRUE;
+   wd->cur.digedit = ELM_CLOCK_NONE;
 
    _time_update(obj);
    _ticker(obj);
@@ -424,7 +551,9 @@ elm_clock_add(Evas_Object *parent)
 EAPI void
 elm_clock_time_set(Evas_Object *obj, int hrs, int min, int sec)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    wd->hrs = hrs;
    wd->min = min;
    wd->sec = sec;
@@ -434,7 +563,7 @@ elm_clock_time_set(Evas_Object *obj, int hrs, int min, int sec)
 /**
  * Get clock time
  *
- * @param obj The clock object 
+ * @param obj The clock object
  * @param hrs Pointer to the variable to get the hour of this clock
  * object
  * @param min Pointer to the variable to get the minute of this clock
@@ -450,7 +579,9 @@ elm_clock_time_set(Evas_Object *obj, int hrs, int min, int sec)
 EAPI void
 elm_clock_time_get(const Evas_Object *obj, int *hrs, int *min, int *sec)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    if (hrs) *hrs = wd->hrs;
    if (min) *min = wd->min;
    if (sec) *sec = wd->sec;
@@ -463,19 +594,86 @@ elm_clock_time_get(const Evas_Object *obj, int *hrs, int *min, int *sec)
  * @param edit Bool option for edited (1 = yes, 0 = no)
  *
  * This function sets if the clock settings can be edited or not.
+ * By default or if digit_edit option was previously set to ELM_CLOCK_NONE,
+ * all digits are editable. To choose what digits to make editable
+ * use elm_clock_digit_edit_set().
  *
  * @ingroup Clock
  */
 EAPI void
 elm_clock_edit_set(Evas_Object *obj, Eina_Bool edit)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    wd->edit = edit;
-   _time_update(obj);
+   if (edit && (wd->digedit == ELM_CLOCK_NONE))
+     elm_clock_digit_edit_set(obj, ELM_CLOCK_ALL);
+   else
+     _time_update(obj);
 }
 
 /**
- * Set if the clock show hours in military or am/pm mode
+ * Get if the clock settings can be edited
+ *
+ * @param obj The clock object
+ * @return Bool option for edited (1 = yes, 0 = no)
+ *
+ * This function gets if the clock settings can be edited or not.
+ *
+ * @ingroup Clock
+ */
+EAPI Eina_Bool
+elm_clock_edit_get(const Evas_Object *obj)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype) EINA_FALSE;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return EINA_FALSE;
+   return wd->edit;
+}
+
+/**
+ * Set what digits of the clock are editable
+ *
+ * @param obj The clock object
+ * @param digedit Bit mask indicating the digits to edit
+ *
+ * If the digedit param is ELM_CLOCK_NONE, editing will be disabled.
+ *
+ * @ingroup Clock
+ */
+EAPI void
+elm_clock_digit_edit_set(Evas_Object *obj, Elm_Clock_Digedit digedit)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype);
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
+   wd->digedit = digedit;
+   if (digedit == ELM_CLOCK_NONE)
+     elm_clock_edit_set(obj, EINA_FALSE);
+   else
+     _time_update(obj);
+}
+
+/**
+ * Get what digits of the clock are editable
+ *
+ * @param obj The clock object
+ * @return Bit mask indicating the digits.
+ *
+ * @ingroup Clock
+ */
+EAPI Elm_Clock_Digedit
+elm_clock_digit_edit_get(const Evas_Object *obj)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype) 0;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return 0;
+   return wd->digedit;
+}
+
+/**
+ * Set if the clock shows hours in military or am/pm mode
  *
  * @param obj The clock object
  * @param am_pm Bool option for the hours mode
@@ -491,13 +689,38 @@ elm_clock_edit_set(Evas_Object *obj, Eina_Bool edit)
 EAPI void
 elm_clock_show_am_pm_set(Evas_Object *obj, Eina_Bool am_pm)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    wd->am_pm = am_pm;
    _time_update(obj);
 }
 
 /**
- * Set if the clock show hour with the seconds
+ * Get if the clock shows hours in military or am/pm mode
+ *
+ * @param obj The clock object
+ * @return Bool option for the hours mode
+ * (1 = am/pm, 0 = military)
+ *
+ * This function gets if the clock show hours in military or am/pm
+ * mode. Some countries like Brazil the military mode (00-24h-format)
+ * is used in opposition to the USA where the am/pm mode is more
+ * common used.
+ *
+ * @ingroup Clock
+ */
+EAPI Eina_Bool
+elm_clock_show_am_pm_get(const Evas_Object *obj)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype) EINA_FALSE;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return EINA_FALSE;
+   return wd->am_pm;
+}
+
+/**
+ * Set if the clock shows hour with the seconds
  *
  * @param obj The clock object
  * @param seconds Bool option for the show seconds
@@ -511,7 +734,30 @@ elm_clock_show_am_pm_set(Evas_Object *obj, Eina_Bool am_pm)
 EAPI void
 elm_clock_show_seconds_set(Evas_Object *obj, Eina_Bool seconds)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    wd->seconds = seconds;
    _time_update(obj);
 }
+
+/**
+ * Get if the clock shows hour with the seconds
+ *
+ * @param obj The clock object
+ * @return Bool option for the show seconds
+ * (1 = show seconds, 0 = not show seconds)
+ *
+ * This function gets if the clock show or not show the elapsed
+ * seconds.
+ *
+ * @ingroup Clock
+ */
+EAPI Eina_Bool
+elm_clock_show_seconds_get(const Evas_Object *obj)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype) EINA_FALSE;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return EINA_FALSE;
+   return wd->seconds;
+}
