@@ -27,6 +27,7 @@ struct _Widget_Data
    Eina_Bool no_scale : 1;
 };
 
+static const char *widtype = NULL;
 static void _del_hook(Evas_Object *obj);
 static void _theme_hook(Evas_Object *obj);
 static void _sizing_eval(Evas_Object *obj);
@@ -46,10 +47,9 @@ static void
 _theme_hook(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
-
    if (!wd) return;
    if (wd->stdicon)
-     _elm_theme_icon_set(wd->img, wd->stdicon, "default");
+     _elm_theme_object_icon_set(obj, wd->img, wd->stdicon, "default");
    _sizing_eval(obj);
 }
 
@@ -88,8 +88,10 @@ _sizing_eval(Evas_Object *obj)
 }
 
 static void
-_mouse_up(void *data, Evas *e, Evas_Object *obj, void *event_info)
+_mouse_up(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
 {
+   Evas_Event_Mouse_Up *ev = event_info;
+   if (ev->event_flags & EVAS_EVENT_FLAG_ON_HOLD) return;
    evas_object_smart_callback_call(data, "clicked", event_info);
 }
 
@@ -111,6 +113,7 @@ elm_icon_add(Evas_Object *parent)
    wd = ELM_NEW(Widget_Data);
    e = evas_object_evas_get(parent);
    obj = elm_widget_add(e);
+   ELM_SET_WIDTYPE(widtype, "icon");
    elm_widget_type_set(obj, "icon");
    elm_widget_sub_object_add(parent, obj);
    elm_widget_data_set(obj, wd);
@@ -146,11 +149,12 @@ elm_icon_add(Evas_Object *parent)
 EAPI Eina_Bool
 elm_icon_file_set(Evas_Object *obj, const char *file, const char *group)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) EINA_FALSE;
    Widget_Data *wd = elm_widget_data_get(obj);
    Eina_Bool ret;
    const char *p;
 
-   if ((!wd) || (!file)) return 0;
+   if ((!wd) || (!file)) return EINA_FALSE;
    if (wd->stdicon) eina_stringshare_del(wd->stdicon);
    wd->stdicon = NULL;
    if (((p = strrchr(file, '.'))) && (!strcasecmp(p, ".edj")))
@@ -167,18 +171,22 @@ elm_icon_file_set(Evas_Object *obj, const char *file, const char *group)
  * @param obj The icon object
  * @param name The theme name
  *
+ * @return (1 = sucess, 0 = error)
+ *
  * @ingroup Icon
  */
-EAPI void
+EAPI Eina_Bool
 elm_icon_standard_set(Evas_Object *obj, const char *name)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) EINA_FALSE;
    Widget_Data *wd = elm_widget_data_get(obj);
+   Eina_Bool ret;
 
-   if ((!wd) || (!name)) return;
-   if (wd->stdicon) eina_stringshare_del(wd->stdicon);
-   wd->stdicon = eina_stringshare_add(name);
-   _elm_theme_icon_set(wd->img, name, "default");
+   if ((!wd) || (!name)) return EINA_FALSE;
+   eina_stringshare_replace(&wd->stdicon, name);
+   ret = _elm_theme_object_icon_set(obj, wd->img, name, "default");
    _sizing_eval(obj);
+   return ret;
 }
 
 /**
@@ -193,6 +201,7 @@ elm_icon_standard_set(Evas_Object *obj, const char *name)
 EAPI void
 elm_icon_smooth_set(Evas_Object *obj, Eina_Bool smooth)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
 
    if (!wd) return;
@@ -212,6 +221,7 @@ elm_icon_smooth_set(Evas_Object *obj, Eina_Bool smooth)
 EAPI void
 elm_icon_no_scale_set(Evas_Object *obj, Eina_Bool no_scale)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
 
    if (!wd) return;
@@ -231,6 +241,7 @@ elm_icon_no_scale_set(Evas_Object *obj, Eina_Bool no_scale)
 EAPI void
 elm_icon_scale_set(Evas_Object *obj, Eina_Bool scale_up, Eina_Bool scale_down)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
 
    if (!wd) return;
@@ -251,6 +262,7 @@ elm_icon_scale_set(Evas_Object *obj, Eina_Bool scale_up, Eina_Bool scale_down)
 EAPI void
 elm_icon_fill_outside_set(Evas_Object *obj, Eina_Bool fill_outside)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
 
    if (!wd) return;
@@ -270,6 +282,7 @@ elm_icon_fill_outside_set(Evas_Object *obj, Eina_Bool fill_outside)
 EAPI void
 elm_icon_prescale_set(Evas_Object *obj, int size)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
 
    if (!wd) return;
