@@ -1,6 +1,13 @@
 #include <Elementary.h>
 #include "elm_priv.h"
 
+/**
+ * @defgroup List List
+ *
+ * A list is a very simple type of list widget.  For more robust
+ * lists, @ref Genlist should probably be used.
+ */
+
 typedef struct _Widget_Data Widget_Data;
 
 struct _Widget_Data
@@ -24,8 +31,8 @@ struct _Elm_List_Item
    Evas_Object *obj, *base;
    const char *label;
    Evas_Object *icon, *end;
-   void (*func) (void *data, Evas_Object *obj, void *event_info);
-   void (*del_cb) (void *data, Evas_Object *obj, void *event_info);
+   Evas_Smart_Cb func;
+   Evas_Smart_Cb del_cb;
    const void *data;
    Ecore_Timer *long_timer;
    Eina_Bool deleted : 1;
@@ -38,7 +45,9 @@ struct _Elm_List_Item
    Eina_Bool dummy_end : 1;
 };
 
+static const char *widtype = NULL;
 static void _del_hook(Evas_Object *obj);
+static void _theme_hook(Evas_Object *obj);
 static void _sizing_eval(Evas_Object *obj);
 static void _on_focus_hook(void *data, Evas_Object *obj);
 static void _changed_size_hints(void *data, Evas *e, Evas_Object *obj, void *event_info);
@@ -48,12 +57,18 @@ static void _mouse_down(void *data, Evas *evas, Evas_Object *obj, void *event_in
 static void _mouse_up(void *data, Evas *evas, Evas_Object *obj, void *event_info);
 static void _mouse_move(void *data, Evas *evas, Evas_Object *obj, void *event_info);
 
-#define ELM_LIST_ITEM_CHECK_DELETED_RETURN(it, ...)		\
-  if ((it) && ((it)->deleted))					\
-    {								\
-       fprintf(stderr, "ERROR: %s:%d:%s() "#it" is NULL.\n",	\
-	       __FILE__, __LINE__, __FUNCTION__);		\
-       return __VA_ARGS__;					\
+#define ELM_LIST_ITEM_CHECK_DELETED_RETURN(it, ...)			\
+  if (!it)								\
+    {									\
+       fprintf(stderr, "ERROR: %s:%d:%s() "#it" is NULL.\n",		\
+	       __FILE__, __LINE__, __FUNCTION__);			\
+       return __VA_ARGS__;						\
+    }								 	\
+  else if (it->deleted)						\
+    {									\
+       fprintf(stderr, "ERROR: %s:%d:%s() "#it" has been DELETED.\n",	\
+	       __FILE__, __LINE__, __FUNCTION__);			\
+       return __VA_ARGS__;						\
     }
 
 
@@ -153,23 +168,17 @@ _del_hook(Evas_Object *obj)
    Elm_List_Item *it;
    Eina_List *n;
 
+   if (!wd) return;
    if (wd->walking != 0)
      fprintf(stderr, "ERROR: list deleted while walking.\n");
 
    _elm_list_walk(wd);
-
-   EINA_LIST_FOREACH(wd->items, n, it)
-     _elm_list_item_call_del_cb(it);
-
+   EINA_LIST_FOREACH(wd->items, n, it) _elm_list_item_call_del_cb(it);
    _elm_list_unwalk(wd);
-   if (wd->to_delete)
-     fprintf(stderr, "ERROR: leaking nodes!\n");
-
-   EINA_LIST_FREE(wd->items, it)
-     _elm_list_item_free(it);
+   if (wd->to_delete) fprintf(stderr, "ERROR: leaking nodes!\n");
 
+   EINA_LIST_FREE(wd->items, it) _elm_list_item_free(it);
    eina_list_free(wd->selected);
-
    free(wd);
 }
 
@@ -178,55 +187,91 @@ _sizing_eval(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    Evas_Coord minw = -1, minh = -1, maxw = -1, maxh = -1;
-
-   evas_object_size_hint_min_get(wd->scr, &minw, &minh);
-   evas_object_size_hint_max_get(wd->scr, &maxw, &maxh);
-   evas_object_size_hint_min_set(obj, minw, minh);
-   evas_object_size_hint_max_set(obj, maxw, maxh);
+   if (!wd) return;
+   if (wd->scr)
+     {
+        evas_object_size_hint_min_get(wd->scr, &minw, &minh);
+        evas_object_size_hint_max_get(wd->scr, &maxw, &maxh);
+        evas_object_size_hint_min_set(obj, minw, minh);
+        evas_object_size_hint_max_set(obj, maxw, maxh);
+     }
 }
 
 static void
-_on_focus_hook(void *data, Evas_Object *obj)
+_theme_hook(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
-//   if (elm_widget_focus_get(obj))
-//     elm_widget_focus_steal(wd->entry);
+   Elm_List_Item *it;
+   Eina_List *n;
+   
+   if (!wd) return;
+   if (wd->scr)
+     {
+        elm_scroller_custom_widget_base_theme_set(wd->scr, "list", "base");
+        elm_object_style_set(wd->scr, elm_widget_style_get(obj));
+//        edje_object_scale_set(wd->scr, elm_widget_scale_get(obj) * _elm_config->scale);
+     }
+   EINA_LIST_FOREACH(wd->items, n, it)
+     {
+        it->fixed = 0;
+     }
+   _fix_items(obj);
+   _sizing_eval(obj);
 }
 
 static void
-_changed_size_hints(void *data, Evas *e, Evas_Object *obj, void *event_info)
+_on_focus_hook(void *data __UNUSED__, Evas_Object *obj)
+{
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
+   if (elm_widget_focus_get(obj))
+     evas_object_focus_set(wd->scr, 1);
+   else
+     evas_object_focus_set(wd->scr, 0);
+}
+
+static void
+_changed_size_hints(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Widget_Data *wd = elm_widget_data_get(data);
+   if (!wd) return;
 //   _fix_items(data);
 //   _sizing_eval(data);
 }
 
 static void
-_sub_del(void *data, Evas_Object *obj, void *event_info)
+_sub_del(void *data __UNUSED__, Evas_Object *obj, void *event_info)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    Evas_Object *sub = event_info;
    const Eina_List *l;
    Elm_List_Item *it;
 
-   EINA_LIST_FOREACH(wd->items, l, it)
+   if (!wd) return;
+   if (sub == NULL) abort();
+   if (sub == wd->scr)
+     wd->scr = NULL;
+   else
      {
-	if ((sub == it->icon) || (sub == it->end))
-	  {
-	     if (it->icon == sub) it->icon = NULL;
-	     if (it->end == sub) it->end = NULL;
-	     evas_object_event_callback_del_full(sub,
-                                            EVAS_CALLBACK_CHANGED_SIZE_HINTS,
-                                            _changed_size_hints, obj);
-	     if (!wd->walking)
-	       {
-		  _fix_items(obj);
-		  _sizing_eval(obj);
-	       }
-	     else
-	       wd->fix_pending = EINA_TRUE;
-	     break;
-	  }
+        EINA_LIST_FOREACH(wd->items, l, it)
+          {
+             if ((sub == it->icon) || (sub == it->end))
+               {
+                  if (it->icon == sub) it->icon = NULL;
+                  if (it->end == sub) it->end = NULL;
+                  evas_object_event_callback_del_full
+                    (sub, EVAS_CALLBACK_CHANGED_SIZE_HINTS, _changed_size_hints, 
+                     obj);
+                  if (!wd->walking)
+                    {
+                       _fix_items(obj);
+                       _sizing_eval(obj);
+                    }
+                  else
+                    wd->fix_pending = EINA_TRUE;
+                  break;
+               }
+          }
      }
 }
 
@@ -236,6 +281,7 @@ _item_hilight(Elm_List_Item *it)
    Widget_Data *wd = elm_widget_data_get(it->obj);
    const char *selectraise;
 
+   if (!wd) return;
    ELM_LIST_ITEM_CHECK_DELETED_RETURN(it);
    if (it->hilighted) return;
    _elm_list_walk(wd);
@@ -253,8 +299,8 @@ static void
 _item_select(Elm_List_Item *it)
 {
    Widget_Data *wd = elm_widget_data_get(it->obj);
-   const char *selectraise;
 
+   if (!wd) return;
    ELM_LIST_ITEM_CHECK_DELETED_RETURN(it);
    if (it->selected)
      {
@@ -278,6 +324,7 @@ _item_unselect(Elm_List_Item *it)
    Widget_Data *wd = elm_widget_data_get(it->obj);
    const char *stacking, *selectraise;
 
+   if (!wd) return;
    ELM_LIST_ITEM_CHECK_DELETED_RETURN(it);
    if (!it->hilighted) return;
    _elm_list_walk(wd);
@@ -302,14 +349,14 @@ _item_unselect(Elm_List_Item *it)
 }
 
 static void
-_mouse_move(void *data, Evas *evas, Evas_Object *obj, void *event_info)
+_mouse_move(void *data, Evas *evas __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
 {
    Elm_List_Item *it = data;
    Widget_Data *wd = elm_widget_data_get(it->obj);
    Evas_Event_Mouse_Move *ev = event_info;
 
+   if (!wd) return;
    ELM_LIST_ITEM_CHECK_DELETED_RETURN(it);
-
    if (ev->event_flags & EVAS_EVENT_FLAG_ON_HOLD)
      {
 	if (!wd->on_hold)
@@ -325,30 +372,29 @@ _mouse_move(void *data, Evas *evas, Evas_Object *obj, void *event_info)
      }
 }
 
-static int
+static Eina_Bool
 _long_press(void *data)
 {
    Elm_List_Item *it = data;
    Widget_Data *wd = elm_widget_data_get(it->obj);
 
+   if (!wd) return ECORE_CALLBACK_CANCEL;
    it->long_timer = NULL;
-
    ELM_LIST_ITEM_CHECK_DELETED_RETURN(it, 0);
-
    wd->longpressed = EINA_TRUE;
    evas_object_smart_callback_call(it->obj, "longpressed", it);
-   return 0;
+   return ECORE_CALLBACK_CANCEL;
 }
 
 static void
-_mouse_down(void *data, Evas *evas, Evas_Object *obj, void *event_info)
+_mouse_down(void *data, Evas *evas __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
 {
    Elm_List_Item *it = data;
    Widget_Data *wd = elm_widget_data_get(it->obj);
    Evas_Event_Mouse_Down *ev = event_info;
 
+   if (!wd) return;
    ELM_LIST_ITEM_CHECK_DELETED_RETURN(it);
-
    if (ev->button != 1) return;
    if (ev->event_flags & EVAS_EVENT_FLAG_ON_HOLD) wd->on_hold = EINA_TRUE;
    else wd->on_hold = EINA_FALSE;
@@ -363,17 +409,18 @@ _mouse_down(void *data, Evas *evas, Evas_Object *obj, void *event_info)
 }
 
 static void
-_mouse_up(void *data, Evas *evas, Evas_Object *obj, void *event_info)
+_mouse_up(void *data, Evas *evas __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
 {
    Elm_List_Item *it = data;
    Widget_Data *wd = elm_widget_data_get(it->obj);
    Evas_Event_Mouse_Up *ev = event_info;
 
+   if (!wd) return;
    ELM_LIST_ITEM_CHECK_DELETED_RETURN(it);
-
    if (ev->button != 1) return;
    if (ev->event_flags & EVAS_EVENT_FLAG_ON_HOLD) wd->on_hold = EINA_TRUE;
    else wd->on_hold = EINA_FALSE;
+   wd->longpressed = EINA_FALSE;
    if (it->long_timer)
      {
         ecore_timer_del(it->long_timer);
@@ -386,7 +433,6 @@ _mouse_up(void *data, Evas *evas, Evas_Object *obj, void *event_info)
      }
    if (wd->longpressed)
      {
-        wd->longpressed = EINA_FALSE;
         if (!wd->wasselected) _item_unselect(it);
         wd->wasselected = 0;
         return;
@@ -428,11 +474,12 @@ _mouse_up(void *data, Evas *evas, Evas_Object *obj, void *event_info)
 }
 
 static Elm_List_Item *
-_item_new(Evas_Object *obj, const char *label, Evas_Object *icon, Evas_Object *end, void (*func) (void *data, Evas_Object *obj, void *event_info), const void *data)
+_item_new(Evas_Object *obj, const char *label, Evas_Object *icon, Evas_Object *end, Evas_Smart_Cb func, const void *data)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    Elm_List_Item *it;
 
+   if (!wd) return NULL;
    it = calloc(1, sizeof(Elm_List_Item));
    it->obj = obj;
    it->label = eina_stringshare_add(label);
@@ -475,6 +522,7 @@ _fix_items(Evas_Object *obj)
    int i, redo = 0;
    const char *style = elm_widget_style_get(obj);
 
+   if (!wd) return;
    if (wd->walking)
      {
 	wd->fix_pending = EINA_TRUE;
@@ -485,7 +533,6 @@ _fix_items(Evas_Object *obj)
 
    EINA_LIST_FOREACH(wd->items, l, it)
      {
-	Evas_Coord mw, mh;
 	if (it->deleted) continue;
 	if (it->icon)
 	  {
@@ -521,16 +568,16 @@ _fix_items(Evas_Object *obj)
 	     if (wd->mode == ELM_LIST_COMPRESS)
 	       {
 		  if (it->even)
-		    _elm_theme_set(it->base, "list", "item_compress", style);
+		    _elm_theme_object_set(obj, it->base, "list", "item_compress", style);
 		  else
-		    _elm_theme_set(it->base, "list", "item_compress_odd", style);
+		    _elm_theme_object_set(obj, it->base, "list", "item_compress_odd", style);
 	       }
 	     else
 	       {
 		  if (it->even)
-		    _elm_theme_set(it->base, "list", "item", style);
+		    _elm_theme_object_set(obj, it->base, "list", "item", style);
 		  else
-		    _elm_theme_set(it->base, "list", "item_odd", style);
+		    _elm_theme_object_set(obj, it->base, "list", "item_odd", style);
 	       }
 	     stacking = edje_object_data_get(it->base, "stacking");
 	     if (stacking)
@@ -606,49 +653,60 @@ _fix_items(Evas_Object *obj)
 
    mw = 0; mh = 0;
    evas_object_size_hint_min_get(wd->box, &mw, &mh);
-   if (wd->mode == ELM_LIST_LIMIT)
-     elm_scroller_content_min_limit(wd->scr, 1, 0);
-   else
-     elm_scroller_content_min_limit(wd->scr, 0, 0);
+   if (wd->scr)
+     {
+        if (wd->mode == ELM_LIST_LIMIT)
+          elm_scroller_content_min_limit(wd->scr, 1, 0);
+        else
+          elm_scroller_content_min_limit(wd->scr, 0, 0);
+     }
    _sizing_eval(obj);
 }
 
 static void
-_hold_on(void *data, Evas_Object *obj, void *event_info)
+_hold_on(void *data __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
-
    if (!wd) return;
-   elm_widget_scroll_hold_push(wd->scr);
+   if (wd->scr)
+     elm_widget_scroll_hold_push(wd->scr);
 }
 
 static void
-_hold_off(void *data, Evas_Object *obj, void *event_info)
+_hold_off(void *data __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
-
    if (!wd) return;
-   elm_widget_scroll_hold_pop(wd->scr);
+   if (wd->scr)
+     elm_widget_scroll_hold_pop(wd->scr);
 }
 
 static void
-_freeze_on(void *data, Evas_Object *obj, void *event_info)
+_freeze_on(void *data __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
-
    if (!wd) return;
-   elm_widget_scroll_hold_push(wd->scr);
+   if (wd->scr)
+     elm_widget_scroll_hold_push(wd->scr);
 }
 
 static void
-_freeze_off(void *data, Evas_Object *obj, void *event_info)
+_freeze_off(void *data __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
-
    if (!wd) return;
-   elm_widget_scroll_hold_pop(wd->scr);
+   if (wd->scr)
+     elm_widget_scroll_hold_pop(wd->scr);
 }
 
+/**
+ * Adds a list object.
+ *
+ * @param parent The parent object
+ * @return The created object or NULL upon failure
+ *
+ * @ingroup List
+ */
 EAPI Evas_Object *
 elm_list_add(Evas_Object *parent)
 {
@@ -659,14 +717,17 @@ elm_list_add(Evas_Object *parent)
    wd = ELM_NEW(Widget_Data);
    e = evas_object_evas_get(parent);
    wd->self = obj = elm_widget_add(e);
+   ELM_SET_WIDTYPE(widtype, "list");
    elm_widget_type_set(obj, "list");
    elm_widget_sub_object_add(parent, obj);
    elm_widget_on_focus_hook_set(obj, _on_focus_hook, NULL);
    elm_widget_data_set(obj, wd);
    elm_widget_del_hook_set(obj, _del_hook);
+   elm_widget_theme_hook_set(obj, _theme_hook);
    elm_widget_can_focus_set(obj, 1);
 
    wd->scr = elm_scroller_add(parent);
+   elm_scroller_custom_widget_base_theme_set(wd->scr, "list", "base");
    elm_widget_resize_object_set(obj, wd->scr);
 
    elm_scroller_bounce_set(wd->scr, 0, 1);
@@ -690,9 +751,24 @@ elm_list_add(Evas_Object *parent)
    return obj;
 }
 
+/**
+ * Appends an item to the list object.
+ *
+ * @param obj The list object
+ * @param label The label of the list item
+ * @param icon The icon object to use for the left side of the item
+ * @param end The icon object to use for the right side of the item
+ * @param func The function to call when the item is clicked
+ * @param data The data to associate with the item for related callbacks
+ *
+ * @return The created item or NULL upon failure
+ *
+ * @ingroup List
+ */
 EAPI Elm_List_Item *
-elm_list_item_append(Evas_Object *obj, const char *label, Evas_Object *icon, Evas_Object *end, void (*func) (void *data, Evas_Object *obj, void *event_info), const void *data)
+elm_list_item_append(Evas_Object *obj, const char *label, Evas_Object *icon, Evas_Object *end, Evas_Smart_Cb func, const void *data)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
    Widget_Data *wd = elm_widget_data_get(obj);
    Elm_List_Item *it = _item_new(obj, label, icon, end, func, data);
 
@@ -702,9 +778,24 @@ elm_list_item_append(Evas_Object *obj, const char *label, Evas_Object *icon, Eva
    return it;
 }
 
+/**
+ * Prepends an item to the list object.
+ *
+ * @param obj The list object
+ * @param label The label of the list item
+ * @param icon The icon object to use for the left side of the item
+ * @param end The icon object to use for the right side of the item
+ * @param func The function to call when the item is clicked
+ * @param data The data to associate with the item for related callbacks
+ *
+ * @return The created item or NULL upon failure
+ *
+ * @ingroup List
+ */
 EAPI Elm_List_Item *
-elm_list_item_prepend(Evas_Object *obj, const char *label, Evas_Object *icon, Evas_Object *end, void (*func) (void *data, Evas_Object *obj, void *event_info), const void *data)
+elm_list_item_prepend(Evas_Object *obj, const char *label, Evas_Object *icon, Evas_Object *end, Evas_Smart_Cb func, const void *data)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
    Widget_Data *wd = elm_widget_data_get(obj);
    Elm_List_Item *it = _item_new(obj, label, icon, end, func, data);
 
@@ -714,8 +805,23 @@ elm_list_item_prepend(Evas_Object *obj, const char *label, Evas_Object *icon, Ev
    return it;
 }
 
+/**
+ * Inserts an item into the list object before @p before.
+ *
+ * @param obj The list object
+ * @param before The list item to insert before
+ * @param label The label of the list item
+ * @param icon The icon object to use for the left side of the item
+ * @param end The icon object to use for the right side of the item
+ * @param func The function to call when the item is clicked
+ * @param data The data to associate with the item for related callbacks
+ *
+ * @return The created item or NULL upon failure
+ *
+ * @ingroup List
+ */
 EAPI Elm_List_Item *
-elm_list_item_insert_before(Evas_Object *obj, Elm_List_Item *before, const char *label, Evas_Object *icon, Evas_Object *end, void (*func) (void *data, Evas_Object *obj, void *event_info), const void *data)
+elm_list_item_insert_before(Evas_Object *obj, Elm_List_Item *before, const char *label, Evas_Object *icon, Evas_Object *end, Evas_Smart_Cb func, const void *data)
 {
    Widget_Data *wd;
    Elm_List_Item *it;
@@ -723,7 +829,9 @@ elm_list_item_insert_before(Evas_Object *obj, Elm_List_Item *before, const char
    if ((!before) || (!before->node)) return NULL;
    ELM_LIST_ITEM_CHECK_DELETED_RETURN(before, NULL);
 
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
    wd = elm_widget_data_get(obj);
+   if (!wd) return NULL;
    it = _item_new(obj, label, icon, end, func, data);
    wd->items = eina_list_prepend_relative_list(wd->items, it, before->node);
    it->node = before->node->prev;
@@ -731,8 +839,23 @@ elm_list_item_insert_before(Evas_Object *obj, Elm_List_Item *before, const char
    return it;
 }
 
+/**
+ * Inserts an item into the list object after @p after.
+ *
+ * @param obj The list object
+ * @param after The list item to insert after
+ * @param label The label of the list item
+ * @param icon The icon object to use for the left side of the item
+ * @param end The icon object to use for the right side of the item
+ * @param func The function to call when the item is clicked
+ * @param data The data to associate with the item for related callbacks
+ *
+ * @return The created item or NULL upon failure
+ *
+ * @ingroup List
+ */
 EAPI Elm_List_Item *
-elm_list_item_insert_after(Evas_Object *obj, Elm_List_Item *after, const char *label, Evas_Object *icon, Evas_Object *end, void (*func) (void *data, Evas_Object *obj, void *event_info), const void *data)
+elm_list_item_insert_after(Evas_Object *obj, Elm_List_Item *after, const char *label, Evas_Object *icon, Evas_Object *end, Evas_Smart_Cb func, const void *data)
 {
    Widget_Data *wd;
    Elm_List_Item *it;
@@ -740,7 +863,9 @@ elm_list_item_insert_after(Evas_Object *obj, Elm_List_Item *after, const char *l
    if ((!after) || (!after->node)) return NULL;
    ELM_LIST_ITEM_CHECK_DELETED_RETURN(after, NULL);
 
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
    wd = elm_widget_data_get(obj);
+   if (!wd) return NULL;
    it = _item_new(obj, label, icon, end, func, data);
    wd->items = eina_list_append_relative_list(wd->items, it, after->node);
    it->node = after->node->next;
@@ -748,14 +873,62 @@ elm_list_item_insert_after(Evas_Object *obj, Elm_List_Item *after, const char *l
    return it;
 }
 
+/**
+ * Insert a new item into the sorted list object.
+ *
+ * @param obj The list object
+ * @param label The label of the list item
+ * @param icon The icon object to use for the left side of the item
+ * @param end The icon object to use for the right side of the item
+ * @param func The function to call when the item is clicked
+ * @param data The data to associate with the item for related callbacks
+ * @param cmp_func The function called for the sort.
+ *
+ * @return The created item or NULL upon failure
+ *
+ * @ingroup List
+ */
+EAPI Elm_List_Item *
+elm_list_item_sorted_insert(Evas_Object *obj, const char *label, Evas_Object *icon, Evas_Object *end, Evas_Smart_Cb func, const void *data, Eina_Compare_Cb cmp_func)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   Elm_List_Item *it = _item_new(obj, label, icon, end, func, data);
+   Eina_List *l;
+
+   wd->items = eina_list_sorted_insert(wd->items, cmp_func, it);
+   l = eina_list_data_find_list(wd->items, it);
+   l = eina_list_next(l);
+   if (!l)
+     {
+	it->node = eina_list_last(wd->items);
+	elm_box_pack_end(wd->box, it->base);
+     }
+   else
+     {
+	Elm_List_Item *before = eina_list_data_get(l);
+	it->node = before->node->prev;
+	elm_box_pack_before(wd->box, it->base, before->base);
+     }
+   return it;
+}
+
+/**
+ * Clears a list of all items.
+ *
+ * @param obj The list object
+ *
+ * @ingroup List
+ */
 EAPI void
 elm_list_clear(Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
    Elm_List_Item *it;
 
-   if (!wd->items)
-     return;
+   if (!wd) return;
+   if (!wd->items) return;
 
    eina_list_free(wd->selected);
    wd->selected = NULL;
@@ -763,11 +936,10 @@ elm_list_clear(Evas_Object *obj)
    if (wd->walking > 0)
      {
 	Eina_List *n;
-	Elm_List_Item *it;
+        
 	EINA_LIST_FOREACH(wd->items, n, it)
 	  {
-	     if (it->deleted)
-	       continue;
+	     if (it->deleted) continue;
 	     it->deleted = EINA_TRUE;
 	     wd->to_delete = eina_list_append(wd->to_delete, it);
 	  }
@@ -788,75 +960,200 @@ elm_list_clear(Evas_Object *obj)
    _sizing_eval(obj);
 }
 
+/**
+ * Starts the list.  Call before running show() on the list object.
+ *
+ * @param obj The list object
+ *
+ * @ingroup List
+ */
 EAPI void
 elm_list_go(Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-
+   if (!wd) return;
    _fix_items(obj);
 }
 
+/**
+ * Enables/disables the state of multi-select on the list object.
+ *
+ * @param obj The list object
+ * @param multi If true, multi-select is enabled
+ *
+ * @ingroup List
+ */
 EAPI void
 elm_list_multi_select_set(Evas_Object *obj, Eina_Bool multi)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-
+   if (!wd) return;
    wd->multi = multi;
 }
 
+/**
+ * Gets the state of multi-select on the list object.
+ *
+ * @param obj The list object
+ * @return If true, multi-select is enabled
+ *
+ * @ingroup List
+ */
+EAPI Eina_Bool
+elm_list_multi_select_get(const Evas_Object *obj)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype) EINA_FALSE;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return EINA_FALSE;
+   return wd->multi;
+}
+
+/**
+ * Enables/disables horizontal mode of the list
+ *
+ * @param obj The list object
+ * @param mode If true, horizontale mode is enabled
+ *
+ * @ingroup List
+ */
 EAPI void
 elm_list_horizontal_mode_set(Evas_Object *obj, Elm_List_Mode mode)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-
+   if (!wd) return;
    if (wd->mode == mode) return;
    wd->mode = mode;
-   if (wd->mode == ELM_LIST_LIMIT)
-     elm_scroller_content_min_limit(wd->scr, 1, 0);
-   else
-     elm_scroller_content_min_limit(wd->scr, 0, 0);
+   if (wd->scr)
+     {
+        if (wd->mode == ELM_LIST_LIMIT)
+          elm_scroller_content_min_limit(wd->scr, 1, 0);
+        else
+          elm_scroller_content_min_limit(wd->scr, 0, 0);
+     }
 }
 
+/**
+ * Gets the state of horizontal mode of the list
+ *
+ * @param obj The list object
+ * @return If true, horizontale mode is enabled
+ *
+ * @ingroup List
+ */
+EAPI Elm_List_Mode
+elm_list_horizontal_mode_get(const Evas_Object *obj)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype) ELM_LIST_SCROLL;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return ELM_LIST_SCROLL;
+   return wd->mode;
+}
+
+/**
+ * Enables/disables the state of always_select, meaning that
+ * an item will always be selected.
+ *
+ * @param obj The list object
+ * @param always_select If true, always_select is enabled
+ *
+ * @ingroup List
+ */
 EAPI void
 elm_list_always_select_mode_set(Evas_Object *obj, Eina_Bool always_select)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-
+   if (!wd) return;
    wd->always_select = always_select;
 }
 
+/**
+ * Gets the state of always_select.
+ * See also elm_list_always_select_mode_set()
+ *
+ * @param obj The list object
+ * @return If true, always_select is enabled
+ *
+ * @ingroup List
+ */
+EAPI Eina_Bool
+elm_list_always_select_mode_get(const Evas_Object *obj)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype) EINA_FALSE;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return EINA_FALSE;
+   return wd->always_select;
+}
+
+/**
+ * Returns a list of all the list items.
+ *
+ * @param obj The list object
+ * @return An Eina_List* of the list items, or NULL on failure
+ *
+ * @ingroup List
+ */
 EAPI const Eina_List *
 elm_list_items_get(const Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
    Widget_Data *wd = elm_widget_data_get(obj);
-
+   if (!wd) return NULL;
    return wd->items;
 }
 
+/**
+ * Returns the currently selected list item.
+ *
+ * @param obj The list object
+ * @return The selected list item, or NULL on failure
+ *
+ * @ingroup List
+ */
 EAPI Elm_List_Item *
 elm_list_selected_item_get(const Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
    Widget_Data *wd = elm_widget_data_get(obj);
-
+   if (!wd) return NULL;
    if (wd->selected) return wd->selected->data;
    return NULL;
 }
 
+/**
+ * Returns a list of the currently selected list items.
+ *
+ * @param obj The list object
+ * @return An Eina_List* of the selected list items, or NULL on failure
+ *
+ * @ingroup List
+ */
 EAPI const Eina_List *
 elm_list_selected_items_get(const Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
    Widget_Data *wd = elm_widget_data_get(obj);
-
+   if (!wd) return NULL;
    return wd->selected;
 }
 
+/**
+ * Sets the selected state of @p it.
+ *
+ * @param it The list item
+ * @param selected Enables/disables the selected state
+ *
+ * @ingroup List
+ */
 EAPI void
 elm_list_item_selected_set(Elm_List_Item *it, Eina_Bool selected)
 {
    Widget_Data *wd = elm_widget_data_get(it->obj);
-
+   if (!wd) return;
    ELM_LIST_ITEM_CHECK_DELETED_RETURN(it);
-
    selected = !!selected;
    if (it->selected == selected) return;
 
@@ -878,6 +1175,28 @@ elm_list_item_selected_set(Elm_List_Item *it, Eina_Bool selected)
    _elm_list_unwalk(wd);
 }
 
+/**
+ * Gets the selected state of @p it.
+ *
+ * @param it The list item
+ * @return If true, the item is selected
+ *
+ * @ingroup List
+ */
+EAPI Eina_Bool
+elm_list_item_selected_get(Elm_List_Item *it)
+{
+   ELM_LIST_ITEM_CHECK_DELETED_RETURN(it, EINA_FALSE);
+   return it->selected;
+}
+
+/**
+ * Brings @p it to the center of the list view.
+ *
+ * @param it The list item
+ *
+ * @ingroup List
+ */
 EAPI void
 elm_list_item_show(Elm_List_Item *it)
 {
@@ -886,19 +1205,26 @@ elm_list_item_show(Elm_List_Item *it)
    Evas_Coord x, y, w, h;
 
    ELM_LIST_ITEM_CHECK_DELETED_RETURN(it);
-
    evas_object_geometry_get(wd->box, &bx, &by, &bw, &bh);
    evas_object_geometry_get(it->base, &x, &y, &w, &h);
    x -= bx;
    y -= by;
-   elm_scroller_region_show(wd->scr, x, y, w, h);
+   if (wd->scr)
+     elm_scroller_region_show(wd->scr, x, y, w, h);
 }
 
+/**
+ * Deletes item @p it from the list.
+ *
+ * @param it The list item to delete
+ *
+ * @ingroup List
+ */
 EAPI void
 elm_list_item_del(Elm_List_Item *it)
 {
    Widget_Data *wd = elm_widget_data_get(it->obj);
-
+   if (!wd) return;
    ELM_LIST_ITEM_CHECK_DELETED_RETURN(it);
 
    if (it->selected) _item_unselect(it);
@@ -927,124 +1253,272 @@ elm_list_item_del(Elm_List_Item *it)
  * @param it The item to set the callback on
  * @param func The function called
  *
- * @ingroup Hoversel
+ * @ingroup List
  */
 EAPI void
-elm_list_item_del_cb_set(Elm_List_Item *it, void (*func)(void *data, Evas_Object *obj, void *event_info))
+elm_list_item_del_cb_set(Elm_List_Item *it, Evas_Smart_Cb func)
 {
    ELM_LIST_ITEM_CHECK_DELETED_RETURN(it);
-
    it->del_cb = func;
 }
 
+/**
+ * Returns the data associated with the item.
+ *
+ * @param it The list item
+ * @return The data associated with @p it
+ *
+ * @ingroup List
+ */
 EAPI void *
 elm_list_item_data_get(const Elm_List_Item *it)
 {
    ELM_LIST_ITEM_CHECK_DELETED_RETURN(it, NULL);
-
    return (void *)it->data;
 }
 
+/**
+ * Returns the left side icon associated with the item.
+ *
+ * @param it The list item
+ * @return The left side icon associated with @p it
+ *
+ * @ingroup List
+ */
 EAPI Evas_Object *
 elm_list_item_icon_get(const Elm_List_Item *it)
 {
    ELM_LIST_ITEM_CHECK_DELETED_RETURN(it, NULL);
-
    if (it->dummy_icon) return NULL;
    return it->icon;
 }
 
+/**
+ * Sets the left side icon associated with the item.
+ *
+ * Once the icon object is set, a previously set one will be deleted.
+ * You probably don't want, then, to have the <b>same</b> icon object set
+ * for more than one item of the list.
+ *
+ * @param it The list item
+ * @param icon The left side icon object to associate with @p it
+ *
+ * @ingroup List
+ */
 EAPI void
 elm_list_item_icon_set(Elm_List_Item *it, Evas_Object *icon)
 {
    ELM_LIST_ITEM_CHECK_DELETED_RETURN(it);
-
    if (it->icon == icon) return;
    if (it->dummy_icon && !icon) return;
-   if (it->dummy_icon) evas_object_del(it->icon);
+   if (it->dummy_icon)
+     {
+	evas_object_del(it->icon);
+	it->dummy_icon = EINA_FALSE;
+     }
    if (!icon)
      {
 	icon = evas_object_rectangle_add(evas_object_evas_get(it->obj));
 	evas_object_color_set(icon, 0, 0, 0, 0);
 	it->dummy_icon = EINA_TRUE;
      }
+   if (it->icon)
+     {
+	evas_object_del(it->icon);
+	it->icon = NULL;
+     }
    it->icon = icon;
    if (it->base)
      edje_object_part_swallow(it->base, "elm.swallow.icon", icon);
 }
 
+/**
+ * Gets the right side icon associated with the item.
+ *
+ * @param it The list item
+ * @return The right side icon object associated with @p it
+ *
+ * @ingroup List
+ */
 EAPI Evas_Object *
 elm_list_item_end_get(const Elm_List_Item *it)
 {
    ELM_LIST_ITEM_CHECK_DELETED_RETURN(it, NULL);
-
    if (it->dummy_end) return NULL;
    return it->end;
 }
 
+/**
+ * Sets the right side icon associated with the item.
+ *
+ * Once the icon object is set, a previously set one will be deleted.
+ * You probably don't want, then, to have the <b>same</b> icon object set
+ * for more than one item of the list.
+ *
+ * @param it The list item
+ * @param icon The right side icon object to associate with @p it
+ *
+ * @ingroup List
+ */
 EAPI void
 elm_list_item_end_set(Elm_List_Item *it, Evas_Object *end)
 {
    ELM_LIST_ITEM_CHECK_DELETED_RETURN(it);
-
    if (it->end == end) return;
    if (it->dummy_end && !end) return;
-   if (it->dummy_end) evas_object_del(it->end);
+   if (it->dummy_end)
+     {
+	evas_object_del(it->end);
+	it->dummy_icon = EINA_FALSE;
+     }
    if (!end)
      {
 	end = evas_object_rectangle_add(evas_object_evas_get(it->obj));
 	evas_object_color_set(end, 0, 0, 0, 0);
 	it->dummy_end = EINA_TRUE;
      }
+   if (it->end)
+     {
+	evas_object_del(it->end);
+	it->end = NULL;
+     }
    it->end = end;
    if (it->base)
      edje_object_part_swallow(it->base, "elm.swallow.end", end);
 }
 
+/**
+ * Gets the base object of the item.
+ *
+ * @param it The list item
+ * @return The base object associated with @p it
+ *
+ * @ingroup List
+ */
 EAPI Evas_Object *
 elm_list_item_base_get(const Elm_List_Item *it)
 {
    ELM_LIST_ITEM_CHECK_DELETED_RETURN(it, NULL);
-
    return it->base;
 }
 
+/**
+ * Gets the label of the item.
+ *
+ * @param it The list item
+ * @return The label of @p it
+ *
+ * @ingroup List
+ */
 EAPI const char *
 elm_list_item_label_get(const Elm_List_Item *it)
 {
    ELM_LIST_ITEM_CHECK_DELETED_RETURN(it, NULL);
-
    return it->label;
 }
 
+/**
+ * Sets the label of the item.
+ *
+ * @param it The list item
+ * @param text The label of @p it
+ *
+ * @ingroup List
+ */
 EAPI void
 elm_list_item_label_set(Elm_List_Item *it, const char *text)
 {
    ELM_LIST_ITEM_CHECK_DELETED_RETURN(it);
-
    if (!eina_stringshare_replace(&it->label, text)) return;
    if (it->base)
      edje_object_part_text_set(it->base, "elm.text", it->label);
 }
 
+/**
+ * Gets the item before @p it in the list.
+ *
+ * @param it The list item
+ * @return The item before @p it, or NULL on failure
+ *
+ * @ingroup List
+ */
 EAPI Elm_List_Item *
 elm_list_item_prev(const Elm_List_Item *it)
 {
    ELM_LIST_ITEM_CHECK_DELETED_RETURN(it, NULL);
-
-   if (it->node->prev)
-     return it->node->prev->data;
-   else
-     return NULL;
+   if (it->node->prev) return it->node->prev->data;
+   else return NULL;
 }
 
+/**
+ * Gets the item after @p it in the list.
+ *
+ * @param it The list item
+ * @return The item after @p it, or NULL on failure
+ *
+ * @ingroup List
+ */
 EAPI Elm_List_Item *
 elm_list_item_next(const Elm_List_Item *it)
 {
    ELM_LIST_ITEM_CHECK_DELETED_RETURN(it, NULL);
+   if (it->node->next) return it->node->next->data;
+   else return NULL;
+}
 
-   if (it->node->next)
-     return it->node->next->data;
-   else
-     return NULL;
+/**
+ * Set bounce mode
+ *
+ * This will enable or disable the scroller bounce mode for the list. See
+ * elm_scroller_bounce_set() for details
+ *
+ * @param obj The list object
+ * @param h_bounce Allow bounce horizontally
+ * @param v_bounce Allow bounce vertically
+ *
+ * @ingroup List
+ */
+EAPI void
+elm_list_bounce_set(Evas_Object *obj, Eina_Bool h_bounce, Eina_Bool v_bounce)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype);
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
+   if (wd->scr)
+     elm_scroller_bounce_set(wd->scr, h_bounce, v_bounce);
+}
+
+/**
+ * Set the scrollbar policy
+ *
+ * This sets the scrollbar visibility policy for the given scroller.
+ * ELM_SMART_SCROLLER_POLICY_AUTO means the scrollber is made visible if it
+ * is needed, and otherwise kept hidden. ELM_SMART_SCROLLER_POLICY_ON turns
+ * it on all the time, and ELM_SMART_SCROLLER_POLICY_OFF always keeps it off.
+ * This applies respectively for the horizontal and vertical scrollbars.
+ *
+ * @param obj The list object
+ * @param policy_h Horizontal scrollbar policy
+ * @param policy_v Vertical scrollbar policy
+ *
+ * @ingroup List
+ */
+EAPI void
+elm_list_scroller_policy_set(Evas_Object *obj, Elm_Scroller_Policy policy_h, Elm_Scroller_Policy policy_v)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype);
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
+   if (wd->scr)
+     elm_scroller_policy_set(wd->scr, policy_h, policy_v);
+}
+
+EAPI void
+elm_list_scroller_policy_get(const Evas_Object *obj, Elm_Scroller_Policy *policy_h, Elm_Scroller_Policy *policy_v)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype);
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
+   if (wd->scr)
+     elm_scroller_policy_get(wd->scr, policy_h, policy_v);
 }
