@@ -1,11 +1,15 @@
+#ifdef HAVE_CONFIG_H
+# include "elementary_config.h"
+#endif
+
 #ifndef _GNU_SOURCE
 # define _GNU_SOURCE
 #endif
 
 #include <dlfcn.h>      /* dlopen,dlclose,etc */
 
-#ifdef HAVE_CONFIG_H
-# include "elementary_config.h"
+#ifdef HAVE_CRT_EXTERNS_H
+# include <crt_externs.h>
 #endif
 
 #ifdef HAVE_EVIL
@@ -15,6 +19,9 @@
 #include <Elementary.h>
 #include "elm_priv.h"
 
+static Elm_Version _version = { VMAJ, VMIN, VMIC, VREV };
+EAPI Elm_Version *elm_version = &_version;
+
 /**
  * @defgroup Start Getting Started
  *
@@ -255,93 +262,27 @@ myapp_CFLAGS =
  *
  */
 
-static int _elm_signal_exit(void *data, int ev_type, void *ev);
-#ifdef HAVE_ELEMENTARY_X
-static int _elm_window_property_change(void *data, int ev_type, void *ev);
-#endif
-static void _elm_rescale(void);
+static Eina_Bool _elm_signal_exit(void *data, int ev_type, void *ev);
 
 char *_elm_appname = NULL;
-Elm_Config *_elm_config = NULL;
 const char *_elm_data_dir = NULL;
 const char *_elm_lib_dir = NULL;
 int _elm_log_dom = -1;
 
 EAPI int ELM_EVENT_POLICY_CHANGED = 0;
 
+static int _elm_init_count = 0;
 static int _elm_policies[ELM_POLICY_LAST];
 static Ecore_Event_Handler *_elm_exit_handler = NULL;
-static Ecore_Event_Handler *_elm_event_property_change = NULL;
-#ifdef HAVE_ELEMENTARY_X
-static Ecore_X_Atom _elm_atom_enlightenment_scale = 0;
-static Ecore_X_Atom _elm_atom_enlightenment_finger_size = 0;
-static Ecore_X_Atom _elm_atom_enlightenment_theme = 0;
-#endif
 
-static int
-_elm_signal_exit(void *data, int ev_type, void *ev)
+static Eina_Bool
+_elm_signal_exit(void *data __UNUSED__, int ev_type __UNUSED__, void *ev __UNUSED__)
 {
    elm_exit();
-   return 1;
+   return ECORE_CALLBACK_PASS_ON;
 }
 
-#ifdef HAVE_ELEMENTARY_X
-static int
-_elm_window_property_change(void *data, int ev_type, void *ev)
-{
-   Ecore_X_Event_Window_Property *event = ev;
-
-   if (event->win == ecore_x_window_root_first_get())
-     {
-	if (event->atom == _elm_atom_enlightenment_scale)
-	  {
-	     int val = 1000;
-
-	     if (ecore_x_window_prop_card32_get(event->win,
-						event->atom,
-						&val, 1) > 0)
-	       {
-		  double pscale;
-
-		  pscale = _elm_config->scale;
-		  if (val > 0) _elm_config->scale = (double)val / 1000.0;
-		  if (pscale != _elm_config->scale) _elm_rescale();
-	       }
-	  }
-	else if (event->atom == _elm_atom_enlightenment_finger_size)
-	  {
-	     int val = 1000;
-
-	     if (ecore_x_window_prop_card32_get(event->win,
-						event->atom,
-						&val, 1) > 0)
-	       {
-		  int pfinger_size;
-
-		  pfinger_size = _elm_config->finger_size;
-		  _elm_config->finger_size = val;
-		  if (pfinger_size != _elm_config->finger_size) _elm_rescale();
-	       }
-	  }
-	else if (event->atom == _elm_atom_enlightenment_theme)
-	  {
-             char *val = NULL;
-             
-             val = ecore_x_window_prop_string_get(event->win,
-                                                  event->atom);
-             if (val)
-               {
-                  _elm_theme_parse(val);
-                  free(val);
-                  _elm_rescale();
-               }
-	  }
-     }
-   return 1;
-}
-#endif
-
-static void
+void
 _elm_rescale(void)
 {
    edje_scale_set(_elm_config->scale);
@@ -362,6 +303,8 @@ _elm_rescale(void)
 EAPI void
 elm_init(int argc, char **argv)
 {
+   _elm_init_count++;
+   if (_elm_init_count != 1) return;
    elm_quicklaunch_init(argc, argv);
    elm_quicklaunch_sub_init(argc, argv);
 }
@@ -378,12 +321,14 @@ elm_init(int argc, char **argv)
 EAPI void
 elm_shutdown(void)
 {
+   _elm_init_count--;
+   if (_elm_init_count != 0) return;
    elm_quicklaunch_sub_shutdown();
    elm_quicklaunch_shutdown();
 }
 
 #ifdef ELM_EDBUS
-static Eina_Bool _elm_need_e_dbus = 0;
+static Eina_Bool _elm_need_e_dbus = EINA_FALSE;
 #endif
 EAPI void
 elm_need_e_dbus(void)
@@ -410,7 +355,7 @@ _elm_unneed_e_dbus(void)
 }
 
 #ifdef ELM_EFREET
-static Eina_Bool _elm_need_efreet = 0;
+static Eina_Bool _elm_need_efreet = EINA_FALSE;
 #endif
 EAPI void
 elm_need_efreet(void)
@@ -419,7 +364,6 @@ elm_need_efreet(void)
    if (_elm_need_efreet) return;
    _elm_need_efreet = 1;
    efreet_init();
-   efreet_util_init();
    efreet_mime_init();
    efreet_trash_init();
    /*
@@ -448,7 +392,6 @@ _elm_unneed_efreet(void)
         _elm_need_efreet = 0;
         efreet_trash_shutdown();
         efreet_mime_shutdown();
-        efreet_util_shutdown();
         efreet_shutdown();
      }
 #endif   
@@ -457,9 +400,8 @@ _elm_unneed_efreet(void)
 EAPI void
 elm_quicklaunch_init(int argc, char **argv)
 {
-   int i;
    char buf[PATH_MAX], *s;
-
+   
    eina_init();
    _elm_log_dom = eina_log_domain_register("elementary", EINA_COLOR_LIGHTBLUE);
    if (!_elm_log_dom)
@@ -480,11 +422,12 @@ elm_quicklaunch_init(int argc, char **argv)
    evas_init();
    edje_init();
    ecore_evas_init(); // FIXME: check errors
+   ecore_imf_init();
    _elm_module_init();
 
    _elm_exit_handler = ecore_event_handler_add(ECORE_EVENT_SIGNAL_EXIT, _elm_signal_exit, NULL);
 
-   _elm_appname = strdup(ecore_file_file_get(argv[0]));
+   if (argv) _elm_appname = strdup(ecore_file_file_get(argv[0]));
 
    if (!_elm_data_dir)
      {
@@ -549,230 +492,21 @@ elm_quicklaunch_init(int argc, char **argv)
 #endif
    if (!_elm_data_dir)
      _elm_data_dir = eina_stringshare_add(PACKAGE_DATA_DIR);
-   if (!_elm_data_dir)
+  if (!_elm_data_dir)
      _elm_data_dir = eina_stringshare_add("/");
    if (!_elm_lib_dir)
      _elm_lib_dir = eina_stringshare_add(PACKAGE_LIB_DIR);
    if (!_elm_lib_dir)
      _elm_lib_dir = eina_stringshare_add("/");
 
-   // FIXME: actually load config from file - use eet. also for X properties,
-   // reduce to single x property with eet data encoded in it.
-   _elm_config = ELM_NEW(Elm_Config);
-   _elm_config->engine = ELM_SOFTWARE_X11;
-   _elm_config->thumbscroll_enable = 1;
-   _elm_config->thumbscroll_threshhold = 24;
-   _elm_config->thumbscroll_momentum_threshhold = 100.0;
-   _elm_config->thumbscroll_friction = 1.0;
-   _elm_config->thumbscroll_bounce_friction = 0.5;
-   _elm_config->thumbscroll_bounce_enable = 1;
-   _elm_config->page_scroll_friction = 0.5;
-   _elm_config->bring_in_scroll_friction = 0.5;
-   _elm_config->zoom_friction = 0.5;
-   _elm_config->scale = 1.0;
-   _elm_config->font_hinting = 2;
-   _elm_config->font_dirs = NULL;
-   _elm_config->image_cache = 4096;
-   _elm_config->font_cache = 512;
-   _elm_config->finger_size = 40;
-   _elm_config->bgpixmap = 0;
-   _elm_config->compositing = 1;
-   _elm_config->fps = 60.0;
-
-   s = getenv("ELM_ENGINE");
-   if (s)
-     {
-	if ((!strcasecmp(s, "x11")) ||
-	    (!strcasecmp(s, "x")) ||
-	    (!strcasecmp(s, "software-x11")) ||
-	    (!strcasecmp(s, "software_x11")))
-	  _elm_config->engine = ELM_SOFTWARE_X11;
-	else if ((!strcasecmp(s, "x11-16")) ||
-		 (!strcasecmp(s, "x16")) ||
-		 (!strcasecmp(s, "software-16-x11")) ||
-		 (!strcasecmp(s, "software_16_x11")))
-	  _elm_config->engine = ELM_SOFTWARE_16_X11;
-	else if ((!strcasecmp(s, "xrender")) ||
-		 (!strcasecmp(s, "xr")) ||
-		 (!strcasecmp(s, "xrender-x11")) ||
-		 (!strcasecmp(s, "xrender_x11")))
-	  _elm_config->engine = ELM_XRENDER_X11;
-	else if ((!strcasecmp(s, "fb")) ||
-		 (!strcasecmp(s, "software-fb")) ||
-		 (!strcasecmp(s, "software_fb")))
-	  _elm_config->engine = ELM_SOFTWARE_FB;
-	else if ((!strcasecmp(s, "opengl")) ||
-		 (!strcasecmp(s, "gl")) ||
-		 (!strcasecmp(s, "opengl-x11")) ||
-		 (!strcasecmp(s, "opengl_x11")))
-	  _elm_config->engine = ELM_OPENGL_X11;
-	else if ((!strcasecmp(s, "gdi")) ||
-		 (!strcasecmp(s, "software-gdi")) ||
-		 (!strcasecmp(s, "software_gdi")))
-	  _elm_config->engine = ELM_SOFTWARE_WIN32;
-	else if ((!strcasecmp(s, "wince-gdi")) ||
-		 (!strcasecmp(s, "software-16-wince-gdi")) ||
-		 (!strcasecmp(s, "software_16_wince_gdi")))
-	  _elm_config->engine = ELM_SOFTWARE_16_WINCE;
-     }
-
-   s = getenv("ELM_THUMBSCROLL_ENABLE");
-   if (s) _elm_config->thumbscroll_enable = atoi(s);
-   s = getenv("ELM_THUMBSCROLL_THRESHOLD");
-   if (s) _elm_config->thumbscroll_threshhold = atoi(s);
-   // FIXME: floatformat locale issues here 1.0 vs 1,0 - should just be 1.0
-   s = getenv("ELM_THUMBSCROLL_MOMENTUM_THRESHOLD");
-   if (s) _elm_config->thumbscroll_momentum_threshhold = atof(s);
-   s = getenv("ELM_THUMBSCROLL_FRICTION");
-   if (s) _elm_config->thumbscroll_friction = atof(s);
-   s = getenv("ELM_PAGE_SCROLL_FRICTION");
-   if (s) _elm_config->page_scroll_friction = atof(s);
-   s = getenv("ELM_BRING_IN_SCROLL_FRICTION");
-   if (s) _elm_config->bring_in_scroll_friction = atof(s);
-   s = getenv("ELM_ZOOM_FRICTION");
-   if (s) _elm_config->zoom_friction = atof(s);
-
-   s = getenv("ELM_THEME");
-   if (s) _elm_theme_parse(s);
-   else _elm_theme_parse("default");
-
-   _elm_config->font_hinting = 2;
-   s = getenv("ELM_FONT_HINTING");
-   if (s)
-     {
-	if (!strcasecmp(s, "none"))          _elm_config->font_hinting = 0;
-	else if (!strcasecmp(s, "auto"))     _elm_config->font_hinting = 1;
-	else if (!strcasecmp(s, "bytecode")) _elm_config->font_hinting = 2;
-     }
-
-   s = getenv("ELM_FONT_PATH");
-   if (s)
-     {
-	const char *p, *pp;
-	char *buf;
-
-	buf = alloca(strlen(s) + 1);
-	p = s;
-	pp = p;
-	for (;;)
-	  {
-	     if ((*p == ':') || (*p == 0))
-	       {
-		  int len;
-
-		  len = p - pp;
-		  strncpy(buf, pp, len);
-		  buf[len] = 0;
-		  _elm_config->font_dirs = 
-                    eina_list_append(_elm_config->font_dirs, 
-                                     eina_stringshare_add(buf));
-		  if (*p == 0) break;
-		  p++;
-		  pp = p;
-	       }
-	     else
-	       {
-		  if (*p == 0) break;
-		  p++;
-	       }
-	  }
-     }
-
-   s = getenv("ELM_IMAGE_CACHE");
-   if (s) _elm_config->image_cache = atoi(s);
-
-   s = getenv("ELM_FONT_CACHE");
-   if (s) _elm_config->font_cache = atoi(s);
-
-   s = getenv("ELM_SCALE");
-   if (s) _elm_config->scale = atof(s);
-
-   _elm_config->finger_size =
-     (double)_elm_config->finger_size * _elm_config->scale;
-   s = getenv("ELM_FINGER_SIZE");
-   if (s) _elm_config->finger_size = atoi(s);
-
-   s = getenv("ELM_FPS");
-   if (s) _elm_config->fps = atof(s);
-   if (_elm_config->fps < 1.0) _elm_config->fps = 1.0;
-
-   ecore_animator_frametime_set(1.0 / _elm_config->fps);
-   edje_frametime_set(1.0 / 60.0);
-   edje_scale_set(_elm_config->scale);
-
-   s = getenv("ELM_MODULES");
-   if (s) _elm_module_parse(s);
+   _elm_config_init();
 }
 
 EAPI void
 elm_quicklaunch_sub_init(int argc, char **argv)
 {
    ecore_app_args_set(argc, (const char **)argv);
-   if ((_elm_config->engine == ELM_SOFTWARE_X11) ||
-       (_elm_config->engine == ELM_SOFTWARE_16_X11) ||
-       (_elm_config->engine == ELM_XRENDER_X11) ||
-       (_elm_config->engine == ELM_OPENGL_X11))
-     {
-#ifdef HAVE_ELEMENTARY_X
-	int val = 1000;
-
-	if (!ecore_x_init(NULL))
-	  {
-	     ERR("Cannot connect to X11 display. check $DISPLAY variable");
-	     exit(1);
-	  }
-	if (!ecore_x_screen_is_composited(0))
-	  _elm_config->compositing = 0;
-        
-	_elm_atom_enlightenment_scale       = ecore_x_atom_get("ENLIGHTENMENT_SCALE");
-	_elm_atom_enlightenment_finger_size = ecore_x_atom_get("ENLIGHTENMENT_FINGER_SIZE");
-	_elm_atom_enlightenment_theme       = ecore_x_atom_get("ENLIGHTENMENT_THEME");
-        
-	ecore_x_event_mask_set(ecore_x_window_root_first_get(),
-			       ECORE_X_EVENT_MASK_WINDOW_PROPERTY);
-	_elm_event_property_change = ecore_event_handler_add
-	  (ECORE_X_EVENT_WINDOW_PROPERTY, _elm_window_property_change, NULL);
-	if (!getenv("ELM_SCALE"))
-	  {
-	     if (ecore_x_window_prop_card32_get(ecore_x_window_root_first_get(),
-						_elm_atom_enlightenment_scale,
-						&val, 1) > 0)
-	       {
-		  if (val > 0)
-		    {
-		       _elm_config->scale = (double)val / 1000.0;
-		       // FIXME: hack until e export finger size too
-		       if (getenv("ELM_FINGER_SIZE"))
-			 _elm_config->finger_size = 40.0 * _elm_config->scale;
-		    }
-	       }
-	  }
-	if (!getenv("ELM_FINGER_SIZE"))
-	  {
-	     if (ecore_x_window_prop_card32_get(ecore_x_window_root_first_get(),
-						_elm_atom_enlightenment_finger_size,
-						&val, 1) > 0)
-	       {
-		  if (val > 0)
-		    {
-		       _elm_config->finger_size = val;
-		    }
-	       }
-	  }
-	if (!getenv("ELM_THEME"))
-	  {
-             char *s;
-             
-             s = ecore_x_window_prop_string_get(ecore_x_window_root_first_get(),
-                                                _elm_atom_enlightenment_theme);
-             if (s)
-	       {
-                  _elm_theme_parse(s);
-                  free(s);
-	       }
-	  }
-#endif
-      }
+   _elm_config_sub_init();
 }
 
 EAPI void
@@ -783,12 +517,13 @@ elm_quicklaunch_sub_shutdown(void)
        (_elm_config->engine == ELM_SOFTWARE_16_X11) ||
        (_elm_config->engine == ELM_XRENDER_X11) ||
        (_elm_config->engine == ELM_OPENGL_X11) ||
+       (_elm_config->engine == ELM_SOFTWARE_SDL) ||
+       (_elm_config->engine == ELM_SOFTWARE_16_SDL) ||
+       (_elm_config->engine == ELM_OPENGL_SDL) ||
        (_elm_config->engine == ELM_SOFTWARE_WIN32) ||
        (_elm_config->engine == ELM_SOFTWARE_16_WINCE))
      {
 #ifdef HAVE_ELEMENTARY_X
-	ecore_event_handler_del(_elm_event_property_change);
-	_elm_event_property_change = NULL;
 	ecore_x_disconnect();
 #endif
 	evas_cserve_disconnect();
@@ -798,24 +533,25 @@ elm_quicklaunch_sub_shutdown(void)
 EAPI void
 elm_quicklaunch_shutdown(void)
 {
-   const char *fontdir;
-
    eina_stringshare_del(_elm_data_dir);
    _elm_data_dir = NULL;
+   eina_stringshare_del(_elm_lib_dir);
+   _elm_lib_dir = NULL;
 
-   EINA_LIST_FREE(_elm_config->font_dirs, fontdir)
-     {
-	eina_stringshare_del(fontdir);
-     }
-
+   free(_elm_appname);
+   _elm_appname = NULL;
+   
+   _elm_config_shutdown();
+   
    ecore_event_handler_del(_elm_exit_handler);
    _elm_exit_handler = NULL;
 
-   free(_elm_config);
-   free(_elm_appname);
+   _elm_theme_shutdown();
    _elm_unneed_efreet();
    _elm_unneed_e_dbus();
+   _elm_unneed_ethumb();
    _elm_module_shutdown();
+   ecore_imf_shutdown();
    ecore_evas_shutdown();
    edje_shutdown();
    evas_shutdown();
@@ -863,7 +599,7 @@ static void *qr_handle = NULL;
 static int (*qr_main) (int argc, char **argv) = NULL;
 
 EAPI Eina_Bool
-elm_quicklaunch_prepare(int argc, char **argv)
+elm_quicklaunch_prepare(int argc __UNUSED__, char **argv)
 {
 #ifdef HAVE_FORK
    char *exe = elm_quicklaunch_exe_path_get(argv[0]);
@@ -995,7 +731,7 @@ elm_quicklaunch_fork(int argc, char **argv, char *cwd, void (postfork_func) (voi
    // so ps etc. report what you'd expect. for other unixes and os's this
    // may just not work
    save_env();
-   if (real_argv)
+   if (argv)
      {
 	char *lastarg, *p;
 
@@ -1051,10 +787,10 @@ elm_quicklaunch_exe_path_get(const char *exe)
    if ((exe[0] == '.') && (exe[1] == '.') && (exe[2] == '/')) return strdup(exe);
    if (!path)
      {
-	const char *p, *pp, *s;
-	char *buf;
+	const char *p, *pp;
+	char *buf2;
 	path = getenv("PATH");
-	buf = alloca(strlen(path) + 1);
+	buf2 = alloca(strlen(path) + 1);
 	p = path;
 	pp = p;
 	for (;;)
@@ -1064,9 +800,9 @@ elm_quicklaunch_exe_path_get(const char *exe)
 		  int len;
 
 		  len = p - pp;
-		  strncpy(buf, pp, len);
-		  buf[len] = 0;
-		  pathlist = eina_list_append(pathlist, eina_stringshare_add(buf));
+		  strncpy(buf2, pp, len);
+		  buf2[len] = 0;
+		  pathlist = eina_list_append(pathlist, eina_stringshare_add(buf2));
 		  if (*p == 0) break;
 		  p++;
 		  pp = p;
@@ -1171,6 +907,25 @@ elm_policy_get(unsigned int policy)
    return _elm_policies[policy];
 }
 
+/**
+ * Flush all caches & dump all data that can be to lean down to use less memory
+ */
+EAPI void
+elm_all_flush(void)
+{
+   const Eina_List *l;
+   Evas_Object *obj;
+   
+   EINA_LIST_FOREACH(_elm_win_list, l, obj)
+     {
+        Evas *e = evas_object_evas_get(obj);
+        edje_file_cache_flush();
+        edje_collection_cache_flush();
+        evas_image_cache_flush(e);
+        evas_font_cache_flush(e);
+        evas_render_dump(e);
+     }
+}
 
 /**
  * @defgroup Scaling Selective Widget Scaling
@@ -1309,12 +1064,33 @@ elm_scale_set(double scale)
    _elm_rescale();
 }
 
+/**
+ * Set the global scaling factor for all applications on the display
+ * 
+ * This sets the globally configured scaling factor that is applied to all
+ * objects for all applications.
+ * @param scale The scaling factor to set
+ * @ingroup Scaling
+ */
+EAPI void
+elm_scale_all_set(double scale)
+{
+#ifdef HAVE_ELEMENTARY_X
+   static Ecore_X_Atom atom = 0;
+   unsigned int scale_i = (unsigned int)(scale * 1000.0);
+
+   if (!atom) atom = ecore_x_atom_get("ENLIGHTENMENT_SCALE");
+   ecore_x_window_prop_card32_set(ecore_x_window_root_first_get(),
+                                  atom, &scale_i, 1);
+#endif   
+}
+
 /**
  * @defgroup Fingers Fingers
  *
  * Elementary is designed to be finger-friendly for touchscreens, and so in
  * addition to scaling for display resolution, it can also scale based on
- * finger "resoltion" (or size).
+ * finger "resolution" (or size).
  */
 
 /**
@@ -1347,6 +1123,28 @@ elm_finger_size_set(Evas_Coord size)
    _elm_rescale();
 }
 
+/**
+ * Set the configured finger size for all applications on the display
+ *
+ * This sets the globally configured finger size in pixels for all applications
+ * on the display
+ *
+ * @param size The finger size
+ * @ingroup Fingers
+ */
+EAPI void
+elm_finger_size_all_set(Evas_Coord size)
+{
+#ifdef HAVE_ELEMENTARY_X
+   static Ecore_X_Atom atom = 0;
+   unsigned int size_i = (unsigned int)size;
+
+   if (!atom) atom = ecore_x_atom_get("ENLIGHTENMENT_FINGER_SIZE");
+   ecore_x_window_prop_card32_set(ecore_x_window_root_first_get(),
+                                  atom, &size_i, 1);
+#endif   
+}
+
 /**
  * Adjust size of an element for finger usage
  *
@@ -1377,10 +1175,25 @@ elm_coords_finger_size_adjust(int times_w, Evas_Coord *w, int times_h, Evas_Coor
  * within the application window.
  */
 
+/**
+ * Get the focus of the object
+ *
+ * This gets the focused property of the object.
+ *
+ * @param obj The object
+ * @return 1 if the object is focused, 0 if not.
+ * @ingroup Focus
+ */
+EAPI Eina_Bool
+elm_object_focus_get(Evas_Object *obj)
+{
+   return elm_widget_focus_get(obj);
+}
+
 /**
  * Set the focus to the object
  *
- * This sets the focus target forkeyboard input to be the object indicated.
+ * This sets the focus target for keyboard input to be the object indicated.
  *
  * @param obj The object
  * @ingroup Focus
@@ -1389,7 +1202,23 @@ EAPI void
 elm_object_focus(Evas_Object *obj)
 {
    if (!elm_widget_can_focus_get(obj)) return;
-   elm_widget_focus_set(obj, 1);
+   elm_widget_focus_steal(obj);
+}
+
+/**
+ * Remove the focus from the object
+ *
+ * This removes the focus target for keyboard input from be the object
+ * indicated.
+ *
+ * @param obj The object
+ * @ingroup Focus
+ */
+EAPI void
+elm_object_unfocus(Evas_Object *obj)
+{
+   if (!elm_widget_can_focus_get(obj)) return;
+   elm_widget_focused_object_clear(obj);
 }
 
 /**
@@ -1419,7 +1248,7 @@ elm_object_focus_allow_set(Evas_Object *obj, Eina_Bool enable)
  * @ingroup Focus
  */
 EAPI Eina_Bool
-elm_object_focus_allow_get(Evas_Object *obj)
+elm_object_focus_allow_get(const Evas_Object *obj)
 {
    return elm_widget_can_focus_get(obj);
 }
@@ -1497,3 +1326,128 @@ elm_object_scroll_freeze_pop(Evas_Object *obj)
 {
    elm_widget_scroll_freeze_pop(obj);
 }
+
+/**
+ * @defgroup WidgetNavigation Widget Tree Navigation.
+ *
+ * How to check if an Evas Object is an Elementary widget? How to get
+ * the first elementary widget that is parent of the given object?
+ * These are all covered in widget tree navigation.
+ */
+
+/**
+ * Check if the given Evas Object is an Elementary widget.
+ *
+ * @param obj the object to query.
+ * @return @c EINA_TRUE if it is an elementary widget variant,
+ *         @c EINA_FALSE otherwise
+ */
+EAPI Eina_Bool
+elm_object_widget_check(const Evas_Object *obj)
+{
+   return elm_widget_is(obj);
+}
+
+/**
+ * Get the first parent of the given object that is an Elementary widget.
+ *
+ * @param obj the object to query.
+ * @return the parent object that is an Elementary widget, or @c NULL
+ *         if no parent is, or no parents at all.
+ */
+EAPI Evas_Object *
+elm_object_parent_widget_get(const Evas_Object *obj)
+{
+   return elm_widget_parent_widget_get(obj);
+}
+
+/**
+ * Get the top level parent of an Elementary widget.
+ *
+ * @param obj The object to query.
+ * @return The top level Elementary widget, or @c NULL if parent cannot be
+ * found.
+ */
+EAPI Evas_Object *
+elm_object_top_widget_get(const Evas_Object *obj)
+{
+   return elm_widget_top_get(obj);
+}
+
+/**
+ * Get the string that represents this Elementary widget.
+ *
+ * @note Elementary is weird and exposes itself as a single
+ *       Evas_Object_Smart_Class of type "elm_widget", so
+ *       evas_object_type_get() always return that, making debug and
+ *       language bindings hard. This function tries to mitigate this
+ *       problem, but the solution is to change Elementary to use
+ *       proper inheritance.
+ *
+ * @param obj the object to query.
+ * @return Elementary widget name, or @c NULL if not a valid widget.
+ */
+EAPI const char *
+elm_object_widget_type_get(const Evas_Object *obj)
+{
+   return elm_widget_type_get(obj);
+}
+
+/**
+ * Send a signal to the widget edje object.
+ *
+ * This function sends a signal to the edje object of the obj. An edje program
+ * can respond to a signal by specifying matching 'signal' and
+ * 'source' fields.
+ *
+ * @param obj The object
+ * @param emission The signal's name.
+ * @param source The signal's source.
+ * @ingroup General
+ */
+EAPI void elm_object_signal_emit(Evas_Object *obj, const char *emission, const char *source)
+{
+    elm_widget_signal_emit(obj, emission, source);
+}
+
+/**
+ * Add a callback for a signal emitted by widget edje object.
+ *
+ * This function connects a callback function to a signal emitted by the
+ * edje object of the obj.
+ * Globs can occur in either the emission or source name.
+ *
+ * @param obj The object
+ * @param emission The signal's name.
+ * @param source The signal's source.
+ * @param func The callback function to be executed when the signal is
+ * emitted.
+ * @param data A pointer to data to pass in to the callback function.
+ * @ingroup General
+ */
+EAPI void elm_object_signal_listen(Evas_Object *obj, const char *emission, const char *source, void (*func) (void *data, Evas_Object *o, const char *emission, const char *source), void *data)
+{
+    elm_widget_signal_listen(obj, emission, source, func, data);
+}
+
+/**
+ * Remove a signal-triggered callback from an widget edje object.
+ *
+ * This function removes a callback, previoulsy attached to a signal emitted
+ * by the edje object of the obj.
+ * The parameters emission, source and func must match exactly those passed to
+ * a previous call to elm_object_signal_listen(). The data pointer that
+ * was passed to this call will be returned.
+ *
+ * @param obj The object
+ * @param emission The signal's name.
+ * @param source The signal's source.
+ * @param func The callback function to be executed when the signal is
+ * emitted.
+ * @return The data pointer
+ * @ingroup General
+ */
+EAPI void *elm_object_signal_unlisten(Evas_Object *obj, const char *emission, const char *source, void (*func) (void *data, Evas_Object *o, const char *emission, const char *source))
+{
+    return elm_widget_signal_unlisten(obj, emission, source, func);
+}
