@@ -37,6 +37,7 @@ struct _Item
    Eina_Bool popme : 1;
 };
 
+static const char *widtype = NULL;
 static void _del_hook(Evas_Object *obj);
 static void _theme_hook(Evas_Object *obj);
 static void _sizing_eval(Evas_Object *obj);
@@ -47,7 +48,7 @@ static void
 _del_hook(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
-
+   if (!wd) return;
    free(wd);
 }
 
@@ -57,7 +58,7 @@ _theme_hook(Evas_Object *obj)
    Widget_Data *wd = elm_widget_data_get(obj);
    Eina_List *l;
    Item *it;
-
+   if (!wd) return;
    EINA_LIST_FOREACH(wd->stack, l, it)
      edje_object_scale_set(it->base, elm_widget_scale_get(obj) * 
                            _elm_config->scale);
@@ -71,7 +72,7 @@ _sizing_eval(Evas_Object *obj)
    Evas_Coord minw = -1, minh = -1;
    Eina_List *l;
    Item *it;
-
+   if (!wd) return;
    EINA_LIST_FOREACH(wd->stack, l, it)
      {
 	if (it->minw > minw) minw = it->minw;
@@ -82,11 +83,10 @@ _sizing_eval(Evas_Object *obj)
 }
 
 static void
-_changed_size_hints(void *data, Evas *e, Evas_Object *obj, void *event_info)
+_changed_size_hints(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Item *it = data;
    Evas_Coord minw = -1, minh = -1;
-
    evas_object_size_hint_min_get(it->content, &minw, &minh);
    // FIXME: why is this needed? how does edje get this unswallowed or
    // lose its callbacks to edje
@@ -99,8 +99,8 @@ static void
 _eval_top(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
-   Item *it, *ittop;
-
+   Item *ittop;
+   if (!wd) return;
    if (!wd->stack) return;
    ittop = eina_list_last(wd->stack)->data;
    if (ittop != wd->top)
@@ -133,26 +133,26 @@ _eval_top(Evas_Object *obj)
 }
 
 static void
-_move(void *data, Evas *e, Evas_Object *obj, void *event_info)
+_move(void *data, Evas *e __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
 {
    Widget_Data *wd = elm_widget_data_get(data);
    Evas_Coord x, y;
    Eina_List *l;
    Item *it;
-
+   if (!wd) return;
    evas_object_geometry_get(obj, &x, &y, NULL, NULL);
    EINA_LIST_FOREACH(wd->stack, l, it)
      evas_object_move(it->base, x, y);
 }
 
 static void
-_sub_del(void *data, Evas_Object *obj, void *event_info)
+_sub_del(void *data, Evas_Object *obj __UNUSED__, void *event_info)
 {
    Widget_Data *wd = elm_widget_data_get(data);
    Evas_Object *sub = event_info;
    Eina_List *l;
    Item *it;
-
+   if (!wd) return;
    EINA_LIST_FOREACH(wd->stack, l, it)
      {
 	if (it->content == sub)
@@ -169,24 +169,22 @@ _sub_del(void *data, Evas_Object *obj, void *event_info)
 }
 
 static void
-_resize(void *data, Evas *e, Evas_Object *obj, void *event_info)
-{
+_resize(void *data, Evas *e __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
+{   
    Widget_Data *wd = elm_widget_data_get(data);
    Evas_Coord w, h;
    Eina_List *l;
    Item *it;
-
+   if (!wd) return;
    evas_object_geometry_get(obj, NULL, NULL, &w, &h);
-   EINA_LIST_FOREACH(wd->stack, l, it)
-     evas_object_resize(it->base, w, h);
+   EINA_LIST_FOREACH(wd->stack, l, it) evas_object_resize(it->base, w, h);
 }
 
 static void
-_signal_hide_finished(void *data, Evas_Object *obj, const char *emission, const char *source)
+_signal_hide_finished(void *data, Evas_Object *obj __UNUSED__, const char *emission __UNUSED__, const char *source __UNUSED__)
 {
    Item *it = data;
    Evas_Object *obj2 = it->obj;
-
    evas_object_hide(it->base);
    edje_object_signal_emit(it->base, "elm,action,reset", "elm");
    evas_object_smart_callback_call(obj2, "hide,finished", it->content);
@@ -213,6 +211,7 @@ elm_pager_add(Evas_Object *parent)
    wd = ELM_NEW(Widget_Data);
    e = evas_object_evas_get(parent);
    obj = elm_widget_add(e);
+   ELM_SET_WIDTYPE(widtype, "pager");
    elm_widget_type_set(obj, "pager");
    elm_widget_sub_object_add(parent, obj);
    elm_widget_data_set(obj, wd);
@@ -251,10 +250,11 @@ elm_pager_add(Evas_Object *parent)
 EAPI void
 elm_pager_content_push(Evas_Object *obj, Evas_Object *content)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
    Item *it = ELM_NEW(Item);
    Evas_Coord x, y, w, h;
-
+   if (!wd) return;
    if (!it) return;
    it->obj = obj;
    it->content = content;
@@ -266,13 +266,13 @@ elm_pager_content_push(Evas_Object *obj, Evas_Object *content)
    evas_object_clip_set(it->base, wd->clip);
    elm_widget_sub_object_add(obj, it->base);
    elm_widget_sub_object_add(obj, it->content);
-   _elm_theme_set(it->base,  "pager", "base", elm_widget_style_get(obj));
+   _elm_theme_object_set(obj, it->base,  "pager", "base", elm_widget_style_get(obj));
    edje_object_signal_callback_add(it->base, "elm,action,hide,finished", "", 
                                    _signal_hide_finished, it);
-   edje_object_part_swallow(it->base, "elm.swallow.content", it->content);
    evas_object_event_callback_add(it->content,
                                   EVAS_CALLBACK_CHANGED_SIZE_HINTS,
 				  _changed_size_hints, it);
+   edje_object_part_swallow(it->base, "elm.swallow.content", it->content);
    edje_object_size_min_calc(it->base, &it->minw, &it->minh);
    evas_object_show(it->content);
    wd->stack = eina_list_append(wd->stack, it);
@@ -294,10 +294,11 @@ elm_pager_content_push(Evas_Object *obj, Evas_Object *content)
 EAPI void
 elm_pager_content_pop(Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
    Eina_List *ll;
    Item *it;
-
+   if (!wd) return;
    if (!wd->stack) return;
    it = eina_list_last(wd->stack)->data;
    it->popme = EINA_TRUE;
@@ -344,10 +345,11 @@ elm_pager_content_pop(Evas_Object *obj)
 EAPI void
 elm_pager_content_promote(Evas_Object *obj, Evas_Object *content)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
    Eina_List *l;
    Item *it;
-
+   if (!wd) return;
    EINA_LIST_FOREACH(wd->stack, l, it)
      {
 	if (it->content == content)
@@ -369,11 +371,12 @@ elm_pager_content_promote(Evas_Object *obj, Evas_Object *content)
  * @ingroup Pager
  */
 EAPI Evas_Object *
-elm_pager_content_bottom_get(Evas_Object *obj)
+elm_pager_content_bottom_get(const Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
    Widget_Data *wd = elm_widget_data_get(obj);
    Item *it;
-
+   if (!wd) return NULL;
    if (!wd->stack) return NULL;
    it = wd->stack->data;
    return it->content;
@@ -388,11 +391,12 @@ elm_pager_content_bottom_get(Evas_Object *obj)
  * @ingroup Pager
  */
 EAPI Evas_Object *
-elm_pager_content_top_get(Evas_Object *obj)
+elm_pager_content_top_get(const Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
    Widget_Data *wd = elm_widget_data_get(obj);
    Item *it;
-
+   if (!wd) return NULL;
    if (!wd->stack) return NULL;
    it = eina_list_last(wd->stack)->data;
    return it->content;
