@@ -22,6 +22,7 @@ struct _Widget_Data
    int size;
 };
 
+static const char *widtype = NULL;
 static void _del_hook(Evas_Object *obj);
 static void _theme_hook(Evas_Object *obj);
 static void _sizing_eval(Evas_Object *obj);
@@ -31,6 +32,7 @@ static void
 _del_hook(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    free(wd);
 }
 
@@ -38,6 +40,7 @@ static void
 _theme_hook(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    edje_object_scale_set(wd->frm, elm_widget_scale_get(obj) * _elm_config->scale);
    _sizing_eval(obj);
 }
@@ -47,7 +50,7 @@ _sizing_eval(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    Evas_Coord minw = -1, minh = -1, maxw = -1, maxh = -1;
-
+   if (!wd) return;
    evas_object_size_hint_min_set(wd->img,
 				 wd->size * elm_widget_scale_get(obj) * _elm_config->scale,
 				 wd->size * elm_widget_scale_get(obj) * _elm_config->scale);
@@ -62,7 +65,7 @@ _sizing_eval(Evas_Object *obj)
 }
 
 static void
-_mouse_up(void *data, Evas *e, Evas_Object *obj, void *event_info)
+_mouse_up(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    evas_object_smart_callback_call(data, "clicked", NULL);
 }
@@ -85,6 +88,7 @@ elm_photo_add(Evas_Object *parent)
    wd = ELM_NEW(Widget_Data);
    e = evas_object_evas_get(parent);
    obj = elm_widget_add(e);
+   ELM_SET_WIDTYPE(widtype, "photo");
    elm_widget_type_set(obj, "photo");
    elm_widget_sub_object_add(parent, obj);
    elm_widget_data_set(obj, wd);
@@ -93,7 +97,7 @@ elm_photo_add(Evas_Object *parent)
    elm_widget_can_focus_set(obj, 0);
 
    wd->frm = edje_object_add(e);
-   _elm_theme_set(wd->frm, "photo", "base", "default");
+   _elm_theme_object_set(obj, wd->frm, "photo", "base", "default");
    elm_widget_resize_object_set(obj, wd->frm);
 
    wd->img = _els_smart_icon_add(e);
@@ -128,11 +132,13 @@ elm_photo_add(Evas_Object *parent)
 EAPI Eina_Bool
 elm_photo_file_set(Evas_Object *obj, const char *file)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) EINA_FALSE;
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return EINA_FALSE;
    if (!_els_smart_icon_file_key_set(wd->img, file, NULL))
-     return 0;
+     return EINA_FALSE;
    _sizing_eval(obj);
-   return 1;
+   return EINA_TRUE;
 }
 
 /**
@@ -146,7 +152,9 @@ elm_photo_file_set(Evas_Object *obj, const char *file)
 EAPI void
 elm_photo_size_set(Evas_Object *obj, int size)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    if (size < 0) size = 0;
    wd->size = size;
    _sizing_eval(obj);
