@@ -42,7 +42,7 @@ struct _Group
 
 struct _Widget_Data
 {
-   Evas_Object *chk;
+   Evas_Object *radio;
    Evas_Object *icon;
    int value;
    const char *label;
@@ -50,6 +50,7 @@ struct _Widget_Data
    Group *group;
 };
 
+static const char *widtype = NULL;
 static void _state_set(Evas_Object *obj, Eina_Bool state);
 static void _del_hook(Evas_Object *obj);
 static void _theme_hook(Evas_Object *obj);
@@ -58,12 +59,18 @@ static void _sizing_eval(Evas_Object *obj);
 static void _changed_size_hints(void *data, Evas *e, Evas_Object *obj, void *event_info);
 static void _sub_del(void *data, Evas_Object *obj, void *event_info);
 static void _signal_radio_on(void *data, Evas_Object *obj, const char *emission, const char *source);
-static void _signal_radio_toggle(void *data, Evas_Object *obj, const char *emission, const char *source);
+
+static const char SIG_CHANGED[] = "changed";
+static const Evas_Smart_Cb_Description _signals[] = {
+  {SIG_CHANGED, ""},
+  {NULL, NULL}
+};
 
 static void
 _del_hook(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    if (wd->label) eina_stringshare_del(wd->label);
    wd->group->radios = eina_list_remove(wd->group->radios, obj);
    if (!wd->group->radios) free(wd->group);
@@ -75,22 +82,23 @@ static void
 _theme_hook(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
-   _elm_theme_set(wd->chk, "radio", "base", elm_widget_style_get(obj));
+   if (!wd) return;
+   _elm_theme_object_set(obj, wd->radio, "radio", "base", elm_widget_style_get(obj));
    if (wd->icon)
-     edje_object_signal_emit(wd->chk, "elm,state,icon,visible", "elm");
+     edje_object_signal_emit(wd->radio, "elm,state,icon,visible", "elm");
    else
-     edje_object_signal_emit(wd->chk, "elm,state,icon,hidden", "elm");
+     edje_object_signal_emit(wd->radio, "elm,state,icon,hidden", "elm");
    if (wd->state)
-     edje_object_signal_emit(wd->chk, "elm,state,radio,on", "elm");
+     edje_object_signal_emit(wd->radio, "elm,state,radio,on", "elm");
    else
-     edje_object_signal_emit(wd->chk, "elm,state,radio,off", "elm");
+     edje_object_signal_emit(wd->radio, "elm,state,radio,off", "elm");
    if (wd->label)
-     edje_object_signal_emit(wd->chk, "elm,state,text,visible", "elm");
+     edje_object_signal_emit(wd->radio, "elm,state,text,visible", "elm");
    else
-     edje_object_signal_emit(wd->chk, "elm,state,text,hidden", "elm");
-   edje_object_part_text_set(wd->chk, "elm.text", wd->label);
-   edje_object_message_signal_process(wd->chk);
-   edje_object_scale_set(wd->chk, elm_widget_scale_get(obj) * _elm_config->scale);
+     edje_object_signal_emit(wd->radio, "elm,state,text,hidden", "elm");
+   edje_object_part_text_set(wd->radio, "elm.text", wd->label);
+   edje_object_message_signal_process(wd->radio);
+   edje_object_scale_set(wd->radio, elm_widget_scale_get(obj) * _elm_config->scale);
    _sizing_eval(obj);
 }
 
@@ -98,13 +106,14 @@ static void
 _disable_hook(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    if (elm_widget_disabled_get(obj))
      {
-        edje_object_signal_emit(wd->chk, "elm,state,disabled", "elm");
+        edje_object_signal_emit(wd->radio, "elm,state,disabled", "elm");
         if (wd->state) _state_set(obj, 0);
      }
    else
-     edje_object_signal_emit(wd->chk, "elm,state,enabled", "elm");
+     edje_object_signal_emit(wd->radio, "elm,state,enabled", "elm");
 }
 
 static void
@@ -112,30 +121,32 @@ _sizing_eval(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    Evas_Coord minw = -1, minh = -1;
-
+   if (!wd) return;
    elm_coords_finger_size_adjust(1, &minw, 1, &minh);
-   edje_object_size_min_restricted_calc(wd->chk, &minw, &minh, minw, minh);
+   edje_object_size_min_restricted_calc(wd->radio, &minw, &minh, minw, minh);
    elm_coords_finger_size_adjust(1, &minw, 1, &minh);
    evas_object_size_hint_min_set(obj, minw, minh);
    evas_object_size_hint_max_set(obj, -1, -1);
 }
 
 static void
-_changed_size_hints(void *data, Evas *e, Evas_Object *obj, void *event_info)
+_changed_size_hints(void *data, Evas *e __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
 {
    Widget_Data *wd = elm_widget_data_get(data);
+   if (!wd) return;
    if (obj != wd->icon) return;
    _sizing_eval(data);
 }
 
 static void
-_sub_del(void *data, Evas_Object *obj, void *event_info)
+_sub_del(void *data __UNUSED__, Evas_Object *obj, void *event_info)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    Evas_Object *sub = event_info;
+   if (!wd) return;
    if (sub == wd->icon)
      {
-	edje_object_signal_emit(wd->chk, "elm,state,icon,hidden", "elm");
+	edje_object_signal_emit(wd->radio, "elm,state,icon,hidden", "elm");
 	evas_object_event_callback_del_full
 	  (sub, EVAS_CALLBACK_CHANGED_SIZE_HINTS, _changed_size_hints, obj);
 	wd->icon = NULL;
@@ -147,13 +158,14 @@ static void
 _state_set(Evas_Object *obj, Eina_Bool state)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
    if ((state != wd->state) && (!elm_widget_disabled_get(obj)))
      {
 	wd->state = state;
 	if (wd->state)
-	  edje_object_signal_emit(wd->chk, "elm,state,radio,on", "elm");
+	  edje_object_signal_emit(wd->radio, "elm,state,radio,on", "elm");
 	else
-	  edje_object_signal_emit(wd->chk, "elm,state,radio,off", "elm");
+	  edje_object_signal_emit(wd->radio, "elm,state,radio,off", "elm");
      }
 }
 
@@ -161,9 +173,8 @@ static void
 _state_set_all(Widget_Data *wd)
 {
    const Eina_List *l;
-   Evas_Object *child, *selected;
+   Evas_Object *child, *selected = NULL;
    Eina_Bool disabled = EINA_FALSE;
-
    EINA_LIST_FOREACH(wd->group->radios, l, child)
      {
 	Widget_Data *wd2 = elm_widget_data_get(child);
@@ -175,19 +186,19 @@ _state_set_all(Widget_Data *wd)
           }
 	else _state_set(child, 0);
      }
-
-   if (disabled) _state_set(selected, 1);
+   if (disabled && selected) _state_set(selected, 1);
 }
 
 static void
-_signal_radio_on(void *data, Evas_Object *obj, const char *emission, const char *source)
+_signal_radio_on(void *data, Evas_Object *obj __UNUSED__, const char *emission __UNUSED__, const char *source __UNUSED__)
 {
    Widget_Data *wd = elm_widget_data_get(data);
+   if (!wd) return;
    if (wd->group->value == wd->value) return;
    wd->group->value = wd->value;
    if (wd->group->valuep) *(wd->group->valuep) = wd->group->value;
    _state_set_all(wd);
-   evas_object_smart_callback_call(data, "changed", NULL);
+   evas_object_smart_callback_call(data, SIG_CHANGED, NULL);
 }
 
 /**
@@ -208,6 +219,7 @@ elm_radio_add(Evas_Object *parent)
    wd = ELM_NEW(Widget_Data);
    e = evas_object_evas_get(parent);
    obj = elm_widget_add(e);
+   ELM_SET_WIDTYPE(widtype, "radio");
    elm_widget_type_set(obj, "radio");
    elm_widget_sub_object_add(parent, obj);
    elm_widget_data_set(obj, wd);
@@ -215,11 +227,11 @@ elm_radio_add(Evas_Object *parent)
    elm_widget_theme_hook_set(obj, _theme_hook);
    elm_widget_disable_hook_set(obj, _disable_hook);
 
-   wd->chk = edje_object_add(e);
-   _elm_theme_set(wd->chk, "radio", "base", "default");
-   edje_object_signal_callback_add(wd->chk, "elm,action,radio,on", "", _signal_radio_on, obj);
-   edje_object_signal_callback_add(wd->chk, "elm,action,radio,toggle", "", _signal_radio_on, obj);
-   elm_widget_resize_object_set(obj, wd->chk);
+   wd->radio = edje_object_add(e);
+   _elm_theme_object_set(obj, wd->radio, "radio", "base", "default");
+   edje_object_signal_callback_add(wd->radio, "elm,action,radio,on", "", _signal_radio_on, obj);
+   edje_object_signal_callback_add(wd->radio, "elm,action,radio,toggle", "", _signal_radio_on, obj);
+   elm_widget_resize_object_set(obj, wd->radio);
 
    evas_object_smart_callback_add(obj, "sub-object-del", _sub_del, obj);
 
@@ -228,6 +240,10 @@ elm_radio_add(Evas_Object *parent)
    wd->state = 0;
 
    _sizing_eval(obj);
+
+   // TODO: convert Elementary to subclassing of Evas_Smart_Class
+   // TODO: and save some bytes, making descriptions per-class and not instance!
+   evas_object_smart_callbacks_descriptions_set(obj, _signals);
    return obj;
 }
 
@@ -242,23 +258,21 @@ elm_radio_add(Evas_Object *parent)
 EAPI void
 elm_radio_label_set(Evas_Object *obj, const char *label)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-   Evas_Coord mw, mh;
-
-   if (wd->label) eina_stringshare_del(wd->label);
+   if (!wd) return;
+   eina_stringshare_replace(&wd->label, label);
    if (label)
      {
-	wd->label = eina_stringshare_add(label);
-	edje_object_signal_emit(wd->chk, "elm,state,text,visible", "elm");
-	edje_object_message_signal_process(wd->chk);
+	edje_object_signal_emit(wd->radio, "elm,state,text,visible", "elm");
+	edje_object_message_signal_process(wd->radio);
      }
    else
      {
-	wd->label = NULL;
-	edje_object_signal_emit(wd->chk, "elm,state,text,hidden", "elm");
-	edje_object_message_signal_process(wd->chk);
+	edje_object_signal_emit(wd->radio, "elm,state,text,hidden", "elm");
+	edje_object_message_signal_process(wd->radio);
      }
-   edje_object_part_text_set(wd->chk, "elm.text", label);
+   edje_object_part_text_set(wd->radio, "elm.text", label);
    _sizing_eval(obj);
 }
 
@@ -270,22 +284,19 @@ elm_radio_label_set(Evas_Object *obj, const char *label)
  *
  * @ingroup Radio
  */
-EAPI const char*
-elm_radio_label_get(Evas_Object *obj)
+EAPI const char *
+elm_radio_label_get(const Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return NULL;
-
    return wd->label;
 }
 
 /**
  * Set the icon object of the radio object
  *
- * Once the icon object is set, it will become a child of the radio object and
- * be deleted when the radio object is deleted. If another icon object is set
- * then the previous one becomes orophaned and will no longer be deleted along
- * with the radio.
+ * Once the icon object is set, a previously set one will be deleted.
  *
  * @param obj The radio object
  * @param icon The icon object
@@ -295,19 +306,22 @@ elm_radio_label_get(Evas_Object *obj)
 EAPI void
 elm_radio_icon_set(Evas_Object *obj, Evas_Object *icon)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-   if ((wd->icon != icon) && (wd->icon))
-     elm_widget_sub_object_del(obj, wd->icon);
+   if (!wd) return;
+   if (wd->icon == icon) return;
+   if (wd->icon) evas_object_del(wd->icon);
    wd->icon = icon;
    if (icon)
      {
 	elm_widget_sub_object_add(obj, icon);
-	edje_object_part_swallow(wd->chk, "elm.swallow.content", icon);
-	edje_object_signal_emit(wd->chk, "elm,state,icon,visible", "elm");
 	evas_object_event_callback_add(icon, EVAS_CALLBACK_CHANGED_SIZE_HINTS,
 				       _changed_size_hints, obj);
-	_sizing_eval(obj);
+	edje_object_part_swallow(wd->radio, "elm.swallow.content", icon);
+	edje_object_signal_emit(wd->radio, "elm,state,icon,visible", "elm");
+	edje_object_message_signal_process(wd->radio);
      }
+   _sizing_eval(obj);
 }
 
 /**
@@ -319,8 +333,9 @@ elm_radio_icon_set(Evas_Object *obj, Evas_Object *icon)
  * @ingroup Radio
  */
 EAPI Evas_Object *
-elm_radio_icon_get(Evas_Object *obj)
+elm_radio_icon_get(const Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return NULL;
    return wd->icon;
@@ -342,14 +357,26 @@ elm_radio_icon_get(Evas_Object *obj)
 EAPI void
 elm_radio_group_add(Evas_Object *obj, Evas_Object *group)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
    Widget_Data *wd2 = elm_widget_data_get(group);
-   Eina_Bool state = EINA_FALSE;
-   if (wd->group == wd2->group) return;
-   wd->group->radios = eina_list_remove(wd->group->radios, obj);
-   if (!wd->group->radios) free(wd->group);
-   wd->group = wd2->group;
-   wd->group->radios = eina_list_append(wd->group->radios, obj);
+   if (!wd) return;
+   if (!wd2)
+     {
+	if (eina_list_count(wd->group->radios) == 1)
+	  return;
+	wd->group->radios = eina_list_remove(wd->group->radios, obj);
+	wd->group = calloc(1, sizeof(Group));
+	wd->group->radios = eina_list_append(wd->group->radios, obj);
+     }
+   else if (wd->group == wd2->group) return;
+   else
+     {
+	wd->group->radios = eina_list_remove(wd->group->radios, obj);
+	if (!wd->group->radios) free(wd->group);
+	wd->group = wd2->group;
+	wd->group->radios = eina_list_append(wd->group->radios, obj);
+     }
    if (wd->value == wd->group->value) _state_set(obj, 1);
    else _state_set(obj, 0);
 }
@@ -367,8 +394,9 @@ elm_radio_group_add(Evas_Object *obj, Evas_Object *group)
 EAPI void
 elm_radio_state_value_set(Evas_Object *obj, int value)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-//   if (wd->value == value) return;
+   if (!wd) return;
    wd->value = value;
    if (wd->value == wd->group->value) _state_set(obj, 1);
    else _state_set(obj, 0);
@@ -388,9 +416,9 @@ elm_radio_state_value_set(Evas_Object *obj, int value)
 EAPI void
 elm_radio_value_set(Evas_Object *obj, int value)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-   const Eina_List *l;
-   Evas_Object *child;
+   if (!wd) return;
    if (value == wd->group->value) return;
    wd->group->value = value;
    if (wd->group->valuep) *(wd->group->valuep) = wd->group->value;
@@ -408,7 +436,9 @@ elm_radio_value_set(Evas_Object *obj, int value)
 EAPI int
 elm_radio_value_get(const Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) 0;
    Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return 0;
    return wd->group->value;
 }
 
@@ -430,16 +460,14 @@ elm_radio_value_get(const Evas_Object *obj)
 EAPI void
 elm_radio_value_pointer_set(Evas_Object *obj, int *valuep)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-
+   if (!wd) return;
    if (valuep)
      {
 	wd->group->valuep = valuep;
 	if (*(wd->group->valuep) != wd->group->value)
 	  {
-	     const Eina_List *l;
-	     Evas_Object *child;
-
 	     wd->group->value = *(wd->group->valuep);
 	     _state_set_all(wd);
 	  }
