@@ -36,18 +36,42 @@ struct _Widget_Data
 {
    Evas_Object *scr;
    Evas_Object *content;
+   const char *widget_name, *widget_base;
    Eina_Bool min_w : 1;
    Eina_Bool min_h : 1;
    double pagerel_h, pagerel_v;
    Evas_Coord pagesize_h, pagesize_v;
 };
 
+static const char *widtype = NULL;
 static void _del_hook(Evas_Object *obj);
 static void _theme_hook(Evas_Object *obj);
 static void _show_region_hook(void *data, Evas_Object *obj);
 static void _sizing_eval(Evas_Object *obj);
 static void _sub_del(void *data, Evas_Object *obj, void *event_info);
 
+static const char SIG_SCROLL[] = "scroll";
+static const char SIG_SCROLL_ANIM_START[] = "scroll,anim,start";
+static const char SIG_SCROLL_ANIM_STOP[] = "scroll,anim,stop";
+static const char SIG_SCROLL_DRAG_START[] = "scroll,drag,start";
+static const char SIG_SCROLL_DRAG_STOP[] = "scroll,drag,stop";
+static const char SIG_EDGE_LEFT[] = "edge,left";
+static const char SIG_EDGE_RIGHT[] = "edge,right";
+static const char SIG_EDGE_TOP[] = "edge,top";
+static const char SIG_EDGE_BOTTOM[] = "edge,bottom";
+static const Evas_Smart_Cb_Description _signals[] = {
+  {SIG_SCROLL, ""},
+  {SIG_SCROLL_ANIM_START, ""},
+  {SIG_SCROLL_ANIM_STOP, ""},
+  {SIG_SCROLL_DRAG_START, ""},
+  {SIG_SCROLL_DRAG_STOP, ""},
+  {SIG_EDGE_LEFT, ""},
+  {SIG_EDGE_RIGHT, ""},
+  {SIG_EDGE_TOP, ""},
+  {SIG_EDGE_BOTTOM, ""},
+  {NULL, NULL}
+};
+
 static void
 _del_hook(Evas_Object *obj)
 {
@@ -61,63 +85,84 @@ _theme_hook(Evas_Object *obj)
 {
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
-   elm_smart_scroller_theme_set(wd->scr, "scroller", "base", elm_widget_style_get(obj));
-   edje_object_scale_set(wd->scr, elm_widget_scale_get(obj) * _elm_config->scale);
+   if (wd->scr)
+     {
+        elm_smart_scroller_object_theme_set(obj, wd->scr, 
+                                            wd->widget_name, wd->widget_base,
+                                            elm_widget_style_get(obj));
+//        edje_object_scale_set(wd->scr, elm_widget_scale_get(obj) * _elm_config->scale);
+     }
    _sizing_eval(obj);
 }
 
+static void
+_signal_emit_hook(Evas_Object *obj, const char *emission, const char *source)
+{
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
+   edje_object_signal_emit(elm_smart_scroller_edje_object_get(wd->scr),
+	 emission, source);
+}
+
 static void
 _show_region_hook(void *data, Evas_Object *obj)
 {
+   
    Widget_Data *wd = elm_widget_data_get(data);
    Evas_Coord x, y, w, h;
    if (!wd) return;
    elm_widget_show_region_get(obj, &x, &y, &w, &h);
-   elm_smart_scroller_child_region_show(wd->scr, x, y, w, h);
+   if (wd->scr)
+     elm_smart_scroller_child_region_show(wd->scr, x, y, w, h);
 }
 
 static void
 _sizing_eval(Evas_Object *obj)
 {
+   
    Widget_Data *wd = elm_widget_data_get(obj);
    Evas_Coord  vw, vh, minw, minh, maxw, maxh, w, h, vmw, vmh;
-   double xw, xy;
+   double xw, yw;
 
    if (!wd) return;
    evas_object_size_hint_min_get(wd->content, &minw, &minh);
    evas_object_size_hint_max_get(wd->content, &maxw, &maxh);
-   evas_object_size_hint_weight_get(wd->content, &xw, &xy);
-   elm_smart_scroller_child_viewport_size_get(wd->scr, &vw, &vh);
-   if (xw > 0.0)
+   evas_object_size_hint_weight_get(wd->content, &xw, &yw);
+   if (wd->scr)
      {
-	if ((minw > 0) && (vw < minw)) vw = minw;
-	else if ((maxw > 0) && (vw > maxw)) vw = maxw;
+        elm_smart_scroller_child_viewport_size_get(wd->scr, &vw, &vh);
+        if (xw > 0.0)
+          {
+             if ((minw > 0) && (vw < minw)) vw = minw;
+             else if ((maxw > 0) && (vw > maxw)) vw = maxw;
+          }
+        else if (minw > 0) vw = minw;
+        if (yw > 0.0)
+          {
+             if ((minh > 0) && (vh < minh)) vh = minh;
+             else if ((maxh > 0) && (vh > maxh)) vh = maxh;
+          }
+        else if (minh > 0) vh = minh;
+        evas_object_resize(wd->content, vw, vh);
+        w = -1;
+        h = -1;
+        edje_object_size_min_calc(elm_smart_scroller_edje_object_get(wd->scr), &vmw, &vmh);
+        if (wd->min_w) w = vmw + minw;
+        if (wd->min_h) h = vmh + minh;
+        evas_object_size_hint_min_set(obj, w, h);
      }
-   else if (minw > 0) vw = minw;
-   if (xy > 0.0)
-     {
-	if ((minh > 0) && (vh < minh)) vh = minh;
-	else if ((maxh > 0) && (vh > maxh)) vh = maxh;
-     }
-   else if (minh > 0) vh = minh;
-   evas_object_resize(wd->content, vw, vh);
-   w = -1;
-   h = -1;
-   edje_object_size_min_calc(elm_smart_scroller_edje_object_get(wd->scr), &vmw, &vmh);
-   if (wd->min_w) w = vmw + minw;
-   if (wd->min_h) h = vmh + minh;
-   evas_object_size_hint_min_set(obj, w, h);
 }
 
 static void
-_changed_size_hints(void *data, Evas *e, Evas_Object *obj, void *event_info)
+_changed_size_hints(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    _sizing_eval(data);
 }
 
 static void
-_sub_del(void *data, Evas_Object *obj, void *event_info)
+_sub_del(void *data __UNUSED__, Evas_Object *obj, void *event_info)
 {
+   
    Widget_Data *wd = elm_widget_data_get(obj);
    Evas_Object *sub = event_info;
 
@@ -130,102 +175,112 @@ _sub_del(void *data, Evas_Object *obj, void *event_info)
 	wd->content = NULL;
 	_sizing_eval(obj);
      }
+   else if (sub == wd->scr)
+     wd->scr = NULL;
 }
 
 static void
-_hold_on(void *data, Evas_Object *obj, void *event_info)
+_hold_on(void *data __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
 {
+   
    Widget_Data *wd = elm_widget_data_get(obj);
 
    if (!wd) return;
-   elm_smart_scroller_hold_set(wd->scr, 1);
+   if (wd->scr)
+     elm_smart_scroller_hold_set(wd->scr, 1);
 }
 
 static void
-_hold_off(void *data, Evas_Object *obj, void *event_info)
+_hold_off(void *data __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
 {
+   
    Widget_Data *wd = elm_widget_data_get(obj);
 
    if (!wd) return;
-   elm_smart_scroller_hold_set(wd->scr, 0);
+   if (wd->scr)
+     elm_smart_scroller_hold_set(wd->scr, 0);
 }
 
 static void
-_freeze_on(void *data, Evas_Object *obj, void *event_info)
+_freeze_on(void *data __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
 {
+   
    Widget_Data *wd = elm_widget_data_get(obj);
 
    if (!wd) return;
-   elm_smart_scroller_freeze_set(wd->scr, 1);
+   if (wd->scr)
+     elm_smart_scroller_freeze_set(wd->scr, 1);
 }
 
 static void
-_freeze_off(void *data, Evas_Object *obj, void *event_info)
+_freeze_off(void *data __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
 {
+   
    Widget_Data *wd = elm_widget_data_get(obj);
 
    if (!wd) return;
-   elm_smart_scroller_freeze_set(wd->scr, 0);
+   if (wd->scr)
+     elm_smart_scroller_freeze_set(wd->scr, 0);
 }
 
 static void
-_resize(void *data, Evas *e, Evas_Object *obj, void *event_info)
+_resize(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    _sizing_eval(data);
 }
 
 static void
-_edge_left(void *data, Evas_Object *obj, void *event_info)
+_edge_left(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-   evas_object_smart_callback_call(data, "edge,left", NULL);
+   evas_object_smart_callback_call(data, SIG_EDGE_LEFT, NULL);
 }
 
 static void
-_edge_right(void *data, Evas_Object *obj, void *event_info)
+_edge_right(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-   evas_object_smart_callback_call(data, "edge,right", NULL);
+   evas_object_smart_callback_call(data, SIG_EDGE_RIGHT, NULL);
 }
 
 static void
-_edge_top(void *data, Evas_Object *obj, void *event_info)
+_edge_top(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-   evas_object_smart_callback_call(data, "edge,top", NULL);
+   evas_object_smart_callback_call(data, SIG_EDGE_TOP, NULL);
 }
 
 static void
-_edge_bottom(void *data, Evas_Object *obj, void *event_info)
+_edge_bottom(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-   evas_object_smart_callback_call(data, "edge,bottom", NULL);
+   evas_object_smart_callback_call(data, SIG_EDGE_BOTTOM, NULL);
 }
 
 static void
-_scroll(void *data, Evas_Object *obj, void *event_info)
+_scroll(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-   evas_object_smart_callback_call(data, "scroll", NULL);
+   evas_object_smart_callback_call(data, SIG_SCROLL, NULL);
 }
 
 static void
-_scroll_anim_start(void *data, Evas_Object *obj, void *event_info)
+_scroll_anim_start(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-   evas_object_smart_callback_call(data, "scroll,anim,start", NULL);
+   evas_object_smart_callback_call(data, SIG_SCROLL_ANIM_START, NULL);
 }
 
 static void
-_scroll_anim_stop(void *data, Evas_Object *obj, void *event_info)
+_scroll_anim_stop(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-   evas_object_smart_callback_call(data, "scroll,anim,stop", NULL);
+   evas_object_smart_callback_call(data, SIG_SCROLL_ANIM_STOP, NULL);
 }
 
 static void
-_scroll_drag_start(void *data, Evas_Object *obj, void *event_info)
+_scroll_drag_start(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-   evas_object_smart_callback_call(data, "scroll,drag,start", NULL);
+   evas_object_smart_callback_call(data, SIG_SCROLL_DRAG_START, NULL);
 }
 
 static void
-_scroll_drag_stop(void *data, Evas_Object *obj, void *event_info)
+_scroll_drag_stop(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
-   evas_object_smart_callback_call(data, "scroll,drag,stop", NULL);
+   evas_object_smart_callback_call(data, SIG_SCROLL_DRAG_STOP, NULL);
 }
 
 /**
@@ -242,18 +297,27 @@ elm_scroller_add(Evas_Object *parent)
    Evas_Object *obj;
    Evas *e;
    Widget_Data *wd;
-   Evas_Coord vw, vh, minw, minh;
+   Evas_Coord minw, minh;
 
    wd = ELM_NEW(Widget_Data);
    e = evas_object_evas_get(parent);
    obj = elm_widget_add(e);
+   ELM_SET_WIDTYPE(widtype, "scroller");
    elm_widget_type_set(obj, "scroller");
    elm_widget_sub_object_add(parent, obj);
    elm_widget_data_set(obj, wd);
    elm_widget_del_hook_set(obj, _del_hook);
    elm_widget_theme_hook_set(obj, _theme_hook);
+   elm_widget_signal_emit_hook_set(obj, _signal_emit_hook);
 
+   wd->widget_name = eina_stringshare_add("scroller");
+   wd->widget_base = eina_stringshare_add("base");
+   
    wd->scr = elm_smart_scroller_add(e);
+   elm_smart_scroller_widget_set(wd->scr, obj);
+   elm_smart_scroller_object_theme_set(obj, wd->scr, 
+                                       wd->widget_name, wd->widget_base,
+                                       elm_widget_style_get(obj));
    elm_widget_resize_object_set(obj, wd->scr);
    evas_object_event_callback_add(wd->scr, EVAS_CALLBACK_CHANGED_SIZE_HINTS,
 				  _changed_size_hints, obj);
@@ -279,14 +343,20 @@ elm_scroller_add(Evas_Object *parent)
    evas_object_smart_callback_add(wd->scr, "drag,stop", _scroll_drag_stop, obj);
 
    _sizing_eval(obj);
+
+   // TODO: convert Elementary to subclassing of Evas_Smart_Class
+   // TODO: and save some bytes, making descriptions per-class and not instance!
+   evas_object_smart_callbacks_descriptions_set(obj, _signals);
    return obj;
 }
 
 
 /**
- * Set the content object
+ * Set the content of the scroller widget (the object to be scrolled around).
  *
- * XXX
+ * Once the content object is set, a previously set one will be deleted.
+ * If you want to keep that old content object, use the
+ * elm_scroller_content_unset() function.
  *
  * @param obj The scroller object
  * @param content The new content object
@@ -296,20 +366,68 @@ elm_scroller_add(Evas_Object *parent)
 EAPI void
 elm_scroller_content_set(Evas_Object *obj, Evas_Object *content)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
-   if ((wd->content != content) && (wd->content))
-     elm_widget_sub_object_del(obj, wd->content);
+   if (wd->content == content) return;
+   if (wd->content) evas_object_del(wd->content);
    wd->content = content;
    if (content)
      {
 	elm_widget_on_show_region_hook_set(content, _show_region_hook, obj);
 	elm_widget_sub_object_add(obj, content);
-	elm_smart_scroller_child_set(wd->scr, content);
+        if (wd->scr)
+          elm_smart_scroller_child_set(wd->scr, content);
 	evas_object_event_callback_add(content, EVAS_CALLBACK_CHANGED_SIZE_HINTS,
 				       _changed_size_hints, obj);
-	_sizing_eval(obj);
      }
+   _sizing_eval(obj);
+}
+
+/**
+ * Unset the content of the scroller widget
+ *
+ * Unparent and return the content object which was set for this widget
+ *
+ * @param obj The slider objecet
+ * @return The content that was being used
+ *
+ * @ingroup Scroller
+ */
+EAPI Evas_Object *
+elm_scroller_content_unset(Evas_Object *obj)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   Evas_Object *content;
+   if (!wd) return NULL;
+   if (!wd->content) return NULL;
+   content = wd->content;
+   elm_widget_sub_object_del(obj, wd->content);
+   edje_object_part_unswallow(wd->scr, wd->content);
+   wd->content = NULL;
+   return content;
+}
+
+/**
+ * Set custom theme elements for the scroller
+ * 
+ * @param obj The scroller object
+ * @param widget The widget name to use (default is "scroller")
+ * @param base The base name to use (default is "base")
+ *
+ * @ingroup Scroller
+ */
+EAPI void
+elm_scroller_custom_widget_base_theme_set(Evas_Object *obj, const char *widget, const char *base)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype);
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return;
+   if ((!widget) || (!base)) return;
+   if (eina_stringshare_replace(&wd->widget_name, widget) |
+       eina_stringshare_replace(&wd->widget_base, base))
+     _theme_hook(obj);
 }
 
 /**
@@ -328,6 +446,7 @@ elm_scroller_content_set(Evas_Object *obj, Evas_Object *content)
 EAPI void
 elm_scroller_content_min_limit(Evas_Object *obj, Eina_Bool w, Eina_Bool h)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
    wd->min_w = w;
@@ -353,9 +472,11 @@ elm_scroller_content_min_limit(Evas_Object *obj, Eina_Bool w, Eina_Bool h)
 EAPI void
 elm_scroller_region_show(Evas_Object *obj, Evas_Coord x, Evas_Coord y, Evas_Coord w, Evas_Coord h)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
-   elm_smart_scroller_child_region_show(wd->scr, x, y, w, h);
+   if (wd->scr)
+     elm_smart_scroller_child_region_show(wd->scr, x, y, w, h);
 }
 
 /**
@@ -376,6 +497,7 @@ elm_scroller_region_show(Evas_Object *obj, Evas_Coord x, Evas_Coord y, Evas_Coor
 EAPI void
 elm_scroller_policy_set(Evas_Object *obj, Elm_Scroller_Policy policy_h, Elm_Scroller_Policy policy_v)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
    const Elm_Scroller_Policy map[3] =
      {
@@ -384,9 +506,22 @@ elm_scroller_policy_set(Evas_Object *obj, Elm_Scroller_Policy policy_h, Elm_Scro
 	  ELM_SMART_SCROLLER_POLICY_OFF
      };
    if (!wd) return;
-   if ((policy_h < 0) || (policy_h >= 3) || (policy_v < 0) || (policy_v >= 3))
-     return;
-   elm_smart_scroller_policy_set(wd->scr, map[policy_h], map[policy_v]);
+   if ((policy_h >= 3) || (policy_v >= 3)) return;
+   if (wd->scr)
+     elm_smart_scroller_policy_set(wd->scr, map[policy_h], map[policy_v]);
+}
+
+EAPI void
+elm_scroller_policy_get(const Evas_Object *obj, Elm_Scroller_Policy *policy_h, Elm_Scroller_Policy *policy_v)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype);
+   Widget_Data *wd = elm_widget_data_get(obj);
+   Elm_Smart_Scroller_Policy s_policy_h, s_policy_v;
+   if (!wd) return;
+   if (wd->scr)
+     elm_smart_scroller_policy_get(wd->scr, &s_policy_h, &s_policy_v);
+   *policy_h = (Elm_Scroller_Policy) s_policy_h;
+   *policy_v = (Elm_Scroller_Policy) s_policy_v;
 }
 
 /**
@@ -405,12 +540,16 @@ elm_scroller_policy_set(Evas_Object *obj, Elm_Scroller_Policy policy_h, Elm_Scro
  * @ingroup Scroller
  */
 EAPI void
-elm_scroller_region_get(Evas_Object *obj, Evas_Coord *x, Evas_Coord *y, Evas_Coord *w, Evas_Coord *h)
+elm_scroller_region_get(const Evas_Object *obj, Evas_Coord *x, Evas_Coord *y, Evas_Coord *w, Evas_Coord *h)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
-   if ((x) && (y)) elm_smart_scroller_child_pos_get(wd->scr, x, y);
-   if ((w) && (h)) elm_smart_scroller_child_viewport_size_get(wd->scr, w, h);
+   if (wd->scr)
+     {
+        if ((x) && (y)) elm_smart_scroller_child_pos_get(wd->scr, x, y);
+        if ((w) && (h)) elm_smart_scroller_child_viewport_size_get(wd->scr, w, h);
+     }
 }
 
 /**
@@ -427,8 +566,9 @@ elm_scroller_region_get(Evas_Object *obj, Evas_Coord *x, Evas_Coord *y, Evas_Coo
  * @ingroup Scroller
  */
 EAPI void
-elm_scroller_child_size_get(Evas_Object *obj, Evas_Coord *w, Evas_Coord *h)
+elm_scroller_child_size_get(const Evas_Object *obj, Evas_Coord *w, Evas_Coord *h)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
    evas_object_geometry_get(wd->content, NULL, NULL, w, h);
@@ -455,9 +595,11 @@ elm_scroller_child_size_get(Evas_Object *obj, Evas_Coord *w, Evas_Coord *h)
 EAPI void
 elm_scroller_bounce_set(Evas_Object *obj, Eina_Bool h_bounce, Eina_Bool v_bounce)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
-   elm_smart_scroller_bounce_allow_set(wd->scr, h_bounce, v_bounce);
+   if (wd->scr)
+     elm_smart_scroller_bounce_allow_set(wd->scr, h_bounce, v_bounce);
 }
 
 /**
@@ -483,12 +625,14 @@ elm_scroller_bounce_set(Evas_Object *obj, Eina_Bool h_bounce, Eina_Bool v_bounce
 EAPI void
 elm_scroller_page_relative_set(Evas_Object *obj, double h_pagerel, double v_pagerel)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
    wd->pagerel_h = h_pagerel;
    wd->pagerel_v = v_pagerel;
-   elm_smart_scroller_paging_set(wd->scr, wd->pagerel_h, wd->pagerel_v,
-                                 wd->pagesize_h, wd->pagesize_v);
+   if (wd->scr)
+     elm_smart_scroller_paging_set(wd->scr, wd->pagerel_h, wd->pagerel_v,
+                                   wd->pagesize_h, wd->pagesize_v);
 }
 
 /**
@@ -507,12 +651,14 @@ elm_scroller_page_relative_set(Evas_Object *obj, double h_pagerel, double v_page
 EAPI void
 elm_scroller_page_size_set(Evas_Object *obj, Evas_Coord h_pagesize, Evas_Coord v_pagesize)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
    wd->pagesize_h = h_pagesize;
    wd->pagesize_v = v_pagesize;
-   elm_smart_scroller_paging_set(wd->scr, wd->pagerel_h, wd->pagerel_v,
-                                 wd->pagesize_h, wd->pagesize_v);
+   if (wd->scr)
+     elm_smart_scroller_paging_set(wd->scr, wd->pagerel_h, wd->pagerel_v,
+                                   wd->pagesize_h, wd->pagesize_v);
 }
 
 /**
@@ -537,7 +683,9 @@ elm_scroller_page_size_set(Evas_Object *obj, Evas_Coord h_pagesize, Evas_Coord v
 EAPI void
 elm_scroller_region_bring_in(Evas_Object *obj, Evas_Coord x, Evas_Coord y, Evas_Coord w, Evas_Coord h)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
    if (!wd) return;
-   elm_smart_scroller_region_bring_in(wd->scr, x, y, w, h);
+   if (wd->scr)
+     elm_smart_scroller_region_bring_in(wd->scr, x, y, w, h);
 }
