@@ -4,6 +4,13 @@
 #include <Elementary.h>
 #include "elm_priv.h"
 
+/**
+ * @defgroup Toolbar Toolbar
+ *
+ * A toolbar is a widget that displays a list of buttons inside
+ * a box.  It is scrollable, and only one item can be selected at a time.
+ */
+
 typedef struct _Widget_Data Widget_Data;
 
 struct _Widget_Data
@@ -23,8 +30,8 @@ struct _Elm_Toolbar_Item
    Evas_Object *base;
    const char *label;
    Evas_Object *icon;
-   void (*func) (void *data, Evas_Object *obj, void *event_info);
-   void (*del_cb) (void *data, Evas_Object *obj, void *event_info);
+   Evas_Smart_Cb func;
+   Evas_Smart_Cb del_cb;
    const void *data;
    Eina_Bool selected : 1;
    Eina_Bool disabled : 1;
@@ -33,6 +40,7 @@ struct _Elm_Toolbar_Item
    Evas_Object *o_menu;
 };
 
+static const char *widtype = NULL;
 static void _item_show(Elm_Toolbar_Item *it);
 static void _item_select(Elm_Toolbar_Item *it);
 static void _item_disable(Elm_Toolbar_Item *it, Eina_Bool disabled);
@@ -43,7 +51,6 @@ static void _sizing_eval(Evas_Object *obj);
 static void _resize(void *data, Evas *e, Evas_Object *obj, void *event_info);
 static void _menu_move_resize(void *data, Evas *e, Evas_Object *obj, void *event_info);
 static void _menu_hide(void *data, Evas *e, Evas_Object *obj, void *event_info);
-static void _menu_item_select(void *data, Evas_Object *obj, void *event_info);
 static void _layout(Evas_Object *o, Evas_Object_Box_Data *priv, void *data);
 
 static void
@@ -63,9 +70,8 @@ _item_select(Elm_Toolbar_Item *it)
 {
    Elm_Toolbar_Item *it2;
    Widget_Data *wd = elm_widget_data_get(it->obj);
-   Evas_Object *obj2, *bt, *hv, *bx, *ic;
+   Evas_Object *obj2;
    const Eina_List *l;
-   Evas_Coord x,y,w,h;
 
    if (!wd) return;
    if ((it->selected) || (it->disabled) || (it->separator)) return;
@@ -97,14 +103,14 @@ _item_select(Elm_Toolbar_Item *it)
 }
 
 static void
-_menu_hide(void *data, Evas *e, Evas_Object *obj, void *event_info)
+_menu_hide(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Elm_Toolbar_Item *it = data;
    elm_toolbar_item_unselect_all(it->obj);
 }
 
 static void
-_menu_del(void *data, Evas *e, Evas_Object *obj, void *event_info)
+_menu_del(void *data, Evas *e __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
 {
    // avoid hide being emitted during object deletion
    evas_object_event_callback_del_full
@@ -112,7 +118,7 @@ _menu_del(void *data, Evas *e, Evas_Object *obj, void *event_info)
 }
 
 static void
-_menu_move_resize(void *data, Evas *e, Evas_Object *obj, void *event_info)
+_menu_move_resize(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
     Elm_Toolbar_Item *it = data;
     Evas_Coord x,y,w,h;
@@ -171,11 +177,12 @@ _theme_hook(Evas_Object *obj)
    const Eina_List *l;
    Elm_Toolbar_Item *it;
    const char *style = elm_widget_style_get(obj);
-   int scale = 0;
+   double scale = 0;
 
    if (!wd) return;
+   elm_smart_scroller_object_theme_set(obj, wd->scr, "toolbar", "base", elm_widget_style_get(obj));
    scale = (elm_widget_scale_get(obj) * _elm_config->scale);
-   edje_object_scale_set(wd->scr, scale);
+//   edje_object_scale_set(wd->scr, scale);
    EINA_LIST_FOREACH(wd->items, l, it)
      {
         Evas_Coord mw, mh;
@@ -187,7 +194,7 @@ _theme_hook(Evas_Object *obj)
                edje_object_signal_emit(it->base, "elm,state,selected", "elm");
              if (it->disabled)
                edje_object_signal_emit(it->base, "elm,state,disabled", "elm");
-             _elm_theme_set(it->base, "toolbar", "item", style);
+             _elm_theme_object_set(obj, it->base, "toolbar", "item", style);
              if (it->icon)
                {
                   int ms = 0;
@@ -201,7 +208,7 @@ _theme_hook(Evas_Object *obj)
              edje_object_part_text_set(it->base, "elm.text", it->label);
           }
         else
-          _elm_theme_set(it->base, "toolbar", "separator", style);
+          _elm_theme_object_set(obj, it->base, "toolbar", "separator", style);
 
 	mw = mh = -1;
 	if (!it->separator)
@@ -246,10 +253,10 @@ _sizing_eval(Evas_Object *obj)
 }
 
 static void
-_resize(void *data, Evas *e, Evas_Object *obj, void *event_info)
+_resize(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Widget_Data *wd = elm_widget_data_get(data);
-   Evas_Coord mw, mh, vw, vh, x, y, w, h;
+   Evas_Coord mw, mh, vw, vh, w, h;
    const Eina_List *l;
    Elm_Toolbar_Item *it;
 
@@ -272,7 +279,7 @@ _resize(void *data, Evas *e, Evas_Object *obj, void *event_info)
 }
 
 static void
-_select(void *data, Evas_Object *obj, const char *emission, const char *source)
+_select(void *data, Evas_Object *obj __UNUSED__, const char *emission __UNUSED__, const char *source __UNUSED__)
 {
    _item_select(data);
 }
@@ -281,10 +288,19 @@ static void
 _layout(Evas_Object *o, Evas_Object_Box_Data *priv, void *data)
 {
    Widget_Data *wd = data;
-
+   if (!wd) return;
    _els_box_layout(o, priv, 1, wd->homogeneous);
 }
 
+/**
+ * Add a toolbar object to @p parent.
+ *
+ * @param parent The parent object
+ *
+ * @return The created object, or NULL on failure
+ *
+ * @ingroup Toolbar
+ */
 EAPI Evas_Object *
 elm_toolbar_add(Evas_Object *parent)
 {
@@ -295,6 +311,7 @@ elm_toolbar_add(Evas_Object *parent)
    wd = ELM_NEW(Widget_Data);
    e = evas_object_evas_get(parent);
    obj = elm_widget_add(e);
+   ELM_SET_WIDTYPE(widtype, "toolbar");
    elm_widget_type_set(obj, "toolbar");
    elm_widget_sub_object_add(parent, obj);
    elm_widget_data_set(obj, wd);
@@ -304,8 +321,9 @@ elm_toolbar_add(Evas_Object *parent)
    elm_widget_can_focus_set(obj, 0);
 
    wd->scr = elm_smart_scroller_add(e);
+   elm_smart_scroller_widget_set(wd->scr, obj);
+   elm_smart_scroller_object_theme_set(obj, wd->scr, "toolbar", "base", "default");
    elm_smart_scroller_bounce_allow_set(wd->scr, 1, 0);
-   elm_smart_scroller_theme_set(wd->scr, "toolbar", "base", "default");
    elm_widget_resize_object_set(obj, wd->scr);
    elm_smart_scroller_policy_set(wd->scr,
 				 ELM_SMART_SCROLLER_POLICY_AUTO,
@@ -329,11 +347,19 @@ elm_toolbar_add(Evas_Object *parent)
    return obj;
 }
 
+/**
+ * Set the icon size (in pixels) for the toolbar.
+ *
+ * @param obj The toolbar object
+ * @param icon_size The icon size in pixels
+ *
+ * @ingroup Toolbar
+ */
 EAPI void
 elm_toolbar_icon_size_set(Evas_Object *obj, int icon_size)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
-
    if (!wd) return;
    if (icon_size > 48) return;
    if (wd->icon_size == icon_size) return;
@@ -341,18 +367,40 @@ elm_toolbar_icon_size_set(Evas_Object *obj, int icon_size)
    _theme_hook(obj);
 }
 
+/**
+ * Get the icon size (in pixels) for the toolbar.
+ *
+ * @param obj The toolbar object
+ * @return The icon size in pixels
+ *
+ * @ingroup Toolbar
+ */
 EAPI int
-elm_toolbar_icon_size_get(Evas_Object *obj)
+elm_toolbar_icon_size_get(const Evas_Object *obj)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) 0;
    Widget_Data *wd = elm_widget_data_get(obj);
-
    if (!wd) return 0;
    return wd->icon_size;
 }
 
+/**
+ * Add an item to the toolbar.
+ *
+ * @param obj The toolbar object
+ * @param icon The icon object of the item
+ * @param label The label of the item
+ * @param func The function to call when the item is clicked
+ * @param data The data to associate with the item
+ *
+ * @return The toolbar item, or NULL upon failure
+ *
+ * @ingroup Toolbar
+ */
 EAPI Elm_Toolbar_Item *
-elm_toolbar_item_add(Evas_Object *obj, Evas_Object *icon, const char *label, void (*func) (void *data, Evas_Object *obj, void *event_info), const void *data)
+elm_toolbar_item_add(Evas_Object *obj, Evas_Object *icon, const char *label, Evas_Smart_Cb func, const void *data)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
    Widget_Data *wd = elm_widget_data_get(obj);
    Evas_Coord mw, mh;
    Elm_Toolbar_Item *it;
@@ -368,7 +416,7 @@ elm_toolbar_item_add(Evas_Object *obj, Evas_Object *icon, const char *label, voi
    it->data = data;
    it->separator = EINA_FALSE;
    it->base = edje_object_add(evas_object_evas_get(obj));
-   _elm_theme_set(it->base, "toolbar", "item", elm_widget_style_get(obj));
+   _elm_theme_object_set(obj, it->base, "toolbar", "item", elm_widget_style_get(obj));
    edje_object_signal_callback_add(it->base, "elm,action,click", "elm",
 				   _select, it);
    elm_widget_sub_object_add(obj, it->base);
@@ -397,6 +445,14 @@ elm_toolbar_item_add(Evas_Object *obj, Evas_Object *icon, const char *label, voi
    return it;
 }
 
+/**
+ * Get the icon associated with @p item.
+ *
+ * @param item The toolbar item
+ * @return The icon object
+ *
+ * @ingroup Toolbar
+ */
 EAPI Evas_Object *
 elm_toolbar_item_icon_get(Elm_Toolbar_Item *item)
 {
@@ -404,21 +460,51 @@ elm_toolbar_item_icon_get(Elm_Toolbar_Item *item)
    return item->icon;
 }
 
+/**
+ * Get the label associated with @p item.
+ *
+ * @param item The toolbar item
+ * @return The label
+ *
+ * @ingroup Toolbar
+ */
 EAPI const char *
-elm_toolbar_item_label_get(Elm_Toolbar_Item *item)
+elm_toolbar_item_label_get(const Elm_Toolbar_Item *item)
 {
    if (!item) return NULL;
    return item->label;
 }
 
+/**
+ * Get the selected state of @p item.
+ *
+ * @param item The toolbar item
+ * @return If true, the item is selected
+ *
+ * @ingroup Toolbar
+ */
+EAPI Eina_Bool
+elm_toolbar_item_selected_get(const Elm_Toolbar_Item *item)
+{
+   if (!item) return EINA_FALSE;
+   return item->selected;
+}
+
+/**
+ * Set the label associated with @p item.
+ *
+ * @param item The toolbar item
+ * @param label The label of @p item
+ *
+ * @ingroup Toolbar
+ */
 EAPI void
 elm_toolbar_item_label_set(Elm_Toolbar_Item *item, const char *label)
 {
    Evas_Coord mw = -1, mh = -1;
 
    if (!item) return;
-   eina_stringshare_del(item->label);
-   item->label = eina_stringshare_add(label);
+   eina_stringshare_replace(&item->label, label);
    edje_object_part_text_set(item->base, "elm.text", item->label);
 
    elm_coords_finger_size_adjust(1, &mw, 1, &mh);
@@ -429,6 +515,14 @@ elm_toolbar_item_label_set(Elm_Toolbar_Item *item, const char *label)
    evas_object_size_hint_min_set(item->base, mw, mh);
 }
 
+
+/**
+ * Delete a toolbar item.
+ *
+ * @param it The toolbar item
+ *
+ * @ingroup Toolbar
+ */
 EAPI void
 elm_toolbar_item_del(Elm_Toolbar_Item *it)
 {
@@ -451,14 +545,21 @@ elm_toolbar_item_del(Elm_Toolbar_Item *it)
  * @param it The item to set the callback on
  * @param func The function called
  *
- * @ingroup Hoversel
+ * @ingroup Toolbar
  */
 EAPI void
-elm_toolbar_item_del_cb_set(Elm_Toolbar_Item *it, void (*func)(void *data, Evas_Object *obj, void *event_info))
+elm_toolbar_item_del_cb_set(Elm_Toolbar_Item *it, Evas_Smart_Cb func)
 {
    it->del_cb = func;
 }
 
+/**
+ * Select the toolbar item @p item.
+ *
+ * @param item The toolbar item
+ *
+ * @ingroup Toolbar
+ */
 EAPI void
 elm_toolbar_item_select(Elm_Toolbar_Item *item)
 {
@@ -466,6 +567,14 @@ elm_toolbar_item_select(Elm_Toolbar_Item *item)
    _item_select(item);
 }
 
+/**
+ * Get the disabled state of @p item.
+ *
+ * @param item The toolbar item
+ * @return If true, the item is disabled
+ *
+ * @ingroup Toolbar
+ */
 EAPI Eina_Bool
 elm_toolbar_item_disabled_get(Elm_Toolbar_Item *item)
 {
@@ -473,6 +582,14 @@ elm_toolbar_item_disabled_get(Elm_Toolbar_Item *item)
    return item->disabled;
 }
 
+/**
+ * Set the disabled state of @p item.
+ *
+ * @param item The toolbar item
+ * @param disabled If true, the item is disabled
+ *
+ * @ingroup Toolbar
+ */
 EAPI void
 elm_toolbar_item_disabled_set(Elm_Toolbar_Item *item, Eina_Bool disabled)
 {
@@ -480,6 +597,14 @@ elm_toolbar_item_disabled_set(Elm_Toolbar_Item *item, Eina_Bool disabled)
    _item_disable(item, disabled);
 }
 
+/**
+ * Get the separator state of @p item.
+ *
+ * @param item The toolbar item
+ * @param separator If true, the item is a separator
+ *
+ * @ingroup Toolbar
+ */
 EAPI void
 elm_toolbar_item_separator_set(Elm_Toolbar_Item *item, Eina_Bool separator)
 {
@@ -489,6 +614,14 @@ elm_toolbar_item_separator_set(Elm_Toolbar_Item *item, Eina_Bool separator)
    _theme_hook(item->obj);
 }
 
+/**
+ * Set the separator state of @p item.
+ *
+ * @param item The toolbar item
+ * @return If true, the item is a separator
+ *
+ * @ingroup Toolbar
+ */
 EAPI Eina_Bool
 elm_toolbar_item_separator_get(Elm_Toolbar_Item *item)
 {
@@ -496,19 +629,56 @@ elm_toolbar_item_separator_get(Elm_Toolbar_Item *item)
    return item->separator;
 }
 
+/**
+ * Set the scrollable state of toolbar @p obj.
+ *
+ * @param obj The toolbar object
+ * @param scrollable If true, the toolbar will be scrollable
+ *
+ * @ingroup Toolbar
+ */
 EAPI void
 elm_toolbar_scrollable_set(Evas_Object *obj, Eina_Bool scrollable)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
 
    if (!wd) return;
    wd->scrollable = scrollable;
+   elm_smart_scroller_bounce_allow_set(wd->scr, wd->scrollable, 0);
    _sizing_eval(obj);
 }
 
+/**
+ * Get the scrollable state of toolbar @p obj.
+ *
+ * @param obj The toolbar object
+ * @return If true, the toolbar is scrollable
+ *
+ * @ingroup Toolbar
+ */
+EAPI Eina_Bool
+elm_toolbar_scrollable_get(Evas_Object *obj)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype) EINA_FALSE;
+   Widget_Data *wd = elm_widget_data_get(obj);
+
+   if (!wd) return EINA_FALSE;
+   return wd->scrollable;
+}
+
+/**
+ * Set the homogenous mode of toolbar @p obj.
+ *
+ * @param obj The toolbar object
+ * @param homogenous If true, the toolbar items will be uniform in size
+ *
+ * @ingroup Toolbar
+ */
 EAPI void
 elm_toolbar_homogenous_set(Evas_Object *obj, Eina_Bool homogenous)
 {
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
 
    if (!wd) return;
@@ -516,11 +686,38 @@ elm_toolbar_homogenous_set(Evas_Object *obj, Eina_Bool homogenous)
    evas_object_smart_calculate(wd->bx);
 }
 
+/**
+ * Get the homogenous mode of toolbar @p obj.
+ *
+ * @param obj The toolbar object
+ * @return If true, the toolbar items are uniform in size
+ *
+ * @ingroup Toolbar
+ */
+EAPI Eina_Bool
+elm_toolbar_homogenous_get(Evas_Object *obj)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype) EINA_FALSE;
+   Widget_Data *wd = elm_widget_data_get(obj);
+
+   if (!wd) return EINA_FALSE;
+   return wd->homogeneous;
+}
+
+/**
+ * Set the parent object of the toolbar menu
+ *
+ * @param obj The toolbar object
+ * @param parent The parent of the menu object
+ *
+ * @ingroup Toolbar
+ */
 EAPI void
 elm_toolbar_menu_parent_set(Evas_Object *obj, Evas_Object *parent)
 {
    Eina_List *l;
    Elm_Toolbar_Item *it;
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
 
    if ((!wd) || (!parent)) return;
@@ -532,11 +729,37 @@ elm_toolbar_menu_parent_set(Evas_Object *obj, Evas_Object *parent)
      }
 }
 
+/**
+ * Get the parent object of the toolbar menu
+ *
+ * @param obj The toolbar object
+ * @return The parent of the menu object
+ *
+ * @ingroup Toolbar
+ */
+EAPI Evas_Object *
+elm_toolbar_menu_parent_get(Evas_Object *obj)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return NULL;
+
+   return wd->menu_parent;
+}
+
+/**
+ * Unselect all of the items in the toolbar.
+ *
+ * @param obj The toolbar object
+ *
+ * @ingroup Toolbar
+ */
 EAPI void
 elm_toolbar_item_unselect_all(Evas_Object *obj)
 {
    Eina_List *l;
    Elm_Toolbar_Item *it;
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
 
    if (!wd) return;
@@ -552,24 +775,66 @@ elm_toolbar_item_unselect_all(Evas_Object *obj)
 }
 
 /**
- * Set if the alignment of the items.
+ * Unselect the specified toolbar item.
+ *
+ * @param item The toolbar item
+ *
+ * @ingroup Toolbar
+ */
+EAPI void
+elm_toolbar_item_unselect(Elm_Toolbar_Item *item)
+{
+   if ((!item) || (!item->selected)) return;
+   item->selected = EINA_FALSE;
+   edje_object_signal_emit(item->base, "elm,state,unselected", "elm");
+}
+
+/**
+ * Set the alignment of the items.
  *
  * @param obj The toolbar object
  * @param align The new alignment. (left) 0.0 ... 1.0 (right)
+ *
+ * @ingroup Toolbar
  */
 EAPI void
 elm_toolbar_align_set(Evas_Object *obj, double align)
 {
-   Eina_List *l;
-   Elm_Toolbar_Item *it;
+   ELM_CHECK_WIDTYPE(obj, widtype);
    Widget_Data *wd = elm_widget_data_get(obj);
 
    if (!wd) return;
-   if(wd->align != align)
+   if (wd->align != align)
      evas_object_size_hint_align_set(wd->bx, align, 0.5);
-    wd->align = align;
+   wd->align = align;
 }
 
+/**
+ * Get the alignment of the items.
+ *
+ * @param obj The toolbar object
+ * @return The alignment. (left) 0.0 ... 1.0 (right)
+ *
+ * @ingroup Toolbar
+ */
+EAPI double
+elm_toolbar_align_get(const Evas_Object *obj)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype) 0.0;
+   Widget_Data *wd = elm_widget_data_get(obj);
+
+   if (!wd) return 0.0;
+   return wd->align;
+}
+
+/**
+ * Set whether the toolbar item opens a menu.
+ *
+ * @param item The toolbar item
+ * @param menu If true, @p item will open a menu when selected
+ *
+ * @ingroup Toolbar
+ */
 EAPI void
 elm_toolbar_item_menu_set(Elm_Toolbar_Item *item, Eina_Bool menu)
 {
@@ -578,8 +843,7 @@ elm_toolbar_item_menu_set(Elm_Toolbar_Item *item, Eina_Bool menu)
    if ((!wd) || (!item)) return;
    if (item->menu == menu) return;
    item->menu = menu;
-
-   if(menu)
+   if (menu)
      {
 	item->o_menu = elm_menu_add(item->base);
 	if (wd->menu_parent)
@@ -595,12 +859,236 @@ elm_toolbar_item_menu_set(Elm_Toolbar_Item *item, Eina_Bool menu)
      }
 }
 
+
+/**
+ * Get whether the toolbar item opens a menu.
+ *
+ * @param item The toolbar item
+ * @return If true, @p item opens a menu when selected
+ *
+ * @ingroup Toolbar
+ */
 EAPI Evas_Object *
 elm_toolbar_item_menu_get(Elm_Toolbar_Item *item)
 {
-   if (!item) return;
+   if (!item) return NULL;
    Widget_Data *wd = elm_widget_data_get(item->obj);
-   if (!wd) return;
+   if (!wd) return NULL;
    elm_toolbar_item_menu_set(item, 1);
    return item->o_menu;
 }
+
+/**
+ * Return a list of all toolbar items.
+ *
+ * @param obj The toolbar object
+ *
+ * @return An Eina_List* of the toolbar items in @p obj
+ *
+ * @ingroup Toolbar
+ */
+EAPI Eina_List *
+elm_toolbar_item_get_all(Evas_Object *obj)
+{
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return NULL;
+
+   return wd->items;
+}
+
+/**
+ * Return the first toolbar item in the list of toolbar items.
+ *
+ * @param obj The toolbar object
+ *
+ * @return The first toolbar item, or NULL on failure
+ *
+ * @ingroup Toolbar
+ */
+EAPI Elm_Toolbar_Item *
+elm_toolbar_item_get_first(Evas_Object *obj)
+{
+   Elm_Toolbar_Item *it;
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return NULL;
+   if (!(it = eina_list_data_get(wd->items))) return NULL;
+
+   return it;
+}
+
+/**
+ * Return the last toolbar item in the list of toolbar items.
+ *
+ * @param obj The toolbar object
+ *
+ * @return The last toolbar item, or NULL on failure
+ *
+ * @ingroup Toolbar
+ */
+EAPI Elm_Toolbar_Item *
+elm_toolbar_item_get_last(Evas_Object *obj)
+{
+   Eina_List *last;
+   Elm_Toolbar_Item *it;
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return NULL;
+   if (!(last = eina_list_last(wd->items))) return NULL;
+   it = eina_list_data_get(last);
+
+   return it;
+}
+
+/**
+ * Return the next toolbar item (relative to the currently selected
+ * toolbar item) in the list of toolbar items.
+ *
+ * @param obj The toolbar object
+ *
+ * @return The next toolbar item, or NULL on failure
+ *
+ * @ingroup Toolbar
+ */
+EAPI Elm_Toolbar_Item *
+elm_toolbar_item_get_next(Evas_Object *obj)
+{
+   Eina_List *l, *l2;
+   Elm_Toolbar_Item *it, *next;
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
+   Widget_Data *wd = elm_widget_data_get(obj);
+
+   if (!wd) return NULL;
+   EINA_LIST_FOREACH_SAFE(wd->items, l, l2, it)
+     {
+        if (it->selected)
+          {
+	     if (!(next = eina_list_data_get(l2))) return NULL;
+	     return next;
+          }
+     }
+   return NULL;
+}
+
+/**
+ * Selects the next non-disabled, non-separator toolbar item in the list
+ * of toolbar items.
+ *
+ * @param obj The toolbar object
+ *
+ * @return The newly selected toolbar item, or NULL on failure
+ *
+ * @ingroup Toolbar
+ */
+EAPI Elm_Toolbar_Item *
+elm_toolbar_item_select_next(Evas_Object *obj)
+{
+   Eina_List *l, *l2;
+   Elm_Toolbar_Item *it, *next;
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
+   Widget_Data *wd = elm_widget_data_get(obj);
+
+   if (!wd) return NULL;
+   EINA_LIST_FOREACH(wd->items, l, it)
+     {
+        if (it->selected)
+          {
+             EINA_LIST_FOREACH(l, l2, next)
+               if ((!next->disabled) && (next->separator))
+                 {
+                    _item_select(next);
+                    return next;
+                 }
+          }
+     }
+  return NULL;
+}
+
+/**
+ * Selects the first non-disabled, non-separator toolbar item in the list
+ * of toolbar items.
+ *
+ * @param obj The toolbar object
+ *
+ * @return The newly selected toolbar item, or NULL on failure
+ *
+ * @ingroup Toolbar
+ */
+EAPI Elm_Toolbar_Item *
+elm_toolbar_item_select_first(Evas_Object *obj)
+{
+   Eina_List *l;
+   Elm_Toolbar_Item *it;
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return NULL;
+   EINA_LIST_FOREACH(wd->items, l, it)
+     {
+        if ((!it->disabled) && (!it->separator))
+          {
+             _item_select(it);
+             return it;
+          }
+     }
+
+   return NULL;
+}
+
+/**
+ * Selects the last non-disabled, non-separator toolbar item in the list
+ * of toolbar items.
+ *
+ * @param obj The toolbar object
+ *
+ * @return The newly selected toolbar item, or NULL on failure
+ *
+ * @ingroup Toolbar
+ */
+EAPI Elm_Toolbar_Item *
+elm_toolbar_item_select_last(Evas_Object *obj)
+{
+   Eina_List *l;
+   Elm_Toolbar_Item *it;
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   if (!wd) return NULL;
+   EINA_LIST_REVERSE_FOREACH(wd->items, l, it)
+     {
+        if ((!it->disabled) && (!it->separator))
+          {
+             _item_select(it);
+             return it;
+          }
+     }
+ 
+   return NULL;
+}
+
+/**
+ * Returns a pointer to a toolbar item by its label
+ *
+ * @param obj The toolbar object
+ * @param label The label of the item to find
+ *
+ * @return The pointer to the toolbar item matching @p label
+ * Returns NULL on failure.
+ *
+ * @ingroup Toolbar
+ */
+EAPI Elm_Toolbar_Item *
+elm_toolbar_item_find_by_label(Evas_Object *obj, const char *label)
+{
+   Eina_List *l;
+   Elm_Toolbar_Item *it;
+   ELM_CHECK_WIDTYPE(obj, widtype) NULL;
+   Widget_Data *wd = elm_widget_data_get(obj);
+   
+   if (!wd) return NULL;
+   EINA_LIST_FOREACH(wd->items, l, it)
+     {
+        if (!strcmp(it->label, label)) return it;
+     }
+
+   return NULL;
+}
