@@ -1,12 +1,11 @@
 #include <Elementary.h>
 #include "elm_priv.h"
 
-#define SMART_NAME "elm_widget"
+static const char SMART_NAME[] = "elm_widget";
+
 #define API_ENTRY \
    Smart_Data *sd = evas_object_smart_data_get(obj); \
-   if ((!obj) || (!sd) || \
-	  (evas_object_type_get(obj) && \
-	      strcmp(evas_object_type_get(obj), SMART_NAME)))
+   if ((!obj) || (!sd) || (!_elm_widget_is(obj)))
 #define INTERNAL_ENTRY \
    Smart_Data *sd = evas_object_smart_data_get(obj); \
    if (!sd) return;
@@ -28,6 +27,16 @@ struct _Smart_Data
    void         (*activate_func) (Evas_Object *obj);
    void         (*disable_func) (Evas_Object *obj);
    void         (*theme_func) (Evas_Object *obj);
+   void         (*signal_func) (Evas_Object *obj, const char *emission,
+	                        const char *source);
+   void         (*listen_func) (Evas_Object *obj, const char *emission,
+	                        const char *source, void (*func) (void *data,
+				   Evas_Object *o, const char *emission,
+				   const char *source), void *data);
+   void         *(*unlisten_func) (Evas_Object *obj, const char *emission,
+	                          const char *source, void (*func) (void *data,
+				     Evas_Object *o, const char *emission,
+				     const char *source));
    void         (*changed_func) (Evas_Object *obj);
    void         (*on_focus_func) (void *data, Evas_Object *obj);
    void          *on_focus_data;
@@ -40,11 +49,19 @@ struct _Smart_Data
    int            scroll_hold;
    int            scroll_freeze;
    double         scale;
+   Elm_Theme     *theme;
    const char    *style;
-   unsigned char  can_focus : 1;
-   unsigned char  child_can_focus : 1;
-   unsigned char  focused : 1;
-   unsigned char  disabled : 1;
+   unsigned int   focus_order;
+   
+   int            child_drag_x_locked;
+   int            child_drag_y_locked;
+   Eina_Bool      drag_x_locked : 1;
+   Eina_Bool      drag_y_locked : 1;
+   
+   Eina_Bool      can_focus : 1;
+   Eina_Bool      child_can_focus : 1;
+   Eina_Bool      focused : 1;
+   Eina_Bool      disabled : 1;
 };
 
 /* local subsystem functions */
@@ -60,12 +77,17 @@ static void _smart_clip_set(Evas_Object *obj, Evas_Object * clip);
 static void _smart_clip_unset(Evas_Object *obj);
 static void _smart_calculate(Evas_Object *obj);
 static void _smart_init(void);
+static inline Eina_Bool _elm_widget_is(const Evas_Object *obj);
+
+static void _if_focused_revert(Evas_Object *obj);
 
 /* local subsystem globals */
 static Evas_Smart *_e_smart = NULL;
 
+static unsigned int focus_order = 0;
+
 static void
-_sub_obj_del(void *data, Evas *e, Evas_Object *obj, void *event_info)
+_sub_obj_del(void *data, Evas *e __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
 {
    Smart_Data *sd = data;
 
@@ -79,25 +101,20 @@ _sub_obj_del(void *data, Evas *e, Evas_Object *obj, void *event_info)
 }
 
 static void
-_sub_obj_mouse_down(void *data, Evas *e, Evas_Object *obj, void *event_info)
+_sub_obj_mouse_down(void *data __UNUSED__, Evas *e __UNUSED__, Evas_Object *obj, void *event_info __UNUSED__)
 {
-   Smart_Data *sd = data;
-   Evas_Object *o;
-   const char *s;
-   
-   o = obj;
-   do {
-      s = evas_object_type_get(o);
-      if (!s) return;
-      if (!strcmp(s, "elm_widget")) break;
-      o = evas_object_smart_parent_get(o);
-   } while (o);
+   Evas_Object *o = obj;
+   do 
+     {
+        if (_elm_widget_is(o)) break;
+        o = evas_object_smart_parent_get(o);
+     }
+   while (o);
    if (!o) return;
    if (!elm_widget_can_focus_get(o)) return;
    elm_widget_focus_steal(o);
 }
 
-/* externally accessible functions */
 EAPI Evas_Object *
 elm_widget_add(Evas *evas)
 {
@@ -154,6 +171,27 @@ elm_widget_changed_hook_set(Evas_Object *obj, void (*func) (Evas_Object *obj))
    sd->changed_func = func;
 }
 
+EAPI void
+elm_widget_signal_emit_hook_set(Evas_Object *obj, void (*func) (Evas_Object *obj, const char *emission, const char *source))
+{
+   API_ENTRY return;
+   sd->signal_func = func;
+}
+
+EAPI void
+elm_widget_signal_listen_hook_set(Evas_Object *obj, void (*func) (Evas_Object *obj, const char *emission, const char *source, void (*func_cb) (void *data, Evas_Object *o, const char *emission, const char *source), void *data))
+{
+   API_ENTRY return;
+   sd->listen_func = func;
+}
+
+EAPI void
+elm_widget_signal_unlisten_hook_set(Evas_Object *obj, void *(*func) (Evas_Object *obj, const char *emission, const char *source, void (*func_cb) (void *data, Evas_Object *o, const char *emission, const char *source)))
+{
+   API_ENTRY return;
+   sd->unlisten_func = func;
+}
+
 EAPI void
 elm_widget_theme(Evas_Object *obj)
 {
@@ -211,13 +249,14 @@ elm_widget_sub_object_add(Evas_Object *obj, Evas_Object *sobj)
 {
    API_ENTRY return;
    double scale, pscale = elm_widget_scale_get(sobj);
+   Elm_Theme *th, *pth = elm_widget_theme_get(sobj);
 
    sd->subobjs = eina_list_append(sd->subobjs, sobj);
    if (!sd->child_can_focus)
      {
 	if (elm_widget_can_focus_get(sobj)) sd->child_can_focus = 1;
      }
-   if (!strcmp(evas_object_type_get(sobj), SMART_NAME))
+   if (_elm_widget_is(sobj))
      {
 	Smart_Data *sd2 = evas_object_smart_data_get(sobj);
 	if (sd2)
@@ -227,22 +266,41 @@ elm_widget_sub_object_add(Evas_Object *obj, Evas_Object *sobj)
 	     sd2->parent_obj = obj;
 	  }
      }
+   evas_object_data_set(sobj, "elm-parent", obj);
    evas_object_event_callback_add(sobj, EVAS_CALLBACK_DEL, _sub_obj_del, sd);
    evas_object_smart_callback_call(obj, "sub-object-add", sobj);
    scale = elm_widget_scale_get(sobj);
-   if (scale != pscale) elm_widget_theme(sobj);
+   th = elm_widget_theme_get(sobj);
+   if ((scale != pscale) || (th != pth)) elm_widget_theme(sobj);
 }
 
 EAPI void
 elm_widget_sub_object_del(Evas_Object *obj, Evas_Object *sobj)
 {
+   Evas_Object *sobj_parent;
    API_ENTRY return;
+   if (!sobj) return;
+
+   sobj_parent = evas_object_data_del(sobj, "elm-parent");
+   if (sobj_parent != obj)
+     {
+	static int abort_on_warn = -1;
+	ERR("removing sub object %p from parent %p, "
+	    "but elm-parent is different %p!",
+	    sobj, obj, sobj_parent);
+	if (EINA_UNLIKELY(abort_on_warn == -1))
+	  {
+	     if (getenv("ELM_ERROR_ABORT")) abort_on_warn = 1;
+	     else abort_on_warn = 0;
+	  }
+	if (abort_on_warn == 1) abort();
+     }
    sd->subobjs = eina_list_remove(sd->subobjs, sobj);
    if (!sd->child_can_focus)
      {
 	if (elm_widget_can_focus_get(sobj)) sd->child_can_focus = 0;
      }
-   if (!strcmp(evas_object_type_get(sobj), SMART_NAME))
+   if (_elm_widget_is(sobj))
      {
 	Smart_Data *sd2 = evas_object_smart_data_get(sobj);
 	if (sd2) sd2->parent_obj = NULL;
@@ -257,7 +315,8 @@ elm_widget_resize_object_set(Evas_Object *obj, Evas_Object *sobj)
    API_ENTRY return;
    if (sd->resize_obj)
      {
-	if (!strcmp(evas_object_type_get(sd->resize_obj), SMART_NAME))
+	evas_object_data_del(sd->resize_obj, "elm-parent");
+	if (_elm_widget_is(sd->resize_obj))
 	  {
 	     Smart_Data *sd2 = evas_object_smart_data_get(sd->resize_obj);
 	     if (sd2) sd2->parent_obj = NULL;
@@ -271,7 +330,7 @@ elm_widget_resize_object_set(Evas_Object *obj, Evas_Object *sobj)
    sd->resize_obj = sobj;
    if (sd->resize_obj)
      {
-	if (!strcmp(evas_object_type_get(sd->resize_obj), SMART_NAME))
+	if (_elm_widget_is(sd->resize_obj))
 	  {
 	     Smart_Data *sd2 = evas_object_smart_data_get(sd->resize_obj);
 	     if (sd2) sd2->parent_obj = obj;
@@ -282,6 +341,7 @@ elm_widget_resize_object_set(Evas_Object *obj, Evas_Object *sobj)
 	evas_object_event_callback_add(sobj, EVAS_CALLBACK_MOUSE_DOWN,
                                        _sub_obj_mouse_down, sd);
 	_smart_reconfigure(sd);
+	evas_object_data_set(sobj, "elm-parent", obj);
 	evas_object_smart_callback_call(obj, "sub-object-add", sobj);
      }
 }
@@ -345,9 +405,63 @@ elm_widget_focused_object_get(const Evas_Object *obj)
 EAPI Evas_Object *
 elm_widget_top_get(const Evas_Object *obj)
 {
+#if 1 // strict way  
    API_ENTRY return NULL;
    if (sd->parent_obj) return elm_widget_top_get(sd->parent_obj);
    return (Evas_Object *)obj;
+#else // loose way
+   Smart_Data *sd = evas_object_smart_data_get(obj);
+   Evas_Object *par;
+   
+   if (!obj) return NULL;
+   if ((sd) && _elm_widget_is(obj))
+     {
+        if ((sd->type) && (!strcmp(sd->type, "win"))) 
+          return (Evas_Object *)obj;
+        if (sd->parent_obj)
+          return elm_widget_top_get(sd->parent_obj);
+     }
+   par = evas_object_smart_parent_get(obj);
+   if (!par) return (Evas_Object *)obj;
+   return elm_widget_top_get(par);
+#endif   
+}
+
+EAPI Eina_Bool
+elm_widget_is(const Evas_Object *obj)
+{
+   return _elm_widget_is(obj);
+}
+
+EAPI Evas_Object *
+elm_widget_parent_widget_get(const Evas_Object *obj)
+{
+   Evas_Object *parent;
+
+   if (_elm_widget_is(obj))
+     {
+	Smart_Data *sd = evas_object_smart_data_get(obj);
+	if (!sd) return NULL;
+	parent = sd->parent_obj;
+     }
+   else
+     {
+	parent = evas_object_data_get(obj, "elm-parent");
+	if (!parent)
+	  parent = evas_object_smart_data_get(obj);
+     }
+
+   while (parent)
+     {
+	Evas_Object *elm_parent;
+	if (_elm_widget_is(parent)) break;
+	elm_parent = evas_object_data_get(parent, "elm-parent");
+	if (elm_parent)
+	  parent = elm_parent;
+	else
+	  parent = evas_object_smart_parent_get(parent);
+     }
+   return parent;
 }
 
 EAPI int
@@ -359,7 +473,12 @@ elm_widget_focus_jump(Evas_Object *obj, int forward)
    /* if it has a focus func its an end-point widget like a button */
    if (sd->focus_func)
      {
-	if (!sd->focused) sd->focused = 1;
+	if (!sd->focused)
+          {
+             focus_order++;
+             sd->focus_order = focus_order;
+             sd->focused = 1;
+          }
 	else sd->focused = 0;
 	if (sd->on_focus_func) sd->on_focus_func(sd->on_focus_data, obj);
 	sd->focus_func(obj);
@@ -375,8 +494,6 @@ elm_widget_focus_jump(Evas_Object *obj, int forward)
 	if (!sd->focused)
 	  {
 	     elm_widget_focus_set(obj, forward);
-	     sd->focused = 1;
-	     if (sd->on_focus_func) sd->on_focus_func(sd->on_focus_data, obj);
 	     return 1;
 	  }
 	else
@@ -506,12 +623,38 @@ elm_widget_focus_jump(Evas_Object *obj, int forward)
    return 0;
 }
 
+EAPI void
+elm_widget_signal_emit(Evas_Object *obj, const char *emission, const char *source)
+{
+   API_ENTRY return;
+   if (!sd->signal_func) return;
+   sd->signal_func(obj, emission, source);
+}
+
+EAPI void
+elm_widget_signal_listen(Evas_Object *obj, const char *emission, const char *source, void (*func) (void *data, Evas_Object *o, const char *emission, const char *source), void *data)
+{
+   API_ENTRY return;
+   if (!sd->listen_func) return;
+   sd->listen_func(obj, emission, source, func, data);
+}
+
+EAPI void *
+elm_widget_signal_unlisten(Evas_Object *obj, const char *emission, const char *source, void (*func) (void *data, Evas_Object *o, const char *emission, const char *source))
+{
+   API_ENTRY return NULL;
+   if (!sd->unlisten_func) return NULL;
+   return sd->unlisten_func(obj, emission, source, func);
+}
+
 EAPI void
 elm_widget_focus_set(Evas_Object *obj, int first)
 {
    API_ENTRY return;
    if (!sd->focused)
      {
+        focus_order++;
+        sd->focus_order = focus_order;
 	sd->focused = 1;
 	if (sd->on_focus_func) sd->on_focus_func(sd->on_focus_data, obj);
      }
@@ -609,6 +752,8 @@ _elm_widget_parent_focus(Evas_Object *obj)
 
    if (sd->focused) return;
    if (o) _elm_widget_parent_focus(o);
+   focus_order++;
+   sd->focus_order = focus_order;
    sd->focused = 1;
    if (sd->on_focus_func) sd->on_focus_func(sd->on_focus_data, obj);
    if (sd->focus_func) sd->focus_func(obj);
@@ -802,6 +947,7 @@ EAPI double
 elm_widget_scale_get(const Evas_Object *obj)
 {
    API_ENTRY return 1.0;
+   // FIXME: save walking up the tree by storing/caching parent scale
    if (sd->scale == 0.0)
      {
 	if (sd->parent_obj)
@@ -812,16 +958,40 @@ elm_widget_scale_get(const Evas_Object *obj)
    return sd->scale;
 }
 
+EAPI void
+elm_widget_theme_set(Evas_Object *obj, Elm_Theme *th)
+{
+   API_ENTRY return;
+   if (sd->theme != th)
+     {
+        if (sd->theme) elm_theme_free(sd->theme);
+        sd->theme = th;
+        if (th) th->ref++;
+        elm_widget_theme(obj);
+     }
+}
+
+EAPI Elm_Theme *
+elm_widget_theme_get(const Evas_Object *obj)
+{
+   API_ENTRY return NULL;
+   if (!sd->theme)
+     {
+        if (sd->parent_obj)
+          return elm_widget_theme_get(sd->parent_obj);
+        else
+          return NULL;
+     }
+   return sd->theme;
+}
+
 EAPI void
 elm_widget_style_set(Evas_Object *obj, const char *style)
 {
-   const char *old;
    API_ENTRY return;
 
-   old = sd->style;
-   sd->style = eina_stringshare_add(style);
-   if (old) eina_stringshare_del(old);
-   if (old != sd->style) elm_widget_theme(obj);
+   if (eina_stringshare_replace(&sd->style, style))
+     elm_widget_theme(obj);
 }
 
 EAPI const char *
@@ -835,11 +1005,8 @@ elm_widget_style_get(const Evas_Object *obj)
 EAPI void
 elm_widget_type_set(Evas_Object *obj, const char *type)
 {
-   const char *old;
    API_ENTRY return;
-   old = sd->type;
-   sd->type = eina_stringshare_add(type);
-   if (old) eina_stringshare_del(old);
+   eina_stringshare_replace(&sd->type, type);
 }
 
 EAPI const char *
@@ -850,6 +1017,102 @@ elm_widget_type_get(const Evas_Object *obj)
    return "";
 }
 
+
+
+
+
+
+
+
+
+static void
+_propagate_x_drag_lock(Evas_Object *obj, int dir)
+{
+   Smart_Data *sd = evas_object_smart_data_get(obj);
+   if (sd->parent_obj)
+     {
+        Smart_Data *sd2 = evas_object_smart_data_get(sd->parent_obj);
+        if (sd2)
+          {
+             sd2->child_drag_x_locked += dir;
+             _propagate_x_drag_lock(sd->parent_obj, dir);
+          }
+     }
+}
+
+static void
+_propagate_y_drag_lock(Evas_Object *obj, int dir)
+{
+   Smart_Data *sd = evas_object_smart_data_get(obj);
+   if (sd->parent_obj)
+     {
+        Smart_Data *sd2 = evas_object_smart_data_get(sd->parent_obj);
+        if (sd2)
+          {
+             sd2->child_drag_y_locked += dir;
+             _propagate_y_drag_lock(sd->parent_obj, dir);
+          }
+     }
+}
+
+EAPI void
+elm_widget_drag_lock_x_set(Evas_Object *obj, Eina_Bool lock)
+{
+   API_ENTRY return;
+   if (sd->drag_x_locked == lock) return;
+   sd->drag_x_locked = lock;
+   if (sd->drag_x_locked) _propagate_x_drag_lock(obj, 1);
+   else _propagate_x_drag_lock(obj, -1);
+}
+
+EAPI void
+elm_widget_drag_lock_y_set(Evas_Object *obj, Eina_Bool lock)
+{
+   API_ENTRY return;
+   if (sd->drag_y_locked == lock) return;
+   sd->drag_y_locked = lock;
+   if (sd->drag_y_locked) _propagate_y_drag_lock(obj, 1);
+   else _propagate_y_drag_lock(obj, -1);
+}
+
+EAPI Eina_Bool
+elm_widget_drag_lock_x_get(Evas_Object *obj)
+{
+   API_ENTRY return EINA_FALSE;
+   return sd->drag_x_locked;
+}
+
+EAPI Eina_Bool
+elm_widget_drag_lock_y_get(Evas_Object *obj)
+{
+   API_ENTRY return EINA_FALSE;
+   return sd->drag_y_locked;
+}
+
+EAPI int
+elm_widget_drag_child_locked_x_get(Evas_Object *obj)
+{
+   API_ENTRY return 0;
+   return sd->child_drag_x_locked;
+}
+
+EAPI int
+elm_widget_drag_child_locked_y_get(Evas_Object *obj)
+{
+   API_ENTRY return 0;
+   return sd->child_drag_y_locked;
+}
+
+
+
+
+
+
+
+
+
+
+
 /* local subsystem functions */
 static void
 _smart_reconfigure(Smart_Data *sd)
@@ -882,6 +1145,53 @@ _smart_add(Evas_Object *obj)
    evas_object_smart_data_set(obj, sd);
 }
 
+static Evas_Object *
+_newest_focus_order_get(Evas_Object *obj, unsigned int *newest_focus_order)
+{
+   const Eina_List *l;
+   Evas_Object *child, *ret, *best;
+   
+   API_ENTRY return NULL;
+   if (!evas_object_visible_get(obj)) return NULL;
+   best = NULL;
+   if (*newest_focus_order < sd->focus_order)
+     {
+        *newest_focus_order = sd->focus_order;
+        best = obj;
+     }
+   EINA_LIST_FOREACH(sd->subobjs, l, child)
+     {
+        ret = _newest_focus_order_get(child, newest_focus_order);
+        if (!ret) continue;
+        best = ret;
+     }
+   return best;
+}
+
+static void
+_if_focused_revert(Evas_Object *obj)
+{
+   Evas_Object *top;
+   Evas_Object *newest = NULL;
+   unsigned int newest_focus_order = 0;
+   
+   INTERNAL_ENTRY;
+   
+   if (!sd->focused) return;
+   if (!sd->parent_obj) return;
+
+   top = elm_widget_top_get(sd->parent_obj);
+   if (top)
+     {
+        newest = _newest_focus_order_get(top, &newest_focus_order);
+        if (newest)
+          {
+             elm_object_unfocus(newest);
+             elm_object_focus(newest);
+          }
+     }
+}
+
 static void
 _smart_del(Evas_Object *obj)
 {
@@ -894,7 +1204,7 @@ _smart_del(Evas_Object *obj)
 	sobj = sd->resize_obj;
 	sd->resize_obj = NULL;
 	evas_object_event_callback_del_full(sobj, EVAS_CALLBACK_DEL, _sub_obj_del, sd);
-	evas_object_smart_callback_call(sd->obj, "sub-object-del", sd->resize_obj);
+	evas_object_smart_callback_call(sd->obj, "sub-object-del", sobj);
 	evas_object_del(sobj);
      }
    if (sd->hover_obj)
@@ -914,6 +1224,8 @@ _smart_del(Evas_Object *obj)
    if (sd->del_func) sd->del_func(obj);
    if (sd->style) eina_stringshare_del(sd->style);
    if (sd->type) eina_stringshare_del(sd->type);
+   if (sd->theme) elm_theme_free(sd->theme);
+   _if_focused_revert(obj);
    free(sd);
 }
 
@@ -947,6 +1259,7 @@ _smart_hide(Evas_Object *obj)
 {
    INTERNAL_ENTRY;
    evas_object_hide(sd->resize_obj);
+   _if_focused_revert(obj);
 }
 
 static void
@@ -1000,15 +1313,19 @@ _smart_init(void)
 	       _smart_calculate,
 	       NULL,
 	       NULL,
-	       NULL
+	       NULL,
+               NULL,
+               NULL,
+               NULL
 	  };
 	_e_smart = evas_smart_class_new(&sc);
      }
 }
 
+/* utilities */
 
 Eina_List *
-_stringlist_get(const char *str)
+_elm_stringlist_get(const char *str)
 {
    Eina_List *list = NULL;
    const char *s, *b;
@@ -1033,9 +1350,39 @@ _stringlist_get(const char *str)
 }
 
 void
-_stringlist_free(Eina_List *list)
+_elm_stringlist_free(Eina_List *list)
 {
    const char *s;
-   EINA_LIST_FREE(list, s)
-     eina_stringshare_del(s);
+   EINA_LIST_FREE(list, s) eina_stringshare_del(s);
+}
+
+Eina_Bool
+_elm_widget_type_check(const Evas_Object *obj, const char *type)
+{
+   const char *provided, *expected = "(unknown)";
+   static int abort_on_warn = -1;
+   provided = elm_widget_type_get(obj);
+   if (EINA_LIKELY(provided == type)) return EINA_TRUE;
+   if (type) expected = type;
+   if ((!provided) || (provided[0] == 0))
+     {
+        provided = evas_object_type_get(obj);
+        if ((!provided) || (provided[0] == 0))
+          provided = "(unknown)";
+     }
+   ERR("Passing Object: %p, of type: '%s' when expecting type: '%s'", obj, provided, expected);
+   if (abort_on_warn == -1)
+     {
+        if (getenv("ELM_ERROR_ABORT")) abort_on_warn = 1;
+        else abort_on_warn = 0;
+     }
+   if (abort_on_warn == 1) abort();
+   return EINA_FALSE;
+}
+
+static inline Eina_Bool
+_elm_widget_is(const Evas_Object *obj)
+{
+   const char *type = evas_object_type_get(obj);
+   return type == SMART_NAME;
 }
