@@ -19,6 +19,8 @@ struct _Smart_Data
    Evas_Object *edje_obj;
    Evas_Object *event_obj;
 
+   Evas_Object *widget;
+   
    Elm_Smart_Scroller_Policy hbar_flags, vbar_flags;
 
    struct {
@@ -45,11 +47,16 @@ struct _Smart_Data
       Ecore_Animator *bounce_x_animator;
       Ecore_Animator *bounce_y_animator;
       Evas_Coord locked_x, locked_y;
+      int hdir, vdir;
       unsigned char now : 1;
+      unsigned char cancelled : 1;
+      unsigned char hold : 1;
+      unsigned char hold_parent : 1;
+      unsigned char want_dragged : 1;
       unsigned char dragged : 1;
+      unsigned char dragged_began : 1;
       unsigned char dir_x : 1;
       unsigned char dir_y : 1;
-      unsigned char dir_none : 1;
       unsigned char locked : 1;
       unsigned char bounce_x_hold : 1;
       unsigned char bounce_y_hold : 1;
@@ -98,10 +105,10 @@ static void _smart_pan_changed_hook(void *data, Evas_Object *obj, void *event_in
 static void _smart_pan_pan_changed_hook(void *data, Evas_Object *obj, void *event_info);
 static void _smart_event_wheel(void *data, Evas *e, Evas_Object *obj, void *event_info);
 static void _smart_event_mouse_down(void *data, Evas *e, Evas_Object *obj, void *event_info);
-static int  _smart_hold_animator(void *data);
-static int  _smart_momentum_animator(void *data);
+static Eina_Bool  _smart_hold_animator(void *data);
+static Eina_Bool  _smart_momentum_animator(void *data);
 static void _smart_event_mouse_up(void *data, Evas *e, Evas_Object *obj, void *event_info);
-static int  _smart_onhold_animator(void *data);
+static Eina_Bool  _smart_onhold_animator(void *data);
 static void _smart_event_mouse_move(void *data, Evas *e, Evas_Object *obj, void *event_info);
 static void _smart_event_key_down(void *data, Evas *e, Evas_Object *obj, void *event_info);
 static void _smart_edje_drag_v_start(void *data, Evas_Object *obj, const char *emission, const char *source);
@@ -272,7 +279,7 @@ _smart_drag_stop(Evas_Object *obj)
    evas_object_smart_callback_call(obj, "drag,stop", NULL);
 }
 
-static int
+static Eina_Bool
 _smart_scrollto_x_animator(void *data)
 {
    Smart_Data *sd = data;
@@ -293,10 +300,10 @@ _smart_scrollto_x_animator(void *data)
         sd->scrollto.x.animator = NULL;
         if (!sd->scrollto.y.animator)
           _smart_anim_stop(sd->smart_obj);
-        return 0;
+        return ECORE_CALLBACK_CANCEL;
      }
    elm_smart_scroller_child_pos_set(sd->smart_obj, px, py);
-   return 1;
+   return ECORE_CALLBACK_RENEW;
 }
 
 static void
@@ -337,7 +344,7 @@ _smart_scrollto_x(Smart_Data *sd, double t_in, Evas_Coord pos_x)
    sd->bouncemex = 0;
 }
 
-static int
+static Eina_Bool
 _smart_scrollto_y_animator(void *data)
 {
    Smart_Data *sd = data;
@@ -358,11 +365,11 @@ _smart_scrollto_y_animator(void *data)
         sd->scrollto.y.animator = NULL;
         if (!sd->scrollto.x.animator)
           _smart_anim_stop(sd->smart_obj);
-        return 0;
+        return ECORE_CALLBACK_CANCEL;
      }
    elm_smart_scroller_child_pos_set(sd->smart_obj, px, py);
 
-   return 1;
+   return ECORE_CALLBACK_RENEW;
 }
 
 static void
@@ -408,8 +415,8 @@ _smart_do_page(Smart_Data *sd)
 {
    if ((sd->pagerel_h == 0.0) && (sd->pagesize_h == 0) &&
        (sd->pagerel_v == 0.0) && (sd->pagesize_v == 0))
-     return 0;
-   return 1;
+     return EINA_FALSE;
+   return EINA_TRUE;
 }
 
 static Evas_Coord
@@ -483,11 +490,11 @@ _smart_page_adjust(Smart_Data *sd)
    elm_smart_scroller_child_region_show(sd->smart_obj, x, y, w, h);
 }
 
-static int
+static Eina_Bool
 _smart_bounce_x_animator(void *data)
 {
    Smart_Data *sd;
-   Evas_Coord x, y, dx, dy/*, ox, oy*/;
+   Evas_Coord x, y, dx;
    double t, p, dt;
 
    sd = data;
@@ -502,24 +509,25 @@ _smart_bounce_x_animator(void *data)
         dx = sd->down.b2x - sd->down.bx;
 	dx = (dx * p);
 	x = sd->down.bx + dx;
-        elm_smart_scroller_child_pos_set(sd->smart_obj, x, y);
+        if (!sd->down.cancelled)
+          elm_smart_scroller_child_pos_set(sd->smart_obj, x, y);
         if (dt >= 1.0)
           {
              if (sd->down.momentum_animator)
                sd->down.bounce_x_hold = 1;
              sd->down.bounce_x_animator = NULL;
              sd->bouncemex = 0;
-             return 0;
+             return ECORE_CALLBACK_CANCEL;
           }
      }
-   return 1;
+   return ECORE_CALLBACK_RENEW;
 }
 
-static int
+static Eina_Bool
 _smart_bounce_y_animator(void *data)
 {
    Smart_Data *sd;
-   Evas_Coord x, y, dx, dy;
+   Evas_Coord x, y, dy;
    double t, p, dt;
 
    sd = data;
@@ -534,20 +542,52 @@ _smart_bounce_y_animator(void *data)
         dy = sd->down.b2y - sd->down.by;
 	dy = (dy * p);
 	y = sd->down.by + dy;
-        elm_smart_scroller_child_pos_set(sd->smart_obj, x, y);
+        if (!sd->down.cancelled)
+          elm_smart_scroller_child_pos_set(sd->smart_obj, x, y);
         if (dt >= 1.0)
           {
              if (sd->down.momentum_animator)
                sd->down.bounce_y_hold = 1;
              sd->down.bounce_y_animator = NULL;
              sd->bouncemey = 0;
-             return 0;
+             return ECORE_CALLBACK_CANCEL;
           }
      }
-   return 1;
+   return ECORE_CALLBACK_RENEW;
 }
 
-static int
+#define LEFT 0
+#define RIGHT 1
+#define UP 2
+#define DOWN 3
+static Eina_Bool
+can_scroll(Smart_Data *sd, int dir)
+{
+   Evas_Coord mx = 0, my = 0, px = 0, py = 0;
+
+   sd->pan_func.max_get(sd->pan_obj, &mx, &my);
+   sd->pan_func.get(sd->pan_obj, &px, &py);
+   switch (dir)
+     {
+     case LEFT:
+        if (px > 0) return EINA_TRUE;
+        break;
+     case RIGHT:
+        if (px < mx) return EINA_TRUE;
+        break;
+     case UP:
+        if (py > 0) return EINA_TRUE;
+        break;
+     case DOWN:
+        if (py < my) return EINA_TRUE;
+        break;
+     default:
+        break;
+     }
+   return EINA_FALSE;
+}
+
+static Eina_Bool
 _smart_momentum_animator(void *data)
 {
    Smart_Data *sd;
@@ -559,6 +599,23 @@ _smart_momentum_animator(void *data)
    dt = t - sd->down.anim_start;
    if (dt >= 0.0)
      {
+        /*
+        if (sd->down.hold_parent)
+          {
+             if ((sd->down.dir_x) && !can_scroll(sd, sd->down.hdir))
+               {
+                  sd->down.dir_x = 0;
+               }
+             if ((sd->down.dir_y) && !can_scroll(sd, sd->down.vdir))
+               {
+                  sd->down.dir_y = 0;
+               }
+          }
+        if ((!sd->down.dir_x) && (!sd->down.dir_y))
+          {
+             sd->down.cancelled = 1;
+          }
+         */
 	dt = dt / _elm_config->thumbscroll_friction;
 	if (dt > 1.0) dt = 1.0;
 	p = 1.0 - ((1.0 - dt) * (1.0 - dt));
@@ -590,10 +647,10 @@ _smart_momentum_animator(void *data)
              sd->down.bounce_y_hold = 0;
              sd->down.ax = 0;
              sd->down.ay = 0;
-	     return 0;
+	     return ECORE_CALLBACK_CANCEL;
 	  }
      }
-   return 1;
+   return ECORE_CALLBACK_RENEW;
 }
 
 static void
@@ -624,40 +681,48 @@ bounce_eval(Smart_Data *sd)
    if (py > my) py = my;
    b2x = px;
    b2y = py;
-   if (!sd->down.bounce_x_animator)
+   if ((!sd->widget) || 
+       (!elm_widget_drag_child_locked_x_get(sd->widget)))
      {
-        if (sd->bouncemex)
+        if (!sd->down.bounce_x_animator)
           {
-             if (sd->scrollto.x.animator)
+             if (sd->bouncemex)
                {
-                  ecore_animator_del(sd->scrollto.x.animator);
-                  sd->scrollto.x.animator = NULL;
+                  if (sd->scrollto.x.animator)
+                    {
+                       ecore_animator_del(sd->scrollto.x.animator);
+                       sd->scrollto.x.animator = NULL;
+                    }
+                  sd->down.bounce_x_animator = ecore_animator_add(_smart_bounce_x_animator, sd);
+                  sd->down.anim_start2 = ecore_loop_time_get();
+                  sd->down.bx = bx;
+                  sd->down.bx0 = bx;
+                  sd->down.b2x = b2x;
+                  if (sd->down.momentum_animator) sd->down.b0x = sd->down.ax;
+                  else sd->down.b0x = 0;
                }
-             sd->down.bounce_x_animator = ecore_animator_add(_smart_bounce_x_animator, sd);
-             sd->down.anim_start2 = ecore_loop_time_get();
-             sd->down.bx = bx;
-             sd->down.bx0 = bx;
-             sd->down.b2x = b2x;
-             if (sd->down.momentum_animator) sd->down.b0x = sd->down.ax;
-             else sd->down.b0x = 0;
           }
      }
-   if (!sd->down.bounce_y_animator)
+   if ((!sd->widget) || 
+       (!elm_widget_drag_child_locked_y_get(sd->widget)))
      {
-        if (sd->bouncemey)
+        if (!sd->down.bounce_y_animator)
           {
-             if (sd->scrollto.y.animator)
+             if (sd->bouncemey)
                {
-                  ecore_animator_del(sd->scrollto.y.animator);
-                  sd->scrollto.y.animator = NULL;
+                  if (sd->scrollto.y.animator)
+                    {
+                       ecore_animator_del(sd->scrollto.y.animator);
+                       sd->scrollto.y.animator = NULL;
+                    }
+                  sd->down.bounce_y_animator = ecore_animator_add(_smart_bounce_y_animator, sd);
+                  sd->down.anim_start3 = ecore_loop_time_get();
+                  sd->down.by = by;
+                  sd->down.by0 = by;
+                  sd->down.b2y = b2y;
+                  if (sd->down.momentum_animator) sd->down.b0y = sd->down.ay;
+                  else sd->down.b0y = 0;
                }
-             sd->down.bounce_y_animator = ecore_animator_add(_smart_bounce_y_animator, sd);
-             sd->down.anim_start3 = ecore_loop_time_get();
-             sd->down.by = by;
-             sd->down.by0 = by;
-             sd->down.b2y = b2y;
-             if (sd->down.momentum_animator) sd->down.b0y = sd->down.ay;
-             else sd->down.b0y = 0;
           }
      }
 }
@@ -913,10 +978,10 @@ elm_smart_scroller_single_dir_get(Evas_Object *obj)
 }
 
 void
-elm_smart_scroller_theme_set(Evas_Object *obj, const char *clas, const char *group, const char *style)
+elm_smart_scroller_object_theme_set(Evas_Object *parent, Evas_Object *obj, const char *clas, const char *group, const char *style)
 {
    API_ENTRY return;
-   _elm_theme_set(sd->edje_obj, clas, group, style);
+   _elm_theme_object_set(parent, sd->edje_obj, clas, group, style);
    if (sd->pan_obj)
      edje_object_part_swallow(sd->edje_obj, "elm.swallow.content", sd->pan_obj);
    sd->vbar_visible = !sd->vbar_visible;
@@ -954,6 +1019,14 @@ elm_smart_scroller_bounce_allow_set(Evas_Object *obj, Eina_Bool horiz, Eina_Bool
    sd->bounce_vert = vert;
 }
 
+void
+elm_smart_scroller_bounce_allow_get(const Evas_Object *obj, Eina_Bool *horiz, Eina_Bool *vert)
+{
+   API_ENTRY return;
+   *horiz = sd->bounce_horiz;
+   *vert = sd->bounce_vert;
+}
+
 void
 elm_smart_scroller_paging_set(Evas_Object *obj, double pagerel_h, double pagerel_v, Evas_Coord pagesize_h, Evas_Coord pagesize_v)
 {
@@ -1042,9 +1115,16 @@ elm_smart_scroller_region_bring_in(Evas_Object *obj, Evas_Coord x, Evas_Coord y,
    _smart_scrollto_y(sd, _elm_config->bring_in_scroll_friction, y);
 }
 
+void
+elm_smart_scroller_widget_set(Evas_Object *obj, Evas_Object *wid)
+{
+   API_ENTRY return;
+   sd->widget = wid;
+}
+
 /* local subsystem functions */
 static void
-_smart_edje_drag_v_start(void *data, Evas_Object *obj, const char *emission, const char *source)
+_smart_edje_drag_v_start(void *data, Evas_Object *obj __UNUSED__, const char *emission __UNUSED__, const char *source __UNUSED__)
 {
    Smart_Data *sd;
 
@@ -1054,7 +1134,7 @@ _smart_edje_drag_v_start(void *data, Evas_Object *obj, const char *emission, con
 }
 
 static void
-_smart_edje_drag_v_stop(void *data, Evas_Object *obj, const char *emission, const char *source)
+_smart_edje_drag_v_stop(void *data, Evas_Object *obj __UNUSED__, const char *emission __UNUSED__, const char *source __UNUSED__)
 {
    Smart_Data *sd;
 
@@ -1064,7 +1144,7 @@ _smart_edje_drag_v_stop(void *data, Evas_Object *obj, const char *emission, cons
 }
 
 static void
-_smart_edje_drag_v(void *data, Evas_Object *obj, const char *emission, const char *source)
+_smart_edje_drag_v(void *data, Evas_Object *obj __UNUSED__, const char *emission __UNUSED__, const char *source __UNUSED__)
 {
    Smart_Data *sd;
 
@@ -1073,7 +1153,7 @@ _smart_edje_drag_v(void *data, Evas_Object *obj, const char *emission, const cha
 }
 
 static void
-_smart_edje_drag_h_start(void *data, Evas_Object *obj, const char *emission, const char *source)
+_smart_edje_drag_h_start(void *data, Evas_Object *obj __UNUSED__, const char *emission __UNUSED__, const char *source __UNUSED__)
 {
    Smart_Data *sd;
 
@@ -1083,7 +1163,7 @@ _smart_edje_drag_h_start(void *data, Evas_Object *obj, const char *emission, con
 }
 
 static void
-_smart_edje_drag_h_stop(void *data, Evas_Object *obj, const char *emission, const char *source)
+_smart_edje_drag_h_stop(void *data, Evas_Object *obj __UNUSED__, const char *emission __UNUSED__, const char *source __UNUSED__)
 {
    Smart_Data *sd;
 
@@ -1093,7 +1173,7 @@ _smart_edje_drag_h_stop(void *data, Evas_Object *obj, const char *emission, cons
 }
 
 static void
-_smart_edje_drag_h(void *data, Evas_Object *obj, const char *emission, const char *source)
+_smart_edje_drag_h(void *data, Evas_Object *obj __UNUSED__, const char *emission __UNUSED__, const char *source __UNUSED__)
 {
    Smart_Data *sd;
 
@@ -1102,7 +1182,7 @@ _smart_edje_drag_h(void *data, Evas_Object *obj, const char *emission, const cha
 }
 
 static void
-_smart_child_del_hook(void *data, Evas *e, Evas_Object *obj, void *event_info)
+_smart_child_del_hook(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Smart_Data *sd;
 
@@ -1113,7 +1193,7 @@ _smart_child_del_hook(void *data, Evas *e, Evas_Object *obj, void *event_info)
 }
 
 static void
-_smart_pan_changed_hook(void *data, Evas_Object *obj, void *event_info)
+_smart_pan_changed_hook(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Evas_Coord x, y;
    Evas_Coord w, h;
@@ -1133,7 +1213,7 @@ _smart_pan_changed_hook(void *data, Evas_Object *obj, void *event_info)
 }
 
 static void
-_smart_pan_pan_changed_hook(void *data, Evas_Object *obj, void *event_info)
+_smart_pan_pan_changed_hook(void *data, Evas_Object *obj __UNUSED__, void *event_info __UNUSED__)
 {
    Evas_Coord x, y;
    Smart_Data *sd;
@@ -1171,7 +1251,7 @@ _smart_pan_pan_changed_hook(void *data, Evas_Object *obj, void *event_info)
 }
 
 static void
-_smart_event_wheel(void *data, Evas *e, Evas_Object *obj, void *event_info)
+_smart_event_wheel(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
 {
    Evas_Event_Mouse_Wheel *ev;
    Smart_Data *sd;
@@ -1223,7 +1303,7 @@ _smart_event_wheel(void *data, Evas *e, Evas_Object *obj, void *event_info)
 }
 
 static void
-_smart_event_mouse_down(void *data, Evas *e, Evas_Object *obj, void *event_info)
+_smart_event_mouse_down(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
 {
    Evas_Event_Mouse_Down *ev;
    Smart_Data *sd;
@@ -1231,7 +1311,7 @@ _smart_event_mouse_down(void *data, Evas *e, Evas_Object *obj, void *event_info)
 
    sd = data;
    ev = event_info;
-   if (ev->event_flags & EVAS_EVENT_FLAG_ON_HOLD) return ;
+//   if (ev->event_flags & EVAS_EVENT_FLAG_ON_HOLD) return ;
    if (_elm_config->thumbscroll_enable)
      {
         if ((sd->down.bounce_x_animator) || (sd->down.bounce_y_animator) ||
@@ -1283,7 +1363,6 @@ _smart_event_mouse_down(void *data, Evas *e, Evas_Object *obj, void *event_info)
 	     sd->down.dragged = 0;
 	     sd->down.dir_x = 0;
 	     sd->down.dir_y = 0;
-	     sd->down.dir_none = 0;
 	     sd->down.x = ev->canvas.x;
 	     sd->down.y = ev->canvas.y;
 	     elm_smart_scroller_child_pos_get(sd->smart_obj, &x, &y);
@@ -1299,22 +1378,57 @@ _smart_event_mouse_down(void *data, Evas *e, Evas_Object *obj, void *event_info)
 	     sd->down.history[0].x = ev->canvas.x;
 	     sd->down.history[0].y = ev->canvas.y;
 	  }
+        sd->down.dragged_began = 0;
+        sd->down.hold = 0;
+        sd->down.hold_parent = 0;
+        sd->down.cancelled = 0;
      }
 }
 
-static int
+static Eina_Bool
 _smart_hold_animator(void *data)
 {
-   Smart_Data *sd;
+   Smart_Data *sd = data;
+   Evas_Coord ox, oy;
+   
+   elm_smart_scroller_child_pos_get(sd->smart_obj, &ox, &oy);
+   if (sd->down.dir_x)
+     {
+        if ((!sd->widget) || 
+            (!elm_widget_drag_child_locked_x_get(sd->widget)))
+          {
+             ox = sd->down.hold_x;
+          }
+     }
+   if (sd->down.dir_y)
+     {
+        if ((!sd->widget) || 
+            (!elm_widget_drag_child_locked_y_get(sd->widget)))
+          {
+             oy = sd->down.hold_y;
+          }
+     }
+   elm_smart_scroller_child_pos_set(sd->smart_obj, ox, oy);
+   return ECORE_CALLBACK_RENEW;
+}
 
-   sd = data;
-   sd->down.hold_animator = NULL;
-   elm_smart_scroller_child_pos_set(sd->smart_obj, sd->down.hold_x, sd->down.hold_y);
-   return 0;
+static Eina_Bool
+_smart_event_post_up(void *data, Evas *e)
+{
+   Smart_Data *sd = data;
+   if (sd->widget)
+     {
+        if (sd->down.dragged)
+          {
+             elm_widget_drag_lock_x_set(sd->widget, 0);
+             elm_widget_drag_lock_y_set(sd->widget, 0);
+          }
+     }
+   return EINA_TRUE;
 }
 
 static void
-_smart_event_mouse_up(void *data, Evas *e, Evas_Object *obj, void *event_info)
+_smart_event_mouse_up(void *data, Evas *e, Evas_Object *obj __UNUSED__, void *event_info)
 {
    Evas_Event_Mouse_Down *ev;
    Smart_Data *sd;
@@ -1322,7 +1436,9 @@ _smart_event_mouse_up(void *data, Evas *e, Evas_Object *obj, void *event_info)
 
    sd = data;
    ev = event_info;
-   if (ev->event_flags & EVAS_EVENT_FLAG_ON_HOLD) return ;
+   sd->down.hold_parent = 0;
+//   if (ev->event_flags & EVAS_EVENT_FLAG_ON_HOLD) return ;
+   evas_post_event_callback_push(e, _smart_event_post_up, sd);
    // FIXME: respect elm_widget_scroll_hold_get of parent container
    if (_elm_config->thumbscroll_enable)
      {
@@ -1354,7 +1470,7 @@ _smart_event_mouse_up(void *data, Evas *e, Evas_Object *obj, void *event_info)
                        ay = ev->canvas.y;
                        at = 0.0;
 #ifdef SCROLLDBG
-                       printf("------\n");
+                       printf("------ %i %i\n", ev->canvas.x, ev->canvas.y);
 #endif
                        for (i = 0; i < 20; i++)
                          {
@@ -1414,10 +1530,18 @@ _smart_event_mouse_up(void *data, Evas *e, Evas_Object *obj, void *event_info)
                        Evas_Coord pgx, pgy;
 
                        elm_smart_scroller_child_pos_get(sd->smart_obj, &x, &y);
-                       pgx = _smart_page_x_get(sd, ox);
-                       if (pgx != x) _smart_scrollto_x(sd, _elm_config->page_scroll_friction, pgx);
-                       pgy = _smart_page_y_get(sd, oy);
-                       if (pgy != y) _smart_scrollto_y(sd, _elm_config->page_scroll_friction, pgy);
+                       if ((!sd->widget) || 
+                           (!elm_widget_drag_child_locked_x_get(sd->widget)))
+                         {
+                            pgx = _smart_page_x_get(sd, ox);
+                            if (pgx != x) _smart_scrollto_x(sd, _elm_config->page_scroll_friction, pgx);
+                         }
+                       if ((!sd->widget) || 
+                           (!elm_widget_drag_child_locked_y_get(sd->widget)))
+                         {
+                            pgy = _smart_page_y_get(sd, oy);
+                            if (pgy != y) _smart_scrollto_y(sd, _elm_config->page_scroll_friction, pgy);
+                         }
                     }
 	       }
              else
@@ -1427,12 +1551,34 @@ _smart_event_mouse_up(void *data, Evas *e, Evas_Object *obj, void *event_info)
                        Evas_Coord pgx, pgy;
 
                        elm_smart_scroller_child_pos_get(sd->smart_obj, &x, &y);
-                       pgx = _smart_page_x_get(sd, ox);
-                       if (pgx != x) _smart_scrollto_x(sd, _elm_config->page_scroll_friction, pgx);
-                       pgy = _smart_page_y_get(sd, oy);
-                       if (pgy != y) _smart_scrollto_y(sd, _elm_config->page_scroll_friction, pgy);
+                       if ((!sd->widget) || 
+                           (!elm_widget_drag_child_locked_x_get(sd->widget)))
+                         {
+                            pgx = _smart_page_x_get(sd, ox);
+                            if (pgx != x) _smart_scrollto_x(sd, _elm_config->page_scroll_friction, pgx);
+                         }
+                       if ((!sd->widget) || 
+                           (!elm_widget_drag_child_locked_y_get(sd->widget)))
+                         {
+                            pgy = _smart_page_y_get(sd, oy);
+                            if (pgy != y) _smart_scrollto_y(sd, _elm_config->page_scroll_friction, pgy);
+                         }
+                    }
+                  if (sd->down.hold_animator)
+                    {
+                       ecore_animator_del(sd->down.hold_animator);
+                       sd->down.hold_animator = NULL;
                     }
                }
+             if (sd->down.hold)
+               {
+                  ev->event_flags |= EVAS_EVENT_FLAG_ON_HOLD;
+                  sd->down.hold = 0;
+               }
+             sd->down.dragged_began = 0;
+             sd->down.dir_x = 0;
+             sd->down.dir_y = 0;
+             sd->down.want_dragged = 0;
 	     sd->down.dragged = 0;
 	     sd->down.now = 0;
              elm_smart_scroller_child_pos_get(sd->smart_obj, &x, &y);
@@ -1443,13 +1589,13 @@ _smart_event_mouse_up(void *data, Evas *e, Evas_Object *obj, void *event_info)
      }
 }
 
-static int
+static Eina_Bool
 _smart_onhold_animator(void *data)
 {
    Smart_Data *sd;
    double t, td;
    double vx, vy;
-   Evas_Coord x, y, ox, oy, dx, dy;
+   Evas_Coord x, y, ox, oy;
 
    sd = data;
    t = ecore_loop_time_get();
@@ -1459,20 +1605,100 @@ _smart_onhold_animator(void *data)
         vx = sd->down.onhold_vx * td * (double)_elm_config->thumbscroll_threshhold * 2.0;
         vy = sd->down.onhold_vy * td * (double)_elm_config->thumbscroll_threshhold * 2.0;
         elm_smart_scroller_child_pos_get(sd->smart_obj, &ox, &oy);
-        sd->down.onhold_vxe += vx;
-        sd->down.onhold_vye += vy;
-        x = ox + (int)sd->down.onhold_vxe;
-        y = oy + (int)sd->down.onhold_vye;
-        sd->down.onhold_vxe -= (int)sd->down.onhold_vxe;
-        sd->down.onhold_vye -= (int)sd->down.onhold_vye;
+        x = ox;
+        y = oy;
+        
+        if (sd->down.dir_x)
+          {
+             if ((!sd->widget) || 
+                 (!elm_widget_drag_child_locked_x_get(sd->widget)))
+               {
+                  sd->down.onhold_vxe += vx;
+                  x = ox + (int)sd->down.onhold_vxe;
+                  sd->down.onhold_vxe -= (int)sd->down.onhold_vxe;
+               }
+          }
+        
+        if (sd->down.dir_y)
+          {
+             if ((!sd->widget) || 
+                 (!elm_widget_drag_child_locked_y_get(sd->widget)))
+               {
+                  sd->down.onhold_vye += vy;
+                  y = oy + (int)sd->down.onhold_vye;
+                  sd->down.onhold_vye -= (int)sd->down.onhold_vye;
+               }
+          }
+        
         elm_smart_scroller_child_pos_set(sd->smart_obj, x, y);
+//        printf("scroll %i %i\n", sd->down.hold_x, sd->down.hold_y);
      }
    sd->down.onhold_tlast = t;
-   return 1;
+   return ECORE_CALLBACK_RENEW;
+}
+
+static Eina_Bool
+_smart_event_post_move(void *data, Evas *e)
+{
+   Smart_Data *sd = data;
+   if (sd->down.want_dragged)
+     {
+        int start = 0;
+        
+        if (sd->down.hold_parent)
+          {
+             if ((sd->down.dir_x) && !can_scroll(sd, sd->down.hdir))
+               {
+                  sd->down.dir_x = 0;
+               }
+             if ((sd->down.dir_y) && !can_scroll(sd, sd->down.vdir))
+               {
+                  sd->down.dir_y = 0;
+               }
+          }
+        if (sd->down.dir_x)
+          {
+             if ((!sd->widget) || 
+                 (!elm_widget_drag_child_locked_x_get(sd->widget)))
+               {
+                  sd->down.want_dragged = 0;
+                  sd->down.dragged = 1;
+                  if (sd->widget)
+                    {
+                       elm_widget_drag_lock_x_set(sd->widget, 1);
+                    }
+                  start = 1;
+               }
+             else
+               sd->down.dir_x = 0;
+          }
+        if (sd->down.dir_y)
+          {
+             if ((!sd->widget) || 
+                 (!elm_widget_drag_child_locked_y_get(sd->widget)))
+               {
+                  sd->down.want_dragged = 0;
+                  sd->down.dragged = 1;
+                  if (sd->widget)
+                    {
+                       elm_widget_drag_lock_y_set(sd->widget, 1);
+                    }
+                  start = 1;
+               }
+             else
+               sd->down.dir_y = 0;
+          }
+        if ((!sd->down.dir_x) && (!sd->down.dir_y))
+          {
+             sd->down.cancelled = 1;
+          }
+        if (start) _smart_drag_start(sd->smart_obj);
+     }
+   return EINA_TRUE;
 }
 
 static void
-_smart_event_mouse_move(void *data, Evas *e, Evas_Object *obj, void *event_info)
+_smart_event_mouse_move(void *data, Evas *e, Evas_Object *obj __UNUSED__, void *event_info)
 {
    Evas_Event_Mouse_Move *ev;
    Smart_Data *sd;
@@ -1480,14 +1706,19 @@ _smart_event_mouse_move(void *data, Evas *e, Evas_Object *obj, void *event_info)
 
    sd = data;
    ev = event_info;
-   if (ev->event_flags & EVAS_EVENT_FLAG_ON_HOLD) return ;
+//   if (ev->event_flags & EVAS_EVENT_FLAG_ON_HOLD) return ;
+   if (ev->event_flags & EVAS_EVENT_FLAG_ON_HOLD) sd->down.hold_parent = 1;
+   evas_post_event_callback_push(e, _smart_event_post_move, sd);
    // FIXME: respect elm_widget_scroll_hold_get of parent container
    if (_elm_config->thumbscroll_enable)
      {
 	if (sd->down.now)
 	  {
-             int faildir = 0;
+             int dodir = 0;
 
+#ifdef SCROLLDBG
+             printf("::: %i %i\n", ev->cur.canvas.x, ev->cur.canvas.y);
+#endif
 	     memmove(&(sd->down.history[1]), &(sd->down.history[0]),
 		     sizeof(sd->down.history[0]) * 19);
 #ifdef EVTIME
@@ -1498,33 +1729,59 @@ _smart_event_mouse_move(void *data, Evas *e, Evas_Object *obj, void *event_info)
 	     sd->down.history[0].x = ev->cur.canvas.x;
 	     sd->down.history[0].y = ev->cur.canvas.y;
 
-	     x = ev->cur.canvas.x - sd->down.x;
-	     if (x < 0) x = -x;
-	     y = ev->cur.canvas.y - sd->down.y;
-	     if (y < 0) y = -y;
-	     if ((sd->one_dir_at_a_time) &&
-		 (!sd->down.dir_x) && (!sd->down.dir_y) && (!sd->down.dir_none))
-	       {
-                  if (x > _elm_config->thumbscroll_threshhold)
+             if (!sd->down.dragged_began)
+               {
+                  x = ev->cur.canvas.x - sd->down.x;
+                  y = ev->cur.canvas.y - sd->down.y;
+                  
+                  sd->down.hdir = -1;
+                  sd->down.vdir = -1;
+                    
+                  if      (x > 0) sd->down.hdir = LEFT;
+                  else if (x < 0) sd->down.hdir = RIGHT;
+                  if      (y > 0) sd->down.vdir = UP;
+                  else if (y < 0) sd->down.vdir = DOWN;
+                  
+                  if (x < 0) x = -x;
+                  if (y < 0) y = -y;
+             
+                  if ((sd->one_dir_at_a_time) &&
+                      (!((sd->down.dir_x) || (sd->down.dir_y))))
                     {
-                       if (x > (y * 2))
+                       if (x > _elm_config->thumbscroll_threshhold)
                          {
-			    sd->down.dir_x = 1;
-			    sd->down.dir_y = 0;
-			 }
-                       else faildir++;
-		    }
-                  if (y > _elm_config->thumbscroll_threshhold)
-		    {
-                       if (y > (x * 2))
-			 {
-			    sd->down.dir_x = 0;
-			    sd->down.dir_y = 1;
+                            if (x > (y * 2))
+                              {
+                                 sd->down.dir_x = 1;
+                                 sd->down.dir_y = 0;
+                                 dodir++;
+                              }
                          }
-                       else faildir++;
-		    }
-                  if (faildir) sd->down.dir_none = 1;
-	       }
+                       if (y > _elm_config->thumbscroll_threshhold)
+                         {
+                            if (y > (x * 2))
+                              {
+                                 sd->down.dir_x = 0;
+                                 sd->down.dir_y = 1;
+                                 dodir++;
+                              }
+                         }
+                       if (!dodir)
+                         {
+                            sd->down.dir_x = 1;
+                            sd->down.dir_y = 1;
+                         }
+                    }
+                  else
+                    {
+//                       can_scroll(sd, LEFT);
+//                       can_scroll(sd, RIGHT);
+//                       can_scroll(sd, UP);
+//                       can_scroll(sd, DOWN);
+                       sd->down.dir_x = 1;
+                       sd->down.dir_y = 1;
+                    }
+               }
              if ((!sd->hold) && (!sd->freeze))
                {
                   if ((sd->down.dragged) ||
@@ -1532,15 +1789,28 @@ _smart_event_mouse_move(void *data, Evas *e, Evas_Object *obj, void *event_info)
                        (_elm_config->thumbscroll_threshhold *
                         _elm_config->thumbscroll_threshhold)))
                     {
+                       sd->down.dragged_began = 1;
                        if (!sd->down.dragged)
                          {
-                            evas_event_feed_hold(e, 1, ev->timestamp, ev->data);
-                            _smart_drag_start(sd->smart_obj);
+                            sd->down.want_dragged = 1;
+                            ev->event_flags |= EVAS_EVENT_FLAG_ON_HOLD;
+//                            evas_event_feed_hold(e, 1, ev->timestamp, ev->data);
+//                            _smart_drag_start(sd->smart_obj);
                          }
-                       ev->event_flags |= EVAS_EVENT_FLAG_ON_HOLD;
-                       sd->down.dragged = 1;
-                       x = sd->down.sx - (ev->cur.canvas.x - sd->down.x);
-                       y = sd->down.sy - (ev->cur.canvas.y - sd->down.y);
+                       if (sd->down.dragged)
+                         {
+                            ev->event_flags |= EVAS_EVENT_FLAG_ON_HOLD;
+                         }
+//                       ev->event_flags |= EVAS_EVENT_FLAG_ON_HOLD;
+//                       sd->down.dragged = 1;
+                       if (sd->down.dir_x)
+                         x = sd->down.sx - (ev->cur.canvas.x - sd->down.x);
+                       else
+                         x = sd->down.sx;
+                       if (sd->down.dir_y)
+                         y = sd->down.sy - (ev->cur.canvas.y - sd->down.y);
+                       else
+                         y = sd->down.sy;
                        if ((sd->down.dir_x) || (sd->down.dir_y))
                          {
                             if (!sd->down.locked)
@@ -1549,15 +1819,33 @@ _smart_event_mouse_move(void *data, Evas *e, Evas_Object *obj, void *event_info)
                                  sd->down.locked_y = y;
                                  sd->down.locked = 1;
                               }
-                            if (sd->down.dir_x) y = sd->down.locked_y;
-                            else x = sd->down.locked_x;
+                            if (!((sd->down.dir_x) && (sd->down.dir_y)))
+                              {
+                                 if (sd->down.dir_x) y = sd->down.locked_y;
+                                 else x = sd->down.locked_x;
+                              }
                          }
                        sd->down.hold_x = x;
                        sd->down.hold_y = y;
                        if (!sd->down.hold_animator)
-                         sd->down.hold_animator = ecore_animator_add(_smart_hold_animator, sd);
+                         sd->down.hold_animator = 
+                         ecore_animator_add(_smart_hold_animator, sd);
+//                       printf("a %i %i\n", sd->down.hold_x, sd->down.hold_y);
+//                       _smart_onhold_animator(sd);
 //                       elm_smart_scroller_child_pos_set(sd->smart_obj, x, y);
                     }
+                  else
+                    {
+                       if (sd->down.dragged_began)
+                         {
+                            ev->event_flags |= EVAS_EVENT_FLAG_ON_HOLD;
+                            if (!sd->down.hold)
+                              {
+                                 sd->down.hold = 1;
+                                 evas_event_feed_hold(e, 1, ev->timestamp, ev->data);
+                              }
+                         }
+                    }
 	       }
              else if (!sd->freeze)
                {
@@ -1610,6 +1898,7 @@ _smart_event_mouse_move(void *data, Evas *e, Evas_Object *obj, void *event_info)
                             sd->down.onhold_tlast = 0.0;
                             sd->down.onhold_animator = ecore_animator_add(_smart_onhold_animator, sd);
                          }
+//                       printf("b %i %i\n", sd->down.hold_x, sd->down.hold_y);
                     }
                   else
                     {
@@ -1625,7 +1914,7 @@ _smart_event_mouse_move(void *data, Evas *e, Evas_Object *obj, void *event_info)
 }
 
 static void
-_smart_event_key_down(void *data, Evas *e, Evas_Object *obj, void *event_info)
+_smart_event_key_down(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event_info)
 {
    Evas_Event_Key_Down *ev;
    Smart_Data *sd;
@@ -2016,7 +2305,8 @@ _smart_add(Evas_Object *obj)
 
    o = edje_object_add(evas_object_evas_get(obj));
    sd->edje_obj = o;
-   _elm_theme_set(o, "scroller", "base", "default");
+   // FIXME: null parent obj ... :(
+   _elm_theme_object_set(NULL, o, "scroller", "base", "default");
    edje_object_signal_callback_add(o, "drag", "elm.dragable.vbar", _smart_edje_drag_v, sd);
    edje_object_signal_callback_add(o, "drag,start", "elm.dragable.vbar", _smart_edje_drag_v_start, sd);
    edje_object_signal_callback_add(o, "drag,stop", "elm.dragable.vbar", _smart_edje_drag_v_stop, sd);
@@ -2146,7 +2436,10 @@ _smart_init(void)
 	       NULL,
 	       NULL,
 	       NULL,
-	       NULL
+	       NULL,
+               NULL,
+               NULL,
+               NULL
 	  };
 	_smart = evas_smart_class_new(&sc);
      }
