@@ -1,8 +1,8 @@
 typedef enum _Elm_Smart_Scroller_Policy
 {
-   ELM_SMART_SCROLLER_POLICY_OFF,
-     ELM_SMART_SCROLLER_POLICY_ON,
-     ELM_SMART_SCROLLER_POLICY_AUTO
+   ELM_SMART_SCROLLER_POLICY_AUTO,
+   ELM_SMART_SCROLLER_POLICY_ON,
+   ELM_SMART_SCROLLER_POLICY_OFF
 }
 Elm_Smart_Scroller_Policy;
 
@@ -23,10 +23,11 @@ void elm_smart_scroller_policy_get              (Evas_Object *obj, Elm_Smart_Scr
 Evas_Object *elm_smart_scroller_edje_object_get (Evas_Object *obj);
 void elm_smart_scroller_single_dir_set          (Evas_Object *obj, Eina_Bool single_dir);
 Eina_Bool elm_smart_scroller_single_dir_get     (Evas_Object *obj);
-void elm_smart_scroller_theme_set               (Evas_Object *obj, const char *clas, const char *group, const char *style);
+void elm_smart_scroller_object_theme_set        (Evas_Object *parent, Evas_Object *obj, const char *clas, const char *group, const char *style);
 void elm_smart_scroller_hold_set                (Evas_Object *obj, Eina_Bool hold);
 void elm_smart_scroller_freeze_set              (Evas_Object *obj, Eina_Bool freeze);
 void elm_smart_scroller_bounce_allow_set        (Evas_Object *obj, Eina_Bool horiz, Eina_Bool vert);
+void elm_smart_scroller_bounce_allow_get        (const Evas_Object *obj, Eina_Bool *horiz, Eina_Bool *vert);
 void elm_smart_scroller_paging_set              (Evas_Object *obj, double pagerel_h, double pagerel_v, Evas_Coord pagesize_h, Evas_Coord pagesize_v);
 void elm_smart_scroller_region_bring_in         (Evas_Object *obj, Evas_Coord x, Evas_Coord y, Evas_Coord w, Evas_Coord h);
-  
+void elm_smart_scroller_widget_set              (Evas_Object *obj, Evas_Object *wid);
